#ifndef CS_DEFINES_H_
#define CS_DEFINES_H_

/*-- pre-processor statements for conditional code --*/

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+

    System: UCL Cryosat IO Library

    Sub-system: Cryosat Specific

    Description: Defines


    ----------------------------------------------
    Documentation cross reference:


    ----------------------------------------------
    This file lives in ---
    ./mssl_cryosat

    Coding by:
    David J Brockley
    http://cryosat.mssl.ucl.ac.uk/public

    ----------------------------------------------
    Filename              :     $RCSfile: CS_Defines.h,v $
    CVS Version Number    :     $Revision: 1.4 $
    Last Update           :     $Date: 2015/06/02 11:07:43 $
    What String           :     @(#)$Id: CS_Defines.h,v 1.4 2015/06/02 11:07:43 djb Exp $
    Log                   :
                                $Log: CS_Defines.h,v $
                                Revision 1.4  2015/06/02 11:07:43  djb
                                Preparing for release 2.2

                                Revision 1.3  2014/12/01 11:30:14  djb
                                Pre-testing for release v2.1

                                Revision 1.2  2010/08/27 09:22:59  djb
                                Prototype changes to fix on 64 bit machines and to correct the size of the quality flag in L2.

                                Revision 1.1  2010/07/16 14:37:55  djb
                                Port in CryoSat tools from internal branch.



    ----------------------------------------------
    Routines:

    -----------------------------------------------------------
    This file contains the Unpublished Intellectual Property of
    University College London and All Rights Are Reserved.
        Copyright (c) University College London, 2005, 2010
    -----------------------------------------------------------
-
*/

#pragma ident "@(#)$Id: CS_Defines.h,v 1.4 2015/06/02 11:07:43 djb Exp $ MSSL"

/*-- pre-processor statements --*/

/*-- standard C headers --*/

/*-- programmer created headers --*/

/*-- constant and macro headers --*/

/*-- type definitions --*/

/*-- global and external variable declarations --*/

/*-- module private (static) variable declarations --*/

/*-- functions --*/

/**
 * @file   CS_Defines.h
 *
 * @brief  Contains types and macros needed by the functions in this library.
 *
 * This file also contains the top-level documentation to be extracted by Doxygen
 * to form the API reference.
 */

/**
 * @mainpage Library Overview
 *
 * @section Introduction Introduction
 *
 * This library contains C functions, and supporting header files, to
 * read CryoSat Level 2 formatted files from disk into memory. The
 * provided functions allow the user to create programs that read
 * whole input files into memory or pick out specific records to be
 * read.
 *
 * CryoSat files come in a number of different formats that correspond
 * to updates of the processing baseline (see https://earth.esa.int/web/guest/missions/cryosat/ipf-baseline).
 * The correct baseline must be specified for the code to correctly ingest the data.
 *
 * In addition to the C library, there is a library of IDL routines
 * to read both Level 1b and Level 2 products, from both Baseline-B and
 * Baseline-C. These may be found in
 * the <c>idl</c> subdirectory of the distribution archive. See the
 * \ref IDL subsection of the 'Module Documentation' section for
 * more information.
 *
 * The C library is split into six modules:
 *  - Low Level API : Requires the user handle files at the system
 * level and allows individual records to be read. There are two
 * modules at this level, one each from Level 1b and Level2.
 *  - High Level API : Hides all system interaction from the user and
 * allows either entire files or a continuous section of a file to be
 * read. There are three modules at this level, one each for Level 1b
 * and Level 2, and a module of routines shared by both.
 *  - Utilities : These functions are of use when processing the data.
 *
 * The APIs (Application Program Interfaces) for these modules are
 * documented in the section 'Module Documentation'.
 *
 * \latexonly
 * Examples of how to use the C library functions are given in \ref{ex2}
 * and \ref{ex1}. Bitfield manipulation functions are shown in \ref{ex3}.
 * \endlatexonly
 *
 * @subsection Compatability Platform compatability and requirements
 *
 * The library is distributed as a TAR file that has been compressed
 * with the GZIP utility. You will need these utilities to unpack it.
 *
 * The library has been built and successfully tested with the
 * following combinations of platform, operating system, and compiler:
 *  - CentOS 5 Linux with GCC 4.1.2
 *  - Mac OSX 10.10 with Apple LLVM version 6.1.0
 *
 * @subsection Installation Installation Instructions
 *
 * After unpacking the library, simply type <tt>make</tt> in the c_code
 * subdirectory to build the libraries and examples. You should then copy
 * the library and header files to the appropriate place on your system.
 * A common example is to place the header files in <tt>/usr/local/include</tt>
 * and the library files in <tt>/usr/local/lib</tt>.
 *
 * @subsection Usage Library Usage
 *
 * To use the library, you must tell the C compiler where to find the
 * include files and library files. If the library was installed at
 * the default location, <tt>/usr/local</tt>, then the flags to pass to gcc are
 * <tt>-I/usr/local/include</tt> during
 * compilation and <tt>-L/usr/local/lib -lmssl_cryosat -lmssl_shared</tt>
 * during linking.
 *
 * @subsection Changes Changes
 *
 * Version 1.1 of the library has changes to allow use in 64-bit mode
 * on 64-bit machines. Also, the variable uj_Quality_F in the L2 product
 * structure is now (correctly) defined as a 32-bit (not a 16-bit) integer.
 *
 * Version 2.0 of the library handles the different baselines of the product
 * format up to Baseline C.
 *
 * Version 2.1 aligns the IDL parameter names for the new products with
 * previous releases and resolves a bug that prevented reading of L1b products.
 *
 * Version 2.2 fixes an issue that resulted in the library being unable to handle
 * Baseline-C L2 GDR files (only). This release also corrects some of the bit
 * definitions for flag words to align with Baseline C.
 *
 * Version 2.3 fixes an issue preventing the IDL code from reading Baseline C
 * GDR files.
 */

/**
 * @defgroup CS_SHARED_IO_API_HIGH High-level shared routines
 *
 * Use of these functions is necessary when calling the high-level
 * APIs to read either Level 1b or Level 2 data.
 */

/**
 * @defgroup CS_L1B_IO_API_HIGH High Level CryoSat Level 1b File IO API
 *
 * These functions comprise the high-level API for reading from CryoSat Level 1b
 * format files in LRM, SAR, and SARin modes.
 *
 * These functions correctly handle both complete files (with MPH and
 * SPH) and interim files (raw data).
 */

/**
 * @defgroup CS_L1B_IO_API_LOW Low-Level CryoSat Level 1b File IO API
 *
 * These functions comprise the low-level API for reading from CryoSat
 * Level 1b format files in LRM, SAR, and SARin modes.
 *
 * Use of these functions requires the calling routine to do all of
 * the manipulation necessary to create a FILE* pointer that is
 * positioned to the start of a Level 1b record on the disk. The
 * functions simply read that record into an area of memory that must
 * be allocated by the calling routine.
 */

/**
 * @defgroup CS_L2_IO_API_HIGH High Level CryoSat Level 2 File IO API
 *
 * These functions comprise the high-level API for reading from CryoSat Level 2
 * format files.
 *
 * These functions correctly handle both complete files (with MPH and
 * SPH) and interim files (raw data).
 */

/**
 * @defgroup CS_L2_IO_API_LOW Low-Level CryoSat Level 2 File IO API
 *
 * These functions comprise the low-level API for reading from CryoSat
 * Level 2 format files.
 *
 * Use of these functions requires the calling routine to do all of
 * the manipulation necessary to create a FILE* pointer that is
 * positioned to the start of a Level 2 record on the disk. The
 * functions simply read that record into an area of memory that must
 * be allocated by the calling routine.
 */

/**
 * @defgroup UTILITIES Utility functions
 *
 * These are useful functions to aid in the processing of data.
 *
 * These functions are not CryoSat specific. They are utility functions
 * that may be of use when processing data read by the main functions of
 * this library.
 */

/**
 * Representation of DS_NAME types read by the CryoSat Level 2 IO
 * Library.
 */
typedef enum
{
    /* L2 Products */
    CS_L2_DS_NAME_LRM,
    CS_L2_DS_NAME_FDM,
    CS_L2_DS_NAME_SAR_A,
    CS_L2_DS_NAME_SAR_B,
    CS_L2_DS_NAME_SAR,
    CS_L2_DS_NAME_SIN,
    CS_L2_DS_NAME_SID,
    CS_L2_DS_NAME_GDR,
    CS_L2_DS_NAME_GDR_A,
    CS_L2_DS_NAME_GDR_B,

    /* L2I Products */
    CS_L2I_DS_NAME_LRM,
    CS_L2I_DS_NAME_SAR_A,
    CS_L2I_DS_NAME_SAR_B,
    CS_L2I_DS_NAME_SAR,
    CS_L2I_DS_NAME_SIN,
    CS_L2I_DS_NAME_SID,
    CS_L2I_DS_NAME_GDR,
    CS_L2I_DS_NAME_GDR_A,
    CS_L2I_DS_NAME_GDR_B,

    /* L1b Products */
    CS_L1B_DS_NAME_LRM,
    CS_L1B_DS_NAME_FDM,
    CS_L1B_DS_NAME_SAR,
    CS_L1B_DS_NAME_SARIN,

    /* Dummy entry to give number of strings */
    CS_DS_NAME_COUNT,

    /* Utilities */
    CS_L2_DS_NAME_START  = ( CS_L2_DS_NAME_LRM ),
    CS_L2_DS_NAME_END    = ( CS_L2_DS_NAME_GDR_B ),
    CS_L2I_DS_NAME_START = ( CS_L2I_DS_NAME_LRM ),
    CS_L2I_DS_NAME_END   = ( CS_L2I_DS_NAME_GDR_B ),
    CS_L1B_DS_NAME_START = ( CS_L1B_DS_NAME_LRM ),
    CS_L1B_DS_NAME_END   = ( CS_L1B_DS_NAME_SARIN ),
    CS_L2_DS_NAME_ANY,
    CS_L2I_DS_NAME_ANY,
    CS_L1B_DS_NAME_ANY,
    CS_DS_NAME_ANY,
    CS_DS_NAME_NONE
}
CS_DS_NAME;

/** External declaration. See CS_Defines.c for definition. */
extern char* as_CS_DS_NAME[ CS_DS_NAME_COUNT ];

/** Size of the MPH in bytes */
#define J_MPH_SIZE 1247

/** Size of the CryoSat Level 2 record on disk, in bytes. */
extern int J_CS_L2_RECORD_SIZE;

/** Size of the CryoSat Level 2I record on disk, in bytes. */
extern int J_CS_L2I_RECORD_SIZE;

#ifndef TRUE
#define TRUE 1 /**< TRUE */
#define FALSE 0 /**< FALSE */
#endif

extern int gj_verbose;

#endif /* ifndef CS_DEFINES_H_ */
