#include "L1bStructure.h"

/**
 * @file   L1bStructure.c
 *
 * @brief  Definition of variables and convenience functions related to the Level 1b structure.
 *
 *
 */

/** Number of bins in the range window for LRM mode. */
int J_LRM_WFM_BINS = 128;

/** Number of bins in the range window for SAR mode. */
int J_SAR_WFM_BINS = 128;

/** Number of bins in the range window for SARin mode. */
int J_SARIN_WFM_BINS = 512;

/** Size in bytes of the Level 1b LRM mode record on disk. */
int J_CS_L1B_LRM_RECORD_SIZE = 9084;

/** Size in bytes of the Level 1b SAR mode record on disk. */
int J_CS_L1B_SAR_RECORD_SIZE = 11084;

/** Size in bytes of the Level 1b SARin mode record on disk. */
int J_CS_L1B_SARIN_RECORD_SIZE = 88652;

static void vDump_TimeAndOrbitGroup( TimeAndOrbitGroup* pt, FILE* pf );
static void vDump_Measurements( Measurements* pt, FILE* pf );
static void vDump_Corrections( Corrections* pt, FILE* pf );
static void vDump_AveragedWaveformsLRM( AveragedWaveformsLRM* pt, FILE* pf );
static void vDump_WaveformsLRM( WaveformsLRM* pt, FILE* pf );
static void vDump_AveragedWaveformsSAR( AveragedWaveformsSAR* pt, FILE* pf );
static void vDump_WaveformsSAR( WaveformsSAR* pt, FILE* pf );
static void vDump_AveragedWaveformsSARIN( AveragedWaveformsSARIN* pt, FILE* pf );
static void vDump_WaveformsSARIN( WaveformsSARIN* pt, FILE* pf );

void vDump_L1B_LRM( L1B_LRM* pt, FILE* pf )
{
    int i;

    for( i = 0; i < J_NUM_HRB; i++ )
    {
        vDump_TimeAndOrbitGroup( &pt->at_Time_Orbit[i], pf );
    }

    for( i = 0; i < J_NUM_HRB; i++ )
    {
        vDump_Measurements( &pt->at_Measurements[i], pf );
    }

    vDump_Corrections( &pt->t_Geo_Corrections, pf );

    vDump_AveragedWaveformsLRM( &pt->t_Average_PL, pf );

    for( i = 0; i < J_NUM_HRB; i++ )
    {
        vDump_WaveformsLRM( &pt->at_Waveforms[i], pf );
    }
}

void vDump_L1B_SAR( L1B_SAR* pt, FILE* pf )
{
    int i;

    for( i = 0; i < J_NUM_HRB; i++ )
    {
        vDump_TimeAndOrbitGroup( &pt->at_Time_Orbit[i], pf );
    }

    for( i = 0; i < J_NUM_HRB; i++ )
    {
        vDump_Measurements( &pt->at_Measurements[i], pf );
    }

    vDump_Corrections( &pt->t_Geo_Corrections, pf );

    vDump_AveragedWaveformsSAR( &pt->t_Average_PL, pf );

    for( i = 0; i < J_NUM_HRB; i++ )
    {
        vDump_WaveformsSAR( &pt->at_Waveforms[i], pf );
    }
}

void vDump_L1B_SARIN( L1B_SARIN* pt, FILE* pf )
{
    int i;

    for( i = 0; i < J_NUM_HRB; i++ )
    {
        vDump_TimeAndOrbitGroup( &pt->at_Time_Orbit[i], pf );
    }

    for( i = 0; i < J_NUM_HRB; i++ )
    {
        vDump_Measurements( &pt->at_Measurements[i], pf );
    }

    vDump_Corrections( &pt->t_Geo_Corrections, pf );

    vDump_AveragedWaveformsSARIN( &pt->t_Average_PL, pf );

    for( i = 0; i < J_NUM_HRB; i++ )
    {
        vDump_WaveformsSARIN( &pt->at_Waveforms[i], pf );
    }
}

void vDump_TimeAndOrbitGroup( TimeAndOrbitGroup* pt, FILE* pf )
{
    if( pf == NULL ) pf = stdout;

    fprintf( pf, "%s = %"PRId32"\n", "j_Day", pt->j_Day );
    fprintf( pf, "%s = %"PRIu32"\n", "uj_Sec", pt->uj_Sec );
    fprintf( pf, "%s = %"PRIu32"\n", "uj_Micsec", pt->uj_Micsec );
    fprintf( pf, "%s = %"PRId32"\n", "j_USO_Corr", pt->j_USO_Corr );;
    fprintf( pf, "%s = %"PRIu16"\n", "ui_Mode_ID", pt->ui_Mode_ID );
    fprintf( pf, "%s = %"PRIu16"\n", "ui_SSC", pt->ui_SSC );
    fprintf( pf, "%s = %"PRIu32"\n", "uj_Inst_config", pt->uj_Inst_config );
    fprintf( pf, "%s = %"PRIu32"\n", "uj_Rec_Count", pt->uj_Rec_Count );
    fprintf( pf, "%s = %"PRId32"\n", "j_Lat", pt->j_Lat );
    fprintf( pf, "%s = %"PRId32"\n", "j_Lon", pt->j_Lon );
    fprintf( pf, "%s = %"PRId32"\n", "j_Alt", pt->j_Alt );
    fprintf( pf, "%s = %"PRId32"\n", "j_Alt_rate", pt->j_Alt_rate );
    fprintf( pf, "%s = %"PRId32" %"PRId32" %"PRId32"\n", "aj_Sat_velocity", pt->aj_Sat_velocity[0], pt->aj_Sat_velocity[1], pt->aj_Sat_velocity[2] );
    fprintf( pf, "%s = %"PRId32" %"PRId32" %"PRId32"\n", "aj_Real_beam", pt->aj_Real_beam[0], pt->aj_Real_beam[1], pt->aj_Real_beam[2] );
    fprintf( pf, "%s = %"PRId32" %"PRId32" %"PRId32"\n", "aj_Baseline", pt->aj_Baseline[0], pt->aj_Baseline[1], pt->aj_Baseline[2] );
    fprintf( pf, "%s = %"PRIu16"\n", "i_ST_ID", pt->i_ST_ID );
    fprintf( pf, "%s = %"PRId32"\n", "j_Roll_angle", pt->j_Roll_angle );
    fprintf( pf, "%s = %"PRId32"\n", "j_Pitch_angle", pt->j_Pitch_angle );
    fprintf( pf, "%s = %"PRId32"\n", "j_Yaw_angle", pt->j_Yaw_angle );
    fprintf( pf, "%s = %"PRIu32"\n", "uj_MCD", pt->uj_MCD );
}

void vDump_Measurements( Measurements* pt, FILE* pf )
{
    if( pf == NULL ) pf = stdout;

    fprintf( pf, "%s = %"PRId64"\n", "k_TD", pt->k_TD );
    fprintf( pf, "%s = %"PRId32"\n", "j_H0", pt->j_H0 );
    fprintf( pf, "%s = %"PRId32"\n", "j_COR2", pt->j_COR2 );
    fprintf( pf, "%s = %"PRId32"\n", "j_LAI", pt->j_LAI );
    fprintf( pf, "%s = %"PRId32"\n", "j_FAI", pt->j_FAI );
    fprintf( pf, "%s = %"PRId32"\n", "j_AGC_ch1", pt->j_AGC_ch1 );
    fprintf( pf, "%s = %"PRId32"\n", "j_AGC_ch2", pt->j_AGC_ch2 );
    fprintf( pf, "%s = %"PRId32"\n", "j_TR_gain_ch1", pt->j_TR_gain_ch1 );
    fprintf( pf, "%s = %"PRId32"\n", "j_TR_gain_ch2", pt->j_TR_gain_ch2 );
    fprintf( pf, "%s = %"PRId32"\n", "j_Tx_power", pt->j_Tx_power );
    fprintf( pf, "%s = %"PRId32"\n", "j_Dopp_rc", pt->j_Dopp_rc );
    fprintf( pf, "%s = %"PRId32"\n", "j_TR_inst_rc", pt->j_TR_inst_rc );
    fprintf( pf, "%s = %"PRId32"\n", "j_R_inst_rc", pt->j_R_inst_rc );
    fprintf( pf, "%s = %"PRId32"\n", "j_TR_inst_gain_C", pt->j_TR_inst_gain_C );
    fprintf( pf, "%s = %"PRId32"\n", "j_R_inst_gain_C", pt->j_R_inst_gain_C );
    fprintf( pf, "%s = %"PRId32"\n", "j_Int_phase_C", pt->j_Int_phase_C );
    fprintf( pf, "%s = %"PRId32"\n", "j_Ext_phase_C", pt->j_Ext_phase_C );
    fprintf( pf, "%s = %"PRId32"\n", "j_Noise_pwr", pt->j_Noise_pwr );
    fprintf( pf, "%s = %"PRId32"\n", "j_Phase_slope_C", pt->j_Phase_slope_C );
}

void vDump_Corrections( Corrections* pt, FILE* pf )
{
    if( pf == NULL ) pf = stdout;

    fprintf( pf, "%s = %"PRId32"\n", "j_Dry_C", pt->j_Dry_C );
    fprintf( pf, "%s = %"PRId32"\n", "j_Wet_C", pt->j_Wet_C );
    fprintf( pf, "%s = %"PRId32"\n", "j_IB_C", pt->j_IB_C );
    fprintf( pf, "%s = %"PRId32"\n", "j_DAC_C", pt->j_DAC_C );
    fprintf( pf, "%s = %"PRId32"\n", "j_Iono_GIM", pt->j_Iono_GIM );
    fprintf( pf, "%s = %"PRId32"\n", "j_Iono_mod", pt->j_Iono_mod );
    fprintf( pf, "%s = %"PRId32"\n", "j_H_OT", pt->j_H_OT );
    fprintf( pf, "%s = %"PRId32"\n", "j_H_LPEOT", pt->j_H_LPEOT );
    fprintf( pf, "%s = %"PRId32"\n", "j_H_OLT", pt->j_H_OLT );
    fprintf( pf, "%s = %"PRId32"\n", "j_H_SET", pt->j_H_SET );
    fprintf( pf, "%s = %"PRId32"\n", "j_H_GPT", pt->j_H_GPT );
    fprintf( pf, "%s = %"PRIu32"\n", "uj_Surf_type", pt->uj_Surf_type );
    fprintf( pf, "%s = %"PRIu32"\n", "uj_Corr_status", pt->uj_Corr_status );
    fprintf( pf, "%s = %"PRIu32"\n", "uj_Corr_error", pt->uj_Corr_error );
}

void vDump_AveragedWaveformsLRM( AveragedWaveformsLRM* pt, FILE* pf )
{
    int i;
    if( pf == NULL ) pf = stdout;

    fprintf( pf, "%s = %"PRId32"\n", "j_Day_1Hz", pt->j_Day_1Hz );
    fprintf( pf, "%s = %"PRIu32"\n", "uj_Sec_1Hz", pt->uj_Sec_1Hz );
    fprintf( pf, "%s = %"PRIu32"\n", "uj_Micsec_1Hz", pt->uj_Micsec_1Hz );
    fprintf( pf, "%s = %"PRId32"\n", "j_Lat_1Hz", pt->j_Lat_1Hz );
    fprintf( pf, "%s = %"PRId32"\n", "j_Lon_1Hz", pt->j_Lon_1Hz );
    fprintf( pf, "%s = %"PRId32"\n", "j_Alt_1Hz", pt->j_Alt_1Hz );
    fprintf( pf, "%s = %"PRId64"\n", "k_TD_1Hz", pt->k_TD_1Hz );
    fprintf( pf, "%s = ", "pui_avg_wfm" );
    for( i = 0; i < J_LRM_WFM_BINS; i++ ) fprintf( pf, "%"PRIu16"%c", pt->pui_avg_wfm[i],(i==(J_LRM_WFM_BINS-1))?'\n':' ' );
    fprintf( pf, "%s = %"PRId32"\n", "j_Linear_wfm_mult", pt->j_Linear_wfm_mult );
    fprintf( pf, "%s = %"PRId32"\n", "j_Power2_wfm_mult", pt->j_Power2_wfm_mult );
    fprintf( pf, "%s = %"PRIu16"\n", "ui_num_avg_echoes", pt->ui_num_avg_echoes );
    fprintf( pf, "%s = %"PRIu16"\n", "ui_flags", pt->ui_flags );
}

void vDump_WaveformsLRM( WaveformsLRM* pt, FILE* pf )
{
    int i;
    if( pf == NULL ) pf = stdout;
    fprintf( pf, "%s = ", "pui_wfm" );
    for( i = 0; i < J_LRM_WFM_BINS; i++ ) fprintf( pf, "%"PRIu16"%c", pt->pui_wfm[i],(i==(J_LRM_WFM_BINS-1))?'\n':' ' );
    fprintf( pf, "%s = %"PRId32"\n", "j_Linear_wfm_mult", pt->j_Linear_wfm_mult );
    fprintf( pf, "%s = %"PRId32"\n", "j_Power2_wfm_mult", pt->j_Power2_wfm_mult );
    fprintf( pf, "%s = %"PRIu16"\n", "ui_num_avg_echoes", pt->ui_num_avg_echoes );
    fprintf( pf, "%s = %"PRIu16"\n", "ui_flags", pt->ui_flags );
}

void vDump_AveragedWaveformsSAR( AveragedWaveformsSAR* pt, FILE* pf )
{
    int i;
    if( pf == NULL ) pf = stdout;

    fprintf( pf, "%s = %"PRId32"\n", "j_Day_1Hz", pt->j_Day_1Hz );
    fprintf( pf, "%s = %"PRIu32"\n", "uj_Sec_1Hz", pt->uj_Sec_1Hz );
    fprintf( pf, "%s = %"PRIu32"\n", "uj_Micsec_1Hz", pt->uj_Micsec_1Hz );
    fprintf( pf, "%s = %"PRId32"\n", "j_Lat_1Hz", pt->j_Lat_1Hz );
    fprintf( pf, "%s = %"PRId32"\n", "j_Lon_1Hz", pt->j_Lon_1Hz );
    fprintf( pf, "%s = %"PRId32"\n", "j_Alt_1Hz", pt->j_Alt_1Hz );
    fprintf( pf, "%s = %"PRId64"\n", "k_TD_1Hz", pt->k_TD_1Hz );
    fprintf( pf, "%s = ", "pui_avg_wfm" );
    for( i = 0; i < J_SAR_WFM_BINS; i++ ) fprintf( pf, "%"PRIu16"%c", pt->pui_avg_wfm[i],(i==(J_SAR_WFM_BINS-1))?'\n':' ' );
    fprintf( pf, "%s = %"PRId32"\n", "j_Linear_wfm_mult", pt->j_Linear_wfm_mult );
    fprintf( pf, "%s = %"PRId32"\n", "j_Power2_wfm_mult", pt->j_Power2_wfm_mult );
    fprintf( pf, "%s = %"PRIu16"\n", "ui_num_avg_echoes", pt->ui_num_avg_echoes );
    fprintf( pf, "%s = %"PRIu16"\n", "ui_flags", pt->ui_flags );
}

void vDump_WaveformsSAR( WaveformsSAR* pt, FILE* pf )
{
    int i;
    if( pf == NULL ) pf = stdout;
    fprintf( pf, "%s = ", "pui_wfm" );
    for( i = 0; i < J_SAR_WFM_BINS; i++ ) fprintf( pf, "%"PRIu16"%c", pt->pui_wfm[i],(i==(J_SAR_WFM_BINS-1))?'\n':' ' );
    fprintf( pf, "%s = %"PRId32"\n", "j_Linear_wfm_mult", pt->j_Linear_wfm_mult );
    fprintf( pf, "%s = %"PRId32"\n", "j_Power2_wfm_mult", pt->j_Power2_wfm_mult );
    fprintf( pf, "%s = %"PRIu16"\n", "ui_num_avg_echoes", pt->ui_num_avg_echoes );
    fprintf( pf, "%s = %"PRIu16"\n", "ui_flags", pt->ui_flags );
    fprintf( pf, "%s = ", "ai_bb_param" );
    for( i = 0; i < 50; i++ ) fprintf( pf, "%"PRId16"%c", pt->ai_bb_param[i],(i==49)?'\n':' ' );
}

void vDump_AveragedWaveformsSARIN( AveragedWaveformsSARIN* pt, FILE* pf )
{
    int i;
    if( pf == NULL ) pf = stdout;

    fprintf( pf, "%s = %"PRId32"\n", "j_Day_1Hz", pt->j_Day_1Hz );
    fprintf( pf, "%s = %"PRIu32"\n", "uj_Sec_1Hz", pt->uj_Sec_1Hz );
    fprintf( pf, "%s = %"PRIu32"\n", "uj_Micsec_1Hz", pt->uj_Micsec_1Hz );
    fprintf( pf, "%s = %"PRId32"\n", "j_Lat_1Hz", pt->j_Lat_1Hz );
    fprintf( pf, "%s = %"PRId32"\n", "j_Lon_1Hz", pt->j_Lon_1Hz );
    fprintf( pf, "%s = %"PRId32"\n", "j_Alt_1Hz", pt->j_Alt_1Hz );
    fprintf( pf, "%s = %"PRId64"\n", "k_TD_1Hz", pt->k_TD_1Hz );
    fprintf( pf, "%s = ", "pui_avg_wfm" );
    for( i = 0; i < J_SARIN_WFM_BINS; i++ ) fprintf( pf, "%"PRIu16"%c", pt->pui_avg_wfm[i],(i==(J_SARIN_WFM_BINS-1))?'\n':' ' );
    fprintf( pf, "%s = %"PRId32"\n", "j_Linear_wfm_mult", pt->j_Linear_wfm_mult );
    fprintf( pf, "%s = %"PRId32"\n", "j_Power2_wfm_mult", pt->j_Power2_wfm_mult );
    fprintf( pf, "%s = %"PRIu16"\n", "ui_num_avg_echoes", pt->ui_num_avg_echoes );
    fprintf( pf, "%s = %"PRIu16"\n", "ui_flags", pt->ui_flags );
}

void vDump_WaveformsSARIN( WaveformsSARIN* pt, FILE* pf )
{
    int i;
    if( pf == NULL ) pf = stdout;
    fprintf( pf, "%s = ", "pui_wfm" );
    for( i = 0; i < J_SARIN_WFM_BINS; i++ ) fprintf( pf, "%"PRIu16"%c", pt->pui_wfm[i],(i==(J_SARIN_WFM_BINS-1))?'\n':' ' );
    fprintf( pf, "%s = %"PRId32"\n", "j_Linear_wfm_mult", pt->j_Linear_wfm_mult );
    fprintf( pf, "%s = %"PRId32"\n", "j_Power2_wfm_mult", pt->j_Power2_wfm_mult );
    fprintf( pf, "%s = %"PRIu16"\n", "ui_num_avg_echoes", pt->ui_num_avg_echoes );
    fprintf( pf, "%s = %"PRIu16"\n", "ui_flags", pt->ui_flags );
    fprintf( pf, "%s = ", "ai_bb_param" );
    for( i = 0; i < 50; i++ ) fprintf( pf, "%"PRId16"%c", pt->ai_bb_param[i],(i==49)?'\n':' ' );
    fprintf( pf, "%s = ", "pui_coherence" );
    for( i = 0; i < J_SARIN_WFM_BINS; i++ ) fprintf( pf, "%"PRIu16"%c", pt->pui_coherence[i],(i==(J_SARIN_WFM_BINS-1))?'\n':' ' );
    fprintf( pf, "%s = ", "pj_phase_diff" );
    for( i = 0; i < J_SARIN_WFM_BINS; i++ ) fprintf( pf, "%"PRId32"%c", pt->pj_phase_diff[i],(i==(J_SARIN_WFM_BINS-1))?'\n':' ' );
    uint16_t* pui_coherence;
    int32_t* pj_phase_diff;
}
