#ifndef L1B_FORMAT_H
#define L1B_FORMAT_H

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+

    System: UCL Cryosat IO Library

    Sub-system: Cryosat Specific

    Description:
    Include File.

    ----------------------------------------------
    Documentation cross reference:
    [PROD-FMT] Product Format Specification v4.5 20/04/2009 CS-RS-ACS-GS-5106

    ----------------------------------------------
    This file lives in ---
    ./mssl_cryosat

    Coding by:
    David J Brockley
    http://cryosat.mssl.ucl.ac.uk/public

    ----------------------------------------------
    Filename              :     $RCSfile: L1bStructure.h,v $
    CVS Version Number    :     $Revision: 1.6 $
    Last Update           :     $Date: 2015/06/02 11:07:43 $
    What String           :     @(#)$Id: L1bStructure.h,v 1.6 2015/06/02 11:07:43 djb Exp $
    Log                   :
                                $Log: L1bStructure.h,v $
                                Revision 1.6  2015/06/02 11:07:43  djb
                                Preparing for release 2.2

                                Revision 1.5  2014/12/01 11:30:14  djb
                                Pre-testing for release v2.1

                                Revision 1.4  2010/11/11 13:57:47  djb
                                Minor changes to types and naming to adhere to standards.

                                Revision 1.3  2010/10/13 08:12:50  djb
                                PRE_RELEASE_1_1

                                Revision 1.2  2010/08/27 09:22:59  djb
                                Prototype changes to fix on 64 bit machines and to correct the size of the quality flag in L2.

                                Revision 1.1  2010/07/16 14:37:55  djb
                                Port in CryoSat tools from internal branch.


    ---------------------------------------------------------------------------
    This file is the Unpublished Intellectual
    Property of University College London and
    All Rights are Reserved.

    Copyright (c) University College London, 2005, 2010
    ---------------------------------------------------------------------------
*/

#include <stdint.h>
#include <inttypes.h>
#include <stdio.h>

/* Included only to get documentation for Doxygen */
#include "SharedFlags.h"

/**
 * @file   L1bStructure.h
 *
 * @brief  Definition of the Level 1b structure.
 *
 *
 */

/** Number of bins in the range window for LRM mode. */
extern int J_LRM_WFM_BINS;

/** Number of bins in the range window for SAR mode. */
extern int J_SAR_WFM_BINS;

/** Number of bins in the range window for SARin mode. */
extern int J_SARIN_WFM_BINS;

/** Number of beam behaviour parameters. */
#define J_NUM_BB_PARAMETERS 50

/** Number of high-rate blocks contained within each record. */
#define J_NUM_HRB 20

/** Size in bytes of the Level 1b LRM mode record on disk. */
extern int J_CS_L1B_LRM_RECORD_SIZE;

/** Size in bytes of the Level 1b SAR mode record on disk. */
extern int J_CS_L1B_SAR_RECORD_SIZE;

/** Size in bytes of the Level 1b SARin mode record on disk. */
extern int J_CS_L1B_SARIN_RECORD_SIZE;

/** Level 1b record substructure containing time and orbit parameters. */
typedef struct _TimeAndOrbitGroup
{
    int32_t j_Day;                     /**< MDSR Time stamp days. */
    uint32_t uj_Sec;                   /**< MDSR Time stamp seconds. */
    uint32_t uj_Micsec;                /**< MDSR Time stamp microseconds. */
    int32_t j_USO_Corr;                /**< USO Correction Factor. */
    uint16_t ui_Mode_ID;               /**< Mode ID. See ::t_ModeID */
    uint16_t ui_SSC;                   /**< Source sequence counter. */
    uint32_t uj_Inst_config;           /**< Instrument Configuration. See ::t_InstConf */
    uint32_t uj_Rec_Count;             /**< Record Counter. */
    int32_t  j_Lat;                    /**< Latitude at nadir.
                                         *
                                         *   Units: 10<SUP>-7</SUP> degrees
                                         */
    int32_t j_Lon;                     /**< Longitude at nadir.
                                         *
                                         *   Units: 10<SUP>-7</SUP> degrees
                                         */
    int32_t j_Alt;                     /**< Altitude of COG above
                                         *   reference ellipsoid.
                                         *
                                         *   Units: mm
                                         */
    int32_t j_Alt_rate;                /**< Instantaneous altitude
                                         *   rate derived from orbit.
                                         *
                                         *   Units: mm/s
                                         */
    int32_t aj_Sat_velocity[ 3 ];      /**< Satellite velocity
                                         *   vector.
                                         *   In ITRF.
                                         *
                                         *   Units: mm/s
                                         */
    int32_t aj_Real_beam[ 3 ];         /**< Real beam direction
                                         *   vector.
                                         *   In CRF.
                                         *
                                         *   Units: micrometer
                                         */
    int32_t aj_Baseline[ 3 ];          /**< Interferometer baseline
                                         *   vector.
                                         *   In CRF.
                                         *
                                         *   Units: micrometer
                                         */
    uint16_t i_ST_ID;                  /**< Star Tracker ID. See ::t_STID
                                        *   Baseline BC and later only
                                        *
                                        *    Units: enum
                                        */
    int32_t j_Roll_angle;              /**< Spacecraft (antenna bench) roll angle.
                                        *   Baseline C only.
                                        *   Derived from star trackers.
                                        *
                                        *   Units: 10<SUP>-7</SUP> degrees
                                        */
    int32_t j_Pitch_angle;            /**< Spacecraft (antenna bench) pitch angle.
                                       *    Baseline C only.
                                        *   Derived from star trackers.
                                        *
                                        *   Units: 10<SUP>-7</SUP> degrees
                                        */
    int32_t j_Yaw_angle;              /**< Spacecraft (antenna bench) yaw angle.
                                       *    Baseline C only.
                                        *   Derived from star trackers.
                                        *
                                        *   Units: 10<SUP>-7</SUP> degrees
                                        */
    uint32_t uj_MCD;                   /**< L1b Measurement Confidence Data, see ::t_MCDFlagBit
                                         *   
                                         */
}
TimeAndOrbitGroup;

/** Level 1b record substructure containing measurement related parameters. */
typedef struct _Measurements
{
    int64_t  k_TD;                     /**< Window delay referenced to COM.
                                         *   Two-way and corrected for
                                         *   instrument delays but not
		                                 *   geophysical effects.
                                         *
                                         *   Units: 10<SUP>-12</SUP> seconds
                                         */
    int32_t j_H0;                      /**< H0 initial height word.
                                         *
                                         *   Units: 48.8 ps
                                         */
    int32_t j_COR2;                    /**< COR2 Height Rate
                                         *
                                         *   Units: 3.05 ps/rc
                                         */
    int32_t j_LAI;                     /**< Coarse Range Word LAI.
                                         *
                                         *   Units: 12.5 ns
                                         */
    int32_t j_FAI;                     /**< Fine Range Word FAI.
                                         *
                                         *   Units: 12.5/256 ns
                                         */
    int32_t j_AGC_ch1;                 /**< AGC channel 1.
                                         *   Gain calibration has been applied.
                                         *
                                         *   Units: dB/100
                                         */
    int32_t j_AGC_ch2;                 /**< AGC channel 2.
                                         *   Gain calibration has been applied.
                                         *
                                         *   Units: dB/100
                                         */
    int32_t j_TR_gain_ch1;             /**< Total fixed gain on channel 1.
                                         *
                                         *   Units: dB/100
                                         */
    int32_t j_TR_gain_ch2;             /**< Total fixed gain on channel 2.
                                         *
                                         *   Units: dB/100
                                         */
    int32_t j_Tx_power;                /**< Transmit power.
                                         *
                                         *   Units: micro-watts
                                         */
    int32_t j_Dopp_rc;                 /**< Doppler range correction.
                                         *   Radial component.
                                         *
                                         *   Units: mm
                                         */
    int32_t j_TR_inst_rc;              /**< Instrument range
                                         *   correction.
                                         *   Tx-Rx antenna. From CAL1.
                                         *
                                         *   Units: mm
                                         */
    int32_t j_R_inst_rc;               /**< Instrument range
                                         *   correction.
                                         *   Rx only antenna. From CAL1.
                                         *
                                         *   Units: mm
                                         */
    int32_t j_TR_inst_gain_C;          /**< Instrument gain
                                         *   correction.
                                         *   Tx-Rx antenna. From CAL1.
                                         *
                                         *   Units: dB/100
                                         */
    int32_t j_R_inst_gain_C;           /**< Instrument gain
                                         *   correction.
                                         *   Rx only antenna. From
                                         *   CAL1.
                                         *
                                         *   Units: dB/100
                                         */
    int32_t j_Int_phase_C;             /**< Internal phase
                                         *   correction from CAL4.
                                         *
                                         *   Units: microradians
                                         */
    int32_t j_Ext_phase_C;             /**< External phase
                                         *   correction.
		                                 *
                                         *   Units: microradians
                                         */
    int32_t j_Noise_pwr;               /**< Noise power measurement.
                                         *
                                         *   Units: dB/100
                                         */
    int32_t j_Phase_slope_C;           /**< Phase slope correction.
                                         *
                                         *   Units: Microradians
                                         */
    int8_t spares[ 4 ];                /**< Spares. */
}
Measurements;

/** Level 1b record substructure containing geophysical corrections. */
typedef struct _Corrections
{
    int32_t j_Dry_C;                   /**< Dry tropospheric
                                         *   correction.
                                         *
                                         *   Units: mm
                                         */
    int32_t j_Wet_C;                   /**< Wet tropospheric
                                         *   correction.
                                         *
                                         *   Units: mm
                                         */
    int32_t j_IB_C;                    /**< Inverse barometric
                                         *   correction.
                                         *
                                         *   Units: mm
                                         */
    int32_t j_DAC_C;                   /**< DAC correction.
                                         *
		                                 *  Includes inverse barometric.
		                                 *
                                         *   Units: mm
                                         */
    int32_t j_Iono_GIM;                /**< DORIS ionospheric
                                         *   correction.
										 *
                                         *   Units: mm
                                         */
    int32_t j_Iono_mod;                /**< Model ionospheric
                                         *   correction.
                                         *
                                         *   Units: mm
                                         */
    int32_t j_H_OT;                    /**< Ocean equilibrium tide.
                                         *
                                         *   Units: mm
                                         */
    int32_t j_H_LPEOT;                 /**< Long period equilibrium ocean tide.
                                         *
                                         *   Units: mm
                                         */
    int32_t j_H_OLT;                   /**< Ocean loading tide.
                                         *
                                         *   Units: mm
                                         */
    int32_t j_H_SET;                   /**< Solid Earth tide.
                                         *
                                         *   Units: mm
                                         */
    int32_t j_H_GPT;                   /**< Geocentric Polar tide.
                                         *
                                         *   Units: mm
                                         */
    uint32_t uj_Surf_type;             /**< Surface type flag. See ::t_SurfaceType */
    int8_t spare1[ 4 ];                /**< Spares. */
    uint32_t uj_Corr_status;           /**< Correction status flag. See ::t_CorrectionStatusFlagBit */
    uint32_t uj_Corr_error;            /**< Correction error flag. See ::t_CorrectionErrorFlagBit */
    int8_t spare2[ 4 ];                /**< Spares. */
}
Corrections;

/** Level 1b record substructure (LRM mode specific) containing averaged waveform data. */
typedef struct _AveragedWaveformsLRM
{
    int32_t j_Day_1Hz;                 /**< MDSR time stamp days. */
    uint32_t uj_Sec_1Hz;               /**< MDSR time stamp seconds. */
    uint32_t uj_Micsec_1Hz;            /**< MDSR time stamp microseconds. */
    int32_t j_Lat_1Hz;                 /**< Latitude.
                                         *
                                         *   Units: 10<SUP>-7</SUP> degrees.
                                         */
    int32_t j_Lon_1Hz;                 /**< Longitude.
                                         *
                                         *   Units: 10<SUP>-7</SUP> degrees.
                                         */
    int32_t j_Alt_1Hz;                 /**< Altitude of COG above
                                         *   reference ellipsoid.
                                         *   Interpolated value.
                                         *
                                         *   Units: mm
                                         */
    int64_t k_TD_1Hz;                  /**< Window delay referenced to COM.
                                         *   Two-way, corrected
                                         *   for instrument
                                         *   delays.
                                         *
                                         *   Units: mm
                                         */
    uint16_t* pui_avg_wfm;             /**< 1Hz Averaged power waveform. */
    int32_t j_Linear_wfm_mult;         /**< Echo scale factor. */
    int32_t j_Power2_wfm_mult;         /**< Echo scale factor. */
    uint16_t ui_num_avg_echoes;        /**< Number of echoes averaged. */
    uint16_t ui_flags;                 /**< Flags, see #t_Level1bAvgWfmLRMFlagBit */
}
AveragedWaveformsLRM;

/** Level 1b record substructure (SAR mode specific) containing averaged waveform data. */
typedef struct _AveragedWaveformsSAR
{
    int32_t j_Day_1Hz;                 /**< MDSR time stamp days. */
    uint32_t uj_Sec_1Hz;                /**< MDSR time stamp seconds. */
    uint32_t uj_Micsec_1Hz;             /**< MDSR time stamp microseconds. */
    int32_t j_Lat_1Hz;                 /**< Latitude.
                                         *
                                         *   Units: 10<SUP>-7</SUP> degrees.
                                         */
    int32_t j_Lon_1Hz;                 /**< Longitude.
                                         *
                                         *   Units: 10<SUP>-7</SUP> degrees.
                                         */
    int32_t j_Alt_1Hz;                 /**< Altitude of COG above
                                         *   reference ellipsoid.
                                         *   Interpolated value.
                                         *
                                         *   Units: mm
                                         */
    int64_t k_TD_1Hz;                  /**< Window delay referenced to COM.
                                         *   Two-way, corrected
                                         *   for instrument
                                         *   delays.
                                         *
                                         *   Units: mm
                                         */
    uint16_t* pui_avg_wfm;             /**< 1Hz Averaged power waveform. */
    int32_t j_Linear_wfm_mult;         /**< Echo scale factor. */
    int32_t j_Power2_wfm_mult;         /**< Echo scale factor. */
    uint16_t ui_num_avg_echoes;        /**< Number of echoes averaged. */
    uint16_t ui_flags;                 /**< Flags, see #t_Level1bAvgWfmSARFlagBit */
}
AveragedWaveformsSAR;

/** Level 1b record substructure (SARin mode specific) containing averaged waveform data. */
typedef struct _AveragedWaveformsSARIN
{
    int32_t j_Day_1Hz;                 /**< MDSR time stamp days. */
    uint32_t uj_Sec_1Hz;                /**< MDSR time stamp seconds. */
    uint32_t uj_Micsec_1Hz;             /**< MDSR time stamp microseconds. */
    int32_t j_Lat_1Hz;                 /**< Latitude.
                                         *
                                         *   Units: 10<SUP>-7</SUP> degrees.
                                         */
    int32_t j_Lon_1Hz;                 /**< Longitude.
                                         *
                                         *   Units: 10<SUP>-7</SUP> degrees.
                                         */
    int32_t j_Alt_1Hz;                 /**< Altitude of COG above
                                         *   reference ellipsoid.
                                         *   Interpolated value.
                                         *
                                         *   Units: mm
                                         */
    int64_t k_TD_1Hz;                  /**< Window delay referenced to COM.
                                         *   Two-way, corrected
                                         *   for instrument
                                         *   delays.
                                         *
                                         *   Units: mm
                                         */
    uint16_t* pui_avg_wfm;             /**< 1Hz Averaged power waveform. */
    int32_t j_Linear_wfm_mult;         /**< Echo scale factor. */
    int32_t j_Power2_wfm_mult;         /**< Echo scale factor. */
    uint16_t ui_num_avg_echoes;        /**< Number of echoes averaged. */
    uint16_t ui_flags;                 /**< Flags, see ::t_Level1bAvgWfmSARinFlagBit */
}
AveragedWaveformsSARIN;

/** Level 1b record substructure (LRM mode specific) containing waveform data. */
typedef struct _WaveformsLRM
{
    uint16_t* pui_wfm;/**< Power waveform. */
    int32_t j_Linear_wfm_mult;         /**< Echo scale factor. */
    int32_t j_Power2_wfm_mult;         /**< Echo scale factor. */
    uint16_t ui_num_avg_echoes;        /**< Number of echoes averaged. */
    uint16_t ui_flags;                 /**< Flags, see ::t_Level1bWfmLRMFlagBit */
}
WaveformsLRM;

/** Level 1b record substructure (SAR mode specific) containing waveform data. */
typedef struct _WaveformsSAR
{
    uint16_t* pui_wfm;/**< Power waveform. */
    int32_t j_Linear_wfm_mult;         /**< Echo scale factor. */
    int32_t j_Power2_wfm_mult;         /**< Echo scale factor. */
    uint16_t ui_num_avg_echoes;        /**< Number of echoes averaged. */
    uint16_t ui_flags;                 /**< Flags, see ::t_Level1bWfmSARFlagBit */
    int16_t ai_bb_param[ J_NUM_BB_PARAMETERS ]; /**< Beam
                                                  *  behaviour parameters.
                                                  */
}
WaveformsSAR;

/** Level 1b record substructure (SARin mode specific) containing waveform data. */
typedef struct _WaveformsSARIN
{
    uint16_t* pui_wfm; /**< Power waveform. */
    int32_t j_Linear_wfm_mult;         /**< Echo scale factor. */
    int32_t j_Power2_wfm_mult;         /**< Echo scale factor. */
    uint16_t ui_num_avg_echoes;        /**< Number of echoes averaged. */
    uint16_t ui_flags;                 /**< Flags, see t_Level1bWfmSARinFlagBit */
    int16_t ai_bb_param[ J_NUM_BB_PARAMETERS ]; /**< Beam
                                                  *  behaviour parameters. */
    uint16_t* pui_coherence; /**< Coherence waveform. */
    int32_t* pj_phase_diff; /**< Phase
                                                 *  difference waveform. */
}
WaveformsSARIN;

/** Level 1b record structure for LRM mode. */
typedef struct _L1B_LRM
{
    TimeAndOrbitGroup    at_Time_Orbit[ J_NUM_HRB ]; /**< Time and
                                                      *   orbit
                                                      *   parameters substructure. */
    Measurements         at_Measurements[ J_NUM_HRB ]; /**< Measurements
                                                        *   substructure.*/
    Corrections          t_Geo_Corrections; /**< Geo-corrections substructure. */
    AveragedWaveformsLRM t_Average_PL; /**< Averaged
                                        *   waveforms substructure. */
    WaveformsLRM         at_Waveforms[ J_NUM_HRB ]; /**< Waveforms substructure. */
}L1B_LRM;

/** Level 1b record structure for SAR mode. */
typedef struct _L1B_SAR
{
    TimeAndOrbitGroup    at_Time_Orbit[ J_NUM_HRB ]; /**< Time and
                                                      *   orbit
                                                      *   parameters substructure. */
    Measurements         at_Measurements[ J_NUM_HRB ]; /**< Measurements
                                                        *   substructure.*/
    Corrections          t_Geo_Corrections; /**< Geo-corrections substructure. */
    AveragedWaveformsSAR t_Average_PL; /**< Averaged
                                        *   waveforms substructure. */
    WaveformsSAR         at_Waveforms[ J_NUM_HRB ]; /**< Waveforms substructure. */
}L1B_SAR;

/** Level 1b record structure for SARin mode. */
typedef struct _L1B_SARIN
{
    TimeAndOrbitGroup    at_Time_Orbit[ J_NUM_HRB ]; /**< Time and
                                                      * orbit
                                                      * parameters substructure. */
    Measurements         at_Measurements[ J_NUM_HRB ]; /**<
                                                        * Measurements
                                                        * substructure.*/
    Corrections            t_Geo_Corrections; /**< Geo-corrections substructure. */
    AveragedWaveformsSARIN t_Average_PL; /**< Averaged
                                          * waveforms substructure. */
    WaveformsSARIN       at_Waveforms[ J_NUM_HRB ]; /**< Waveforms substructure. */
}L1B_SARIN;

/** Values for Averaged Waveform Flag field in LRM mode.
 *  See AveragedWaveformsLRM#ui_flags
 *
 * @warning These values are valid in LRM mode ONLY.
 */

typedef enum _t_Level1bAvgWfmLRMFlagBit
{
    WFM_LRM_ERROR = 15                  /**< 1 Hz echo error.
                                         *   Not computed.
                                         */
} t_Level1bAvgWfmLRMFlagBit;

/** Values for Averaged Waveform Flag field in SAR and SARin mode.
 *  See AveragedWaveformsSAR#ui_flags, AveragedWaveformsSARIN#ui_flags
 */
 typedef enum _t_Level1bAvgWfmSARFlagBit
 {
     WFM_SAR_ERROR = 15,                /**< 1 Hz echo error.
                                         *   Not computed.
                                         */
     WFM_SAR_MISPOINT = 0               /**< Bad angle. */
 } t_Level1bAvgWfmSARFlagBit;

/** Identical for SAR and SARin */
typedef t_Level1bAvgWfmSARFlagBit t_Level1bAvgWfmSARinFlagBit;

/** Values for Waveform Flag field in LRM mode.
 *  See WaveformsLRM#ui_flags
 *
 * The three-bit integer assembled from the WFM_LRM_TRK bits has the following values:

Value | Meaning
------|--------
0 | no errors
1 | loss of echo
2 | run time error
3 | echo saturation error
7 | unknown error

 */
typedef enum _t_Level1bWfmLRMFlagBit
{
	WFM_LRM_TRK_2 = 2, /**< Bit 2 for WFM_LRM_TRK integer */
	WFM_LRM_TRK_1 = 1, /**< Bit 1 for WFM_LRM_TRK integer */
	WFM_LRM_TRK_0 = 0 /**< Bit 0 for WFM_LRM_TRK integer */
} t_Level1bWfmLRMFlagBit;

/** Values for Waveform Flag field in SAR and SARin mode.
 *  See WaveformsSAR#ui_flags, WaveformsSARIN::ui_flags
 *
 */
typedef enum _t_Level1bWfmSARFlagBit
{
    WFM_APPROX_BEAM_STEER     = 15, /**< Set if approximate beam
                                     *   steering was used.
                                     */
    WFM_EXACT_BEAM_STEER      = 14, /**< Set if exact beam
                                     *   steering was used.
                                     */
    WFM_DOPPLER_COMP          = 13, /**< Set if doppler weighting
                                     *   was computed.
                                     */
    WFM_DOPPLER_APPLY         = 12, /**< Set if doppler weighting
                                     *   was applied before
                                     *   stacking.
                                     */
    WFM_MULTI_LOOK_INCOMPLETE = 11, /**< Set if multi-look was
                                     *   incomplete.
                                     */
    WFM_STEER_ERROR           = 10, /**< Set if a beam angle
                                     *   steering error occurred.
                                     */
    WFM_AA_POWER_ECHOES       = 9,  /**< Set if power echos are
                                     * anti-aliased.
                                     */
    WFM_AUTO_STEER            = 8   /**< Set if auto beam steering
                                     *   was used.*/
}
t_Level1bWfmSARFlagBit;

/** Identical for SAR and SARin */
typedef t_Level1bWfmSARFlagBit t_Level1bWfmSARinFlagBit;

/**
 Create an ASCII dump of a L1B LRM product.

 @param pt Pointer to the product to dump.
 @param pf File pointer to dump to
*/
void vDump_L1B_LRM( L1B_LRM* pt, FILE* pf );

/**
 Create an ASCII dump of a L1B SAR product.

 @param pt Pointer to the product to dump.
 @param pf File pointer to dump to
*/
void vDump_L1B_SAR( L1B_SAR* pt, FILE* pf );

/**
 Create an ASCII dump of a L1B SARIN product.

 @param pt Pointer to the product to dump.
 @param pf File pointer to dump to
*/
void vDump_L1B_SARIN( L1B_SARIN* pt, FILE* pf );

#endif /* ifdef L1B_FORMAT_H */




