#ifndef _L2ISTRUCTURE_H
#define _L2ISTRUCTURE_H

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+

    System: UCL Cryosat IO Library

    Sub-system: Cryosat Specific

    Description:
    Fill a L2 structure from a file.

    ----------------------------------------------
    Documentation cross reference:
    [C2_FMT] L2 Products Format Specification v2.5 09/06/2009 CS-RS-ACS-GS-5123

    ----------------------------------------------
    This file lives in ---
    ./mssl_cryosat

    Coding by:
    David J Brockley
    http://cryosat.mssl.ucl.ac.uk/public

    ----------------------------------------------
    Filename              :     $RCSfile: L2IStructure.h,v $
    CVS Version Number    :     $Revision: 1.5 $
    Last Update           :     $Date: 2015/06/02 11:07:43 $
    What String           :     @(#)$Id: L2IStructure.h,v 1.5 2015/06/02 11:07:43 djb Exp $
    Log                   :
                                $Log: L2IStructure.h,v $
                                Revision 1.5  2015/06/02 11:07:43  djb
                                Preparing for release 2.2

                                Revision 1.4  2014/12/01 11:30:14  djb
                                Pre-testing for release v2.1

                                Revision 1.3  2010/11/11 13:57:47  djb
                                Minor changes to types and naming to adhere to standards.

                                Revision 1.2  2010/08/27 09:22:59  djb
                                Prototype changes to fix on 64 bit machines and to correct the size of the quality flag in L2.

                                Revision 1.1  2010/07/16 14:37:55  djb
                                Port in CryoSat tools from internal branch.


    ---------------------------------------------------------------------------
    This file is the Unpublished Intellectual
    Property of University College London and
    All Rights are Reserved.

    Copyright (c) University College London, 2005, 2010
    ---------------------------------------------------------------------------
*/

/**
 * @file   L2iStructure.h
 *
 * @brief  Definition of the Level 2i structure.
 *
 *
 */

#pragma ident "@(#)$Id: L2IStructure.h,v 1.5 2015/06/02 11:07:43 djb Exp $ MSSL"

/* INCLUDE FILES */

/*-- standard C headers --*/
#include <stdint.h>
#include <inttypes.h>
#include <stdio.h>

/*-- programmer created headers --*/
/* Included only to get documentation for Doxygen */
#include "SharedFlags.h"

/*-- global and external variable declarations --*/

/* STRUCTURES */

/* FUNCTIONS */

/*-- local function prototypes --*/

/* Note that the MCD definition is in the file InternalFormat.h */

/** The interim L2 structure */
typedef struct _CL2L2IStruct
{
    /* ----- Time and Orbit group ------------------------------------------ */
    int32_t j_Day;                      /**< Time: day part.
                                         *
										 *   Units: DAY
                                         */

    uint32_t uj_Sec;                    /**< Time: second part.
                                         *
										 *   Units: msec
                                         */

    uint32_t uj_Micsec;                 /**< Time: microsecond part.
                                         *
										 *   Units: mu-sec
                                         */

    int32_t j_USO_Corr;                 /**< USO Correction factor.
                                         *
										 *   Units: -
                                         */

    uint16_t ui_Mode_ID;                /**< SIRAL Mode ID. See ::t_ModeID 
                                         *
										 *   Units: -
                                         */

    uint16_t ui_SSC;                    /**< Source Sequence Counter.
                                         *   Units: -
                                         */

    uint32_t uj_Inst_config;            /**< Instrument configuration. See ::t_InstConf
                                         *
										 *   Includes loop status.
                                         *
										 *   Units: -
                                         */

    uint32_t uj_Rec_Count;              /**< Surface Sample Counter.
                                         *
										 *   Record counter.
                                         *
										 *   Units: -
                                         */

    int32_t j_Lat;                      /**< Latitude of measurement.
                                         *
										 *   Units: 0.1 micro-degree
                                         */

    int32_t j_Lon;                      /**< Longitude of measurement.
                                         *
										 *   Units: 0.1 micro-degree
                                         */

    int32_t j_Alt;                      /**< Altitude of COG above reference
                                         *   ellipsoid (interpolated value)
                                         *
										 *   Units: mm
                                         */

    int32_t j_Alt_rate;                 /**< Instantaneous altitude rate
                                         *   derived from orbit.
                                         *
										 *   Units: mm/s
                                         */

    int32_t aj_Sat_velocity[ 3 ];       /**< Satellite velocity vector.
                                         *   In CRF
                                         *
										 *   Units: mm/s
                                         */

    int32_t aj_Real_beam[ 3 ];          /**< Real beam direction vector.
                                         *
										 *   In CRF
                                         *
										 *   Units: micro-metre
                                         */

    int32_t aj_Baseline[ 3 ];           /**< Interferometer baseline vector.
                                         *
										 *   In CRF
                                         *
										 *   Units: micro-metre
                                         */
    uint16_t i_ST_ID;                  /**< Star Tracker ID. See ::t_STID
                                         *
										 *   Baseline BC and later only
                                        *
                                        *    Units: enum
                                        */
    int32_t j_Roll_angle;              /**< Spacecraft roll angle.
                                         *
										 *   Baseline C only.
                                         *
										 *   Derived from star trackers.
                                        *
                                        *   Units: 10<SUP>-7</SUP> degrees
                                        */
    int32_t j_Pitch_angle;            /**< Spacecraft roll angle.
                                         *
										 *    Baseline C only.
                                         *
										 *   Derived from star trackers.
                                        *
                                        *   Units: 10<SUP>-7</SUP> degrees
                                        */
    int32_t j_Yaw_angle;              /**< Spacecraft roll angle.
                                         *
										 *    Baseline C only.
                                         *
										 *   Derived from star trackers.
                                        *
                                        *   Units: 10<SUP>-7</SUP> degrees
                                        */
    uint32_t uj_L2_MCD;                 /**< Level 2 Confidence data (flag). See ::t_MCDFlagBit
                                         *
										 *   Units: -
                                         */

    /* ----- Measurements group -------------------------------------------- */
    int32_t j_Height;                   /**< Height with respect to the
                                         *   ellipsoid.
                                         *
										 *   Units: mm
                                         */

    int32_t j_Height_2;                 /**< Height with respect to the
                                         *   ellipsoid from retracker 2.
                                         *
										 *   Baseline BC and above only.
                                         *
										 *   Units: mm
                                         */

    int32_t j_Height_3;                 /**< Height with respect to the
                                         *   ellipsoid from retracker 3.
                                         *
										 *   Baseline BC and above only.
                                         *
										 *   Units: mm
                                         */

    int32_t j_Sig0;                     /**< Sigma 0.
                                         *   Units: dB/100
                                         */

    int32_t j_Sig0_2;                   /**< Sigma 0 from retracker 2.
                                         *
										 *   Baseline BC and above only.
                                         *
										 *   Units: dB/100
                                         */

    int32_t j_Sig0_3;                   /**< Sigma 0 from retracker 3.
                                         *
										 *   Baseline BC and above only.
                                         *
										 *   Units: dB/100
                                         */

    int32_t j_SWH;                      /**< Significant Wave
                                         *   Height.
                                         *
										 *   Units: mm
                                         */

    int32_t j_Peakiness;                /**< Peakiness.
                                         *
										 *   Units: 1/1000
                                         */

    int32_t j_Retrk_range_C;            /**< Retracked range correction.
                                         *
										 *   Units: mm
                                         */

    int32_t j_Retrk_range_C_2;          /**< Retracked range correction for retracker 2.
                                         *
										 *   Baseline BC and above.
                                         *
										 *   Units: mm
                                         */

    int32_t j_Retrk_range_C_3;          /**< Retracked range correction for retracker 3.
                                         *
										 *   Baseline BC and above.
                                         *
										 *   Units: mm
                                         */

    int32_t j_Retrk_sig0_C;             /**< Retracked sigma 0 correction.
                                         *
										 *   Spare in Baseline BC and above.
                                         *
										 *   Units: db/100
                                         */

    int32_t j_Retrk_sig0_C_2;           /**< Retracked sigma 0 correction for retracker 2.
                                         *
										 *   Spare in Baseline BC and above.
                                         *
										 *   Units: db/100
                                         */

    int32_t j_Retrk_sig0_C_3;           /**< Retracked sigma 0 correction for retracker 3.
                                         *
										 *   Spare in Baseline BC and above.
                                         *
										 *   Units: db/100
                                         */

    int32_t j_Retrk_Quality_1;          /**< Retracker 1 quality metric.
                                         *
										 *   Baseline BC and above.
                                         *
										 *   Units: -
                                         */

    int32_t j_Retrk_Quality_2;          /**< Retracker 2 quality metric.
                                         *
										 *   Baseline BC and above.
                                         *
										 *   Units: -
                                         */

    int32_t j_Retrk_Quality_3;          /**< Retracker 3 quality metric.
                                         *
										 *   Baseline BC and above.
                                         *
										 *   Units: -
                                         */

    int32_t j_Retrk_3;                  /**< 3rd retracker output.
                                         *
										 *   Units: -
                                         */

    int32_t j_Retrk_4;                  /**< 4th retracker output.
                                         *
										 *   Units: -
                                         */

    int32_t j_Retrk_5;                  /**< 5th retracker output.
                                         *
										 *   Units: -
                                         */

    int32_t j_Retrk_6;                  /**< 6th retracker output.
                                         *
										 *   Units: -
                                         */

    int32_t j_Retrk_7;                  /**< 7th retracker output.
                                         *
										 *   Units: -
                                         */

    int32_t j_Retrk_8;                  /**< 8th retracker output.
                                         *
										 *   Units: -
                                         */

    int32_t j_Retrk_9;                  /**< 9th retracker output.
                                         *
										 *   Units: -
                                         */

    int32_t j_Retrk_10;                 /**< 10th retracker output.
                                         *
										 *   Units: -
                                         */

    int32_t j_Retrk_11;                 /**< 11th retracker output.
                                         *
										 *   Units: -
                                         */

    int32_t j_Retrk_12;                 /**< 12th retracker output.
                                         *
										 *   Units: -
                                         */

    int32_t j_Retrk_13;                 /**< 13th retracker output.
                                         *
										 *   Units: -
                                         */

    int32_t j_Retrk_14;                 /**< 14th retracker output.
                                         *
										 *   Baseline BC and later.
                                         *
										 *   Units: -
                                         */

    int32_t j_Retrk_15;                 /**< 15th retracker output.
                                         *
										 *   Baseline BC and later.
                                         *
										 *   Units: -
                                         */

    int32_t j_Retrk_16;                 /**< 16th retracker output.
                                         *
										 *   Baseline BC and later.
                                         *
										 *   Units: -
                                         */

    int32_t j_Retrk_17;                 /**< 17th retracker output.
                                         *
										 *   Baseline BC and later.
                                         *
										 *   Units: -
                                         */

    int32_t j_Retrk_18;                 /**< 18th retracker output.
                                         *
										 *   Baseline BC and later.
                                         *
										 *   Units: -
                                         */

    int32_t j_Retrk_19;                 /**< 19th retracker output.
                                         *
										 *   Units: -
                                         */

    int32_t j_Retrk_20;                 /**< 20th retracker output.
                                         *
										 *   Baseline BC and later.
                                         *
										 *   Units: -
                                         */

    int32_t j_Retrk_21;                 /**< 21st retracker output.
                                         *
										 *   Baseline BC and later.
                                         *
										 *   Units: -
                                         */

    int32_t j_Retrk_22;                 /**< 22nd retracker output.
                                         *
										 *   Baseline BC and later.
                                         *
										 *   Units: -
                                         */

    int32_t j_Retrk_23;                 /**< 23rd retracker output.
                                         *
										 *   Baseline BC and later.
                                         *
										 *   Units: -
                                         */

    int32_t j_echo_shape;               /**< Power echo shape parameter.
                                         *
										 *   Units: dB/100
                                         */

    int16_t ai_bb_param[ 50 ];         /**< Beam behaviour parameter.
                                         *
										 *   The beam behaviour parameter is a
                                         *   unitless code number related to
                                         *   surface characteristics.
                                         *
										 *   Units: -
                                         */

    int32_t j_XTrack_angle;             /**< Cross track angle.
                                         *
										 *   At retrack point.
                                         *
										 *   Units: micro-radians
                                         */

    int32_t j_XTrack_angle_C;           /**< Cross track angle correction.
                                         *
										 *   Baseline BC and later.
                                         *
										 *   Units: micro-radians
                                         */

    int32_t j_Coherence;                /**< Coherence (leading edge)
                                         *
										 *   At retrack point.
                                         *
										 *   Units: 1/1000
                                         */

    int32_t j_Ocean_ht;                 /**< Interpolated Ocean Height.
                                         *   -
                                         *   Units: mm
                                         */

    int32_t j_Freeboard;                /**< Freeboard.
                                         *
										 *   Units: mm
                                         */

    int32_t j_SHA;                      /**< Surface height anomaly.
                                         *
										 *   Units: mm
                                         */

    int32_t j_SSHA_interp;              /**< Interpolated sea surface height
                                         *   anomaly.
                                         *
										 *   Units: mm
                                         */

    uint16_t ui_interp_err;             /**< Error in ocean height interpolation.
                                         *
										 *   Estimated.
                                         *
										 *   Units: mm
                                         */

    uint16_t ui_interp_cnt_fwd;          /**< Number of forward records
                                          *   interpolated.
                                         *
										 *   Units: count
                                          */

    uint16_t ui_interp_cnt_bkwd;          /**< Number of backward records
                                           *   interpolated.
                                         *
										 *   Units: count
                                           */

    uint16_t ui_interp_time_fwd;          /**< Distance in time of most
                                           *   forward record interpolated.
                                         *
										 *   Units: milli-seconds
                                           */

    uint16_t ui_interp_time_bkwd;          /**< Distance in time of most
                                            *   backward record interpolated.
                                         *
										 *   Units: milli-seconds
                                            */

    uint16_t ui_interp_error_F;           /**< Interpolation error flag. See ::t_InterpolationErrorFlagBit
                                           */

    uint32_t uj_Meas_Mode;              /**< Measurement mode. See ::t_MeasurementMode
                                         *
                                         */

    uint32_t uj_Quality_F;              /**< Quality flags. See ::t_MeasurementQualityFlagBit
                                         *
										 *   Units: -
                                         */

    uint32_t uj_Retracker_F;            /**< Retracker flags. See ::t_RetrackerFlagBit
                                         *
										 *   Units: -
                                         */

    uint32_t uj_Ht_status;              /**< Height calculation details. See ::t_HeightStatusFlagBit
                                         *
										 *   Specifies what was applied during
                                         *   the height calculation.
                                         *
										 *   Units: -
                                         */

    uint32_t uj_Freeb_status;           /**< SAR freeboard status flag. See ::t_FreeboardStatusFlagBit
                                         *
										 *   Units: -
                                         */

    uint16_t ui_n_avg;                  /**< Number of echoes or beams
                                         *   averaged.
                                         *
										 *   Units: -
                                         */

    uint16_t ui_Wind_speed;             /**< Wind speed.
                                         *
										 *   Units: mm/s
                                         */
    uint8_t auc_spares1[ 12 ];          /**< Spares.
                                         *
										 *   Units: -
                                         */

    /* ----- Auxiliary Measurements Group ---------------------------------- */
    int32_t j_Ice_conc;                 /**< Ice concentration parameter.
                                         *
										 *   Units: %/1000
                                         */

    int32_t j_Snow_depth;               /**< Snow depth.
                                         *
										 *   Units: mm
                                         */

    int32_t j_Snow_density;             /**< Snow density.
                                         *
										 *   Units: kg/m<SUP>3</SUP>
                                         */

    int32_t j_Discriminator;            /**< Discriminator result. See ::t_DiscriminatorResult
                                         *
										 *   Units: Enumeration
                                         */

    int32_t j_SARin_disc_1;             /**< SARin discriminator parameter 1.
                                         *
										 *   Units: TBD
                                         */

    int32_t j_SARin_disc_2;             /**< SARin discriminator parameter 2.
                                         *
										 *   Units: TBD
                                         */

    int32_t j_SARin_disc_3;             /**< SARin discriminator parameter 3.
                                         *
										 *   Units: TBD
                                         */

    int32_t j_SARin_disc_4;             /**< SARin discriminator parameter 4.
                                         *
										 *   Units: TBD
                                         */

    int32_t j_SARin_disc_5;             /**< SARin discriminator parameter 5.
                                         *   Units: TBD
                                         */

    int32_t j_SARin_disc_6;             /**< SARin discriminator parameter 6.
                                         *
										 *   Units: TBD
                                         */

    int32_t j_SARin_disc_7;             /**< SARin discriminator parameter 7.
                                         *
										 *   Units: TBD
                                         */

    int32_t j_SARin_disc_8;             /**< SARin discriminator parameter 8.
                                         *
										 *   Units: TBD
                                         */

    int32_t j_SARin_disc_9;             /**< SARin discriminator parameter 9.
                                         *
										 *   Units: TBD
                                         */

    int32_t j_SARin_disc_10;            /**< SARin discriminator parameter 10.
                                         *
										 *   Units: TBD
                                         */

    uint32_t uj_Discrim_F;              /**< Discriminator flags. See ::t_DiscriminatorStatusFlagBit
                                         *
										 *   Units: -
                                         */

    int32_t j_Attitude;                 /**< Slope model correction.
                                         *
										 *   Attitude of echo.
                                         *
										 *   Units: micro-degrees
                                         */

    int32_t j_Azimuth;                  /**< Slope model correction.
                                         *
										 *   Azimuth of echo.
                                         *
										 *   Units: micro-degrees
                                         */

    int32_t j_Slope_Doppler_C;          /**< Slope doppler correction.
                                         *
										 *   Baseline BC and later.
                                         *
										 *   Units: mm
                                         */

    int32_t j_Lat_sat;                  /**< The original lattitude of
                                         *   the satellite.
                                         *
										 *   Units: micro-degrees
                                         */

    int32_t j_Lon_sat;                  /**< The original longitude of
                                         *   the satellite.
                                         *
										 *   Units: micro degree
                                         *          (-180,180)
                                         *          +ve E does NOT contain
                                         *          180.
                                         *          -ve W DOES contain -180
                                         */

    uint32_t uj_Ambiguity;              /**< Ambiguity indicator. See ::t_Level2AmbiguityErrorFlagBit
                                         *
										 *   Units: -
                                         */

    int32_t j_MSS_mod;                  /**< MSS from standard model.
                                         *   
                                         *   Units: mm
                                         */

    int32_t j_Geoid_mod;                /**< Geoid from standard model.
                                         *   
                                         *   Units: mm
                                         */

    int32_t j_ODLE_mod;                 /**< ODLE from standard model.
                                         *   
                                         *   Units: mm
                                         */

    int32_t j_DEM_elev;                 /**< The interpolated elevation value
                                         *   obtained from the DEM.
                                         *
										 *   Units: mm
                                         */

    uint32_t uj_DEM_id;                 /**< Identification of DEM used in
                                         *   SARin ambiguity test.
                                         *
										 *   Units: -
                                         */
    uint8_t auc_spares2[ 16 ];          /**< Spares.
                                         *
										 *   Units: -
                                         */

    /* ----- External Corrections Group ------------------------------------ */
    int32_t j_Dry_C;                    /**< Dry Tropospheric Correction.
                                         *
										 *   Units: mm
                                         */

    int32_t j_Wet_C;                    /**< Wet Tropospheric Correction.
                                         *
										 *   Units: mm
                                         */

    int32_t j_IB_C;                     /**< Inverse Barometric Correction.
                                         *
										 *   Units: mm
                                         */

    int32_t j_DAC_C;                    /**< Delta Inverse Barometric Correction.
                                         *
										 *   Units: mm
                                         */

    int32_t j_Iono_GIM;                 /**< GIM Ionospheric Correction.
                                         *
										 *   Units: mm
                                         */

    int32_t j_Iono_mod;                 /**< Model Ionospheric Correction.
                                         *
										 *   Units: mm
                                         */

    int32_t j_H_OT;                     /**< Ocean Tide .
                                         *
										 *   Units: mm
                                         */

    int32_t j_H_LPEOT;                  /**< Long-period equilibrium ocean tide.
                                         *
										 *   Units: mm
                                         */

    int32_t j_H_OLT;                    /**< Ocean Loading Tide.
                                         *
										 *   Units: mm
                                         */

    int32_t j_H_SET;                    /**< Solid Earth Tide.
                                         *
										 *   Units: mm
                                         */

    int32_t j_H_GPT;                    /**< Geocentric Polar Tide.
                                         *
										 *   Units: mm
                                         */

    uint32_t uj_Surf_type;              /**< Surface type flag. See ::t_SurfaceType
                                         *
										 *   Units: -
                                         */

    uint32_t uj_Corr_status;            /**< Correction status flag. See ::t_CorrectionStatusFlagBit
                                         *
										 *   Units: -
                                         */

    uint32_t uj_Corr_error;             /**< Correction error flags. See ::t_CorrectionErrorFlagBit
                                         *
										 *   Units: -
                                         */

    int32_t j_SSB;                      /**< Sea state bias correction.
                                         *
										 *   Units: mm
                                         */

    uint8_t auc_spares3[ 8 ];           /**< Spare.
                                         *
										 *   Units: -
                                         */

    /* ----- Internal Corrections Group ------------------------------------ */
    int32_t j_Dopp_rc;                  /**< Doppler range correction.
                                         *
										 *   Radial + slope.
                                         *
										 *   Units: mm
                                         */

    int32_t j_TR_inst_rc;               /**< Instrument Range Correction.
                                         *
										 *   t-r antenna.
                                         *
										 *   Units: mm
                                         */

    int32_t j_R_inst_rc;                /**< Instrument Range Correction.
                                         *
										 *   r-only antenna.
                                         *
										 *   Units: mm
                                         */

    int32_t j_TR_inst_gain_C;           /**< Instrument Sigma 0 Correction.
                                         *
										 *   t-r antenna.
                                         *
										 *   Units: dB/100
                                         */

    int32_t j_R_inst_gain_C;            /**< Instrument Sigma 0 Correction.
                                         *
										 *   r-only antenna.
                                         *
										 *   Units: dB/100
                                         */

    int32_t j_Int_phase_C;              /**< Internal Phase Correction.
                                         *
										 *   Units: milli-radians
                                         */

    int32_t j_Ext_phase_C;              /**< External Phase Correction.
                                         *
										 *   Units: milli-radians
                                         */

    int32_t j_Noise_pwr;                /**< Noise Power measurement.
                                         *
										 *   Noise sampled average. Bandwidth
                                         *   320 MHz or 40 MHz.
                                         *
										 *   Units: TBD
                                         */

    int32_t j_Phase_slope_C;            /**< Phase slope correction.
                                         *
										 *   Units: microradians
                                         */

    uint8_t auc_spares4[ 8 ];           /**< Spares.
                                         *
										 *   Units: -
                                         */
}
L2IData;





/** Bit IDs for Interpolation error flag. See L2IData#ui_interp_error_F */
typedef enum _t_InterpolationErrorFlagBit{
    INTERP_SAR_UNREL     = 0,          /**< Interpolation result may be
                                        *   unreliable.
                                        */
    INTERP_SAR_ONE_SIDED     = 1,      /**< All of the interpolation points
                                        *   were taken from one side of the
                                        *   centre point.
                                        */
    INTERP_SAR_NO_POINTS     = 2       /**< No ocean points were within the
                                        *   interpolation radius.
                                        */
}
t_InterpolationErrorFlagBit;

/** Values for Measurement mode. See L2IData#uj_Meas_Mode

Vaue | Mode
-----|-----
1    | LRM
2    | SAR
3    | SARin

*/
typedef enum _t_MeasurementMode {
	MODE_LRM = 1, /**< Measurement mode is LRM. */
	MODE_SAR = 2, /**< Measurement mode is SAR. */
	MODE_SARIN = 3 /**< Measurement mode is SARin */
} t_MeasurementMode;

/** Bit IDs for Quality flags. See L2IData#uj_Quality_F */
typedef enum _t_MeasurementQualityFlagBit {
    MQ_HEIGHT_1_ERR         = 31,       /**< Height Error. */
    MQ_HEIGHT_2_ERR         = 30,       /**< Height Error. */
    MQ_HEIGHT_3_ERR         = 29,       /**< Height Error. */
    MQ_SIGMA0_1_ERR         = 28,       /**< Sigma 0 Error. */
    MQ_SIGMA0_2_ERR         = 27,       /**< Sigma 0 Error. */
    MQ_SIGMA0_3_ERR         = 26,       /**< Sigma 0 Error. */
    MQ_PK_ERR               = 25,       /**< Peakiness Error. */
    MQ_ECHO_SHAPE_ERR       = 24,       /**< Echo shape error. */
    MQ_XTRACK_ERR           = 23,       /**< X-Track angle error. */
    MQ_COHERENCE_ERR        = 22,        /**< Coherence error. */
    MQ_ARITHMETIC_ERR       = 21,        /**< IEEE Maths flag set. */
    MQ_ALTWIND_ERR          = 20,        /**< Error calculating altimeter windspeed */
    MQ_SWH_ERR              = 19,        /**< Error calculating SWH */
}
t_MeasurementQualityFlagBit;

/** Bit IDs for Retracker flags. See L2IData#uj_Retracker_F */
typedef enum _t_RetrackerFlagBit {
    RTRK_FAIL                   = 31,  /**< Overall retracker warning. Not used. */
    RTRK_LO_PWR                 = 30,  /**< Zero or low power waveform. */
    RTRK_LO_PK                  = 29,  /**< Low peakiness. */
    RTRK_HI_PK                  = 28,  /**< High peakiness. */
    RTRK_HI_NOISE               = 27,  /**< High noise. */
    RTRK_LO_VAR                 = 26,  /**< Low variance. */
    RTRK_LE_BAD                 = 25,  /**< Bad leading edge. */
    RTRK_OUT_OF_RNGE            = 24,  /**< Retrack position out of range. */
    RTRK_BAD_BB                 = 23,  /**< Abnormal Beam Behaviour Params. */
    RTRK_SPARE                  = 22,  /**< Spare. */
    RTRK_SPARE1                 = 21,  /**< Spare. */
    RTRK_BAD_U10                = 20,  /**< Reserved U10. */
    RTRK_SPARE2                 = 19,  /**< Spare 2. */
    RTRK_SARIN_INTERPOL         = 17,  /**< Retrack interpolation failure. */
    RTRK_SARIN_LO_COHERENCE     = 16,  /**< Low coherence. */
    RTRK_FIT_FAILED             = 15,  /**< Retracker fit failure. */
    RTRK_FDM_OCOG_FAIL          = 14,  /**< OCOG retrack failed */
    RTRK_POOR_FIT               = 13,  /**< Retracker fit was of poor quality. */
    RTRK_POOR_PHASE_FIT         = 12,  /**< SARin phase fit was of poor quality. */
    RTRK_1_FAIL                 = 11,  /**< Retracker 1 failed. */
    RTRK_2_FAIL                 = 10,  /**< Retracker 2 failed. */
    RTRK_3_FAIL                 = 9    /**< Retracker 3 failed. */
}
t_RetrackerFlagBit;

/** Bit IDs for Height calculation details. See L2IData#uj_Ht_status */
typedef enum _t_HeightStatusFlagBit {
    HGT_INTERNAL          = 31,         /**< Internal callibration. */
    HGT_RAD_DOPPLER       = 30,         /**< Radial doppler. */
    HGT_DRY_TROPO         = 29,         /**< Dry tropospheric. */
    HGT_WET_TROPO         = 28,         /**< Wet tropospheric. */
    HGT_INV_BARO          = 27,         /**< Inverse barometric. */
    HGT_DAC               = 26,         /**< Dynamic Atmospheric Correction. */
    HGT_IONO_GIM          = 25,         /**< GIM ionospheric. */
    HGT_IONO_MODEL        = 24,         /**< Model ionospheric. */
    HGT_OT                = 23,         /**< Ocean Tide. */
    HGT_LPEOT             = 22,         /**< LPEOT. */
    HGT_OLT               = 21,         /**< Ocean loading time. */
    HGT_SET               = 20,         /**< Solid Earth tide. */
    HGT_GPT               = 19,         /**< Geocentric Polar tide. */
    HGT_SLOPE_DOPPLER     = 18,         /**< Slope Doppler. */
    HGT_WINDOW_OFFSET     = 17,         /**< Mode specific window offset. */
    HGT_SAR_RETRK         = 16,         /**< SAR retracker offset. */
    HGT_SARIN_RETRK       = 15,         /**< SARIN retracker offset. */
    HGT_LRM_RETRK         = 14,         /**< LRM retracker offset. */
    HGT_LRM_OCEANBIAS     = 13,         /**< LRM ocean bias applied. */
    HGT_LRM_ICEBIAS       = 12,         /**< LRM ice bias applied. */
    HGT_SAR_OCEANBIAS     = 11,         /**< SAR ocean bias applied. */
    HGT_SAR_ICEBIAS       = 10,         /**< SAR ice bias applied. */
    HGT_SARIN_OCEANBIAS     = 9,        /**< SARin ocean bias applied. */
    HGT_SARIN_ICEBIAS     = 8,          /**< SARin ice bias applied. */
    LRM_SLOPE_DATA_VALID     = 7,       /**< LRM slope model data valid. */
    HGT_SARIN_BAD_BASELINE     = 6,     /**< SARin Bad Baseline downward. */
    HGT_SARIN_OUT_OF_RANGE     = 5,     /**< SARin out of range. */
    HGT_SARIN_BAD_VELOCITY     = 4,     /**< SARin Bad Velocity. */
    HGT_SSB                    = 3,     /**< SSB Applied */
    HGT_FAIL                   = 0      /**< Generic failure flag. Not used. */
}
t_HeightStatusFlagBit;

/** Bit IDs for SAR freeboard status flag. See L2IData#uj_Freeb_status */
typedef enum _t_FreeboardStatusFlagBit
{
     FREB_SAR_AVAIL     = 0,             /**< Freeboard estimation
                                         * unavailable. */
     FREB_SAR_UNREL     = 1,             /**< Freeboard estimation
                                         * unreliable. */
     FREB_SAR_IN_NORTH_BOUND     = 2,    /**< Freeboard estimation is in
                                         * northern geographical boundary. */
     FREB_SAR_IN_SOUTH_BOUND     = 3     /**< Freeboard estimation is in
                                         * southern geographical boundary. */
}
t_FreeboardStatusFlagBit;

/** Values for Discriminator result. See L2IData#j_Discriminator */
typedef enum _t_DiscriminatorResult {
    CL2_LRM_IS_UNDEFINED = 1,           /**< Result of LRM discrimination
                                         *   is undefined.
                                         */
    CL2_LRM_IS_OCEAN = 2,               /**< Result of LRM discrimination
                                         *   is ocean.
                                         */
    CL2_LRM_IS_LAND_ICE_PLATEAU = 3,    /**< Result of LRM discrimination
                                         *   is land ice plateau.
                                         */
    CL2_SARIN_IS_UNDEFINED = 101,       /**< Result of SARIN discrimination
                                         *   is undefined.
                                         */
    CL2_SARIN_IS_ALTIMETER_MODE = 102,  /**< Result of SARIN discrimination
                                         *   is altimeter mode.
                                         */
    CL2_SAR_IS_UNDEFINED = 201,         /**< Result of SAR discrimination
                                         *   is undefined.
                                         */
    CL2_SAR_IS_OCEAN = 202,             /**< Result of SAR discrimination
                                         *   is ocean.
                                         */
    CL2_SAR_IS_SEA_ICE = 203,           /**< Result of SAR discrimination
                                         *   is sea ice.
                                         */
    CL2_SAR_IS_LEADS = 204              /**< Result of SAR discrimination
                                         *   is leads.
                                         */
}
t_DiscriminatorResult;

/** Bit IDs for Discriminator flags. See L2IData#uj_Discrim_F */
typedef enum _t_DiscriminatorStatusFlagBit
{
    DISC_FAIL     = 31,	                /**< Overall discriminator failure. */
    DISC_TBD     = 30,                  /**< LRM discriminator flag TBD. */
    DISC_SARIN_LO_VARIANCE     = 21,    /**< SARin Low Variance. */
    DISC_SARIN_BAD_LE     = 20,         /**< SARin Bad Leading Edge. */
    DISC_SARIN_HI_NOISE     = 19,       /**< SARin High Noise. */
    DISC_SARIN_LOW_PEAKINESS     = 18,  /**< SARin Low Peakiness. */
    DISC_SARIN_LOW_POWER     = 17,      /**< SARin Low Power. */
    DISC_SARIN_HI_PEAKINESS     = 16,   /**< SARin High Peakiness. */
    DISC_SAR_HI_PK     = 11,            /**< SAR very high peakiness. */
    DISC_SAR_LO_PK     = 10,            /**< SAR very low peakiness. */
    DISC_SAR_LO_POWER     = 9,          /**< SAR low or zero power. */
    DISC_SAR_BAD_BB     = 8,            /**< SAR abnormal beam behaviour. */
    DISC_SAR_ICECONC_UNAV     = 7,      /**< SAR ice conc unavailable. */
    DISC_SAR_ICECONC_UNREAL     = 6,    /**< SAR ice conc unreliable. */
    DISC_SAR_BAD_NOISE     = 5,         /**< SAR signal to noise too low. */
    DISC_SAR_BAD_WIDTH     = 4,         /**< SAR waveform too wide. */
    DISC_SAR_NO_MATCH     = 28,         /**< SAR none of the available
                                         *   classifications match for
                                         *   this record.
                                         */
    DISC_SAR_MULTIPLE_MATCH     = 29    /**< More than one of the available
                                         *   classifications match for this
                                         *   record.
                                         */
}
t_DiscriminatorStatusFlagBit;

/** Bit IDs for Ambiguity indicator.*/
typedef enum _t_Level2AmbiguityErrorFlagBit
{
  AMB_FAIL     = 31, /**< Ambiguity test failed. Result is possibly affected by phase wrapping. */
  AMB_SARIN_NO_DEM     = 21, /**< Test failed because there is no DEM for this location. */
  AMB_SARIN_DIFF_ELEV     = 20, /**< Test failed because the elevation difference was beyond threshold. */
  AMB_SARIN_NO_RTRK     = 19, /**< Test failed because retracking failed. */
  AMB_SARIN_BAD_MATH     = 18 /**< Test failed due to a mathematical error. */
}
t_Level2AmbiguityErrorFlagBit;


/** Size of an L2i prodyct record. */
#define J_CS_L2I_STRUCT_SIZE ( sizeof( L2IData ) )

/**
 Dump an L2i product to a file in ASCII format.

 @param pt Pointer to the L2i product to dump.
 @param pf Pointer to the file to dump to.
*/
void vDump_L2IData( L2IData* pt, FILE* pf );

#endif /* _L2ISTRUCTURE_H */
