#ifndef _L2STRUCTURE_H
#define _L2STRUCTURE_H

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+

    System: UCL Cryosat IO Library

    Sub-system: Cryosat Specific

    Description:
    Include File.

    ----------------------------------------------
    Documentation cross reference:
    [C2_FMT] L2 Products Format Specification v2.5 09/06/2009 CS-RS-ACS-GS-5123

    ----------------------------------------------
    This file lives in ---
    ./mssl_cryosat

    Coding by:
    David J Brockley
    http://cryosat.mssl.ucl.ac.uk/public

    ----------------------------------------------
    Filename              :     $RCSfile: L2Structure.h,v $
    CVS Version Number    :     $Revision: 1.6 $
    Last Update           :     $Date: 2015/06/02 11:07:43 $
    What String           :     @(#)$Id: L2Structure.h,v 1.6 2015/06/02 11:07:43 djb Exp $
    Log                   :
                                $Log: L2Structure.h,v $
                                Revision 1.6  2015/06/02 11:07:43  djb
                                Preparing for release 2.2

                                Revision 1.5  2014/12/01 11:30:14  djb
                                Pre-testing for release v2.1

                                Revision 1.4  2010/11/11 13:57:47  djb
                                Minor changes to types and naming to adhere to standards.

                                Revision 1.3  2010/10/13 08:12:51  djb
                                PRE_RELEASE_1_1

                                Revision 1.2  2010/08/27 09:22:59  djb
                                Prototype changes to fix on 64 bit machines and to correct the size of the quality flag in L2.

                                Revision 1.1  2010/07/16 14:37:55  djb
                                Port in CryoSat tools from internal branch.



    -----------------------------------------------------------
    This file contains the Unpublished Intellectual Property of
    University College London and All Rights Are Reserved.
        Copyright (c) University College London, 2005, 2010
    -----------------------------------------------------------
-
*/

#pragma ident "@(#)$Id: L2Structure.h,v 1.6 2015/06/02 11:07:43 djb Exp $ MSSL"

/**
 * @file   L2Structure.h
 *
 * @brief  Structure and constant definitions for the Cryosat Level 2 format.
 *
 *
 */

/* INCLUDE FILES */

/*-- standard C headers --*/
#include <stdint.h>
#include <inttypes.h>
#include <stdio.h>

/*-- programmer created headers --*/

/*-- global and external variable declarations --*/

/* STRUCTURES */

/* FUNCTIONS */

/*-- local function prototypes --*/

#include "BitfieldStructure.h"

/* Note that the MCD definition is in the file InternalFormat.h */

/**
 * The corrections substructure of the Cryosat Level 2 structure.
 */
typedef struct _L2FinalC
{
    int16_t i_Dry_C;                    /**< Dry Tropospheric Correction.
                                         *
										 *   Units: mm
                                         */
    int16_t i_Wet_C;                    /**< Wet Tropospheric Correction.
                                         *
										 *   Units: mm
                                         */
    int16_t i_IB_C;                     /**< Inverse Barometric Correction.
                                         *
										 *   Units: mm
                                         */
    int16_t i_DAC_C;                    /**< Dynamic Atmosphere Correction.
                                         *
										 *   Units: mm
                                         */
    int16_t i_Iono_C;                   /**< Ionospheric Correction.
                                         *
										 *   Units: mm
                                         */
    int16_t i_SSB;                      /**< Sea State Bias Correction.
                                         *
										 *   Units: mm
                                         */
    int16_t i_H_OT;                     /**< Ocean tide.
                                         *
										 *   Units: mm
                                         */
    int16_t i_H_LPEOT;                  /**< Long period equilibrium ocean tide.
                                         *
										 *   Units: mm
                                         */
    int16_t i_H_OLT;                    /**< Ocean Loading Tide.
                                         *
										 *   Units: mm
                                         */
    int16_t i_H_SET;                    /**< Solid Earth Tide.
                                         *
										 *   Units: mm
                                         */
    int16_t i_H_GPT;                    /**< Geocentric Polar Tide.
                                         *
										 *   Units: mm
                                         */
    int16_t i_Spare1;                   /**< spare
                                         *
										 *   Units:
                                         */
    uint64_t uk_Surf_type;              /**< Surface Type.
                                         *
                                         *   Packed in groups of three bits for
                                         *   each of the 20 records.
                                         *
										 *   Units: -
                                         */
    int32_t j_MSS_Geoid;                /**< Mean Sea Surface or Geoid.
                                         *
										 *   Units: mm
                                         */
    int32_t j_ODLE;                     /**< ODLE
                                         *
										 *   Units: mm
                                         */
    int16_t i_Ice_conc;                 /**< Ice Conc.
                                         *
										 *   Units: %/100
                                         */
    int16_t i_Snow_depth;               /**< Snow Depth.
                                         *   Units: mm
                                         */
    int16_t i_Snow_density;             /**< Snow Density.
                                         *
										 *   Units: kg/m<SUP>3</SUP>
                                         */
    uint32_t uj_C_status;               /**< Corrections Status Flag, see ::t_Level2CorrectionsStatusFlagBit
                                         *
										 *   Units: -
                                         */
    int16_t i_SWH;                      /**< SWH
                                         *
										 *   Units: mm
                                         */
    uint16_t ui_Wind_speed;             /**< Wind Speed.
                                         *
										 *   Units: mm/s
                                         */

} L2FinalC;

/**
 * The measurements substructure of the Cryosat Level 2 structure.
 */
typedef struct _L2FinalM
{
    int32_t j_D_time_mics;              /**< Delta time.
                                         *
                                         *   The delta between the timestamp
                                         *   for this record and the timestamp
                                         *   given at 1HZ.
                                         *
										 *   Units: microseconds
                                         */
    int32_t j_Lat;                      /**< Lattitude.
                                         *
										 *   Units: 0.1 microdegrees
                                         */
    int32_t j_Lon;                      /**< Longitude.
                                         *
										 *   Units: 0.1 microdegrees
                                         */
    int32_t j_Height;                   /**< Measured elevation above ellipsoid from retracker 1.
                                         *
										 *   Units: mm
                                         */
    int32_t j_Height_2;                 /**< Measured elevation above ellipsoid from retracker 2.
                                         *
										 *   Baseline C only.
                                         *
										 *   Units: mm
                                         */
    int32_t j_Height_3;                 /**< Measured elevation above ellipsoid from retracker 3.
                                         *
										 *   Baseline C only.
                                         *
										 *   Units: mm
                                         */
    int16_t i_SSHA_interp;              /**< Interpolated Sea Surface Height Anomaly.
                                         *
										 *   Units: mm
                                         */
    int16_t i_SSHA_num;                 /**< Interpolated Sea Surface Height Anomaly.
                                         *
										 *   Units: mm
                                         */
    int16_t i_SSHA_qual;                /**< Interpolation quality estimate RSS.
                                         *
										 *   Units: mm
                                         */
    int16_t i_Sig0;                     /**< Sigma Zero Backscatter for retracker 1.
                                         *
										 *   Units: 10<SUP>-2</SUP> dB
                                         */
    int16_t i_Sig0_2;                   /**< Sigma Zero Backscatter for retracker 2.
                                         *
										 *   Baseline C only.
                                         *
										 *   Units: 10<SUP>-2</SUP> dB
                                         */
    int16_t i_Sig0_3;                   /**< Sigma Zero Backscatter for retracker 3.
                                         *
										 *   Baseline C only.
                                         *
										 *   Units: 10<SUP>-2</SUP> dB
                                         */
    int16_t i_Peakiness;                /**< Peakiness.
                                         *
										 *   Units: 1/100
                                         */
    int16_t i_Freeboard;                /**< Freeboard.
                                         *
										 *   -9999 is a default value indicating that
                                         *   the computation has not been performed.
                                         *
										 *   Units: mm
                                         */
    int16_t i_N_avg;                    /**< Number of averaged echoes or beams.
                                         *
										 *   Units: -
                                         */

    uint32_t uj_Quality_F;              /**< Quality flags. See ::t_Level2QualityStatusFlagBit
                                         *
										 *   Units: -
                                         */
    uint32_t uj_C_applied;              /**< Corrections Application Flag. See ::t_Level2CorrectionsAppliedFlagBit
                                         *
										 *   Baseline C only.
                                         *
										 *   Units: -
                                         */
    int32_t j_Quality_1;                /**< Quality metric for retracker 1.
                                         *
										 *   Baseline C only.
                                         *
										 *   Units: -
                                         */
    int32_t j_Quality_2;                /**< Quality metric for retracker 2.
                                         *
										 *   Baseline C only.
                                         *
										 *   Units: -
                                         */
    int32_t j_Quality_3;                /**< Quality metric for retracker 3.
                                         *
										 *   Baseline C only.
                                         *
										 *   Units: -
                                         */
} L2FinalM;

/**
 * The Cryosat Level 2 data structure.
 */
typedef struct _L2FinalData
{
    int32_t j_Day;                      /**< Time: day part.
                                         *
										 *   Units: DAY
                                         */

    uint32_t uj_Sec;                    /**< Time: second part.
                                         *
										 *   Units: sec
                                         */

    uint32_t uj_Micsec;                 /**< Time: microsecond part.
                                         *
										 *   Units: mu-sec
                                         */
    uint64_t uk_Siral_mode;             /**< SIRAL mode.
                                         *
										 *   Packed at 2 bits for each
                                         *   of 20 records.
                                         *
										 *   Units: -
                                         */
    int32_t j_Lat_1Hz;                  /**< Latitude of measurement.
                                         *
										 *   Units: 0.1 micro-degree
                                         */
    int32_t j_Lon_1Hz;                  /**< Longitude of measurement.
                                         *
										 *   Units: 0.1 micro-degree
                                         */
    int32_t j_Alt_1Hz;                  /**< Altitude of COG above reference
                                         *   ellipsoid (interpolated value).
                                         *
										 *   Units: mm
                                         */
    int16_t i_Mispointing;              /**< Mispointing angle.
                                         *   Not used in Baseline C.
                                         *
										 *   Units: milli-degree
                                         */
    int32_t j_Roll_angle;              /**< Spacecraft roll angle.
                                        *
										*   Baseline C only.
                                        *
										*   Derived from star trackers.
                                        *
                                        *   Units: 10<SUP>-7</SUP> degrees
                                        */
    int32_t j_Pitch_angle;            /**< Spacecraft roll angle.
                                        *
										*   Baseline C only.
                                        *
										*   Derived from star trackers.
                                        *
                                        *   Units: 10<SUP>-7</SUP> degrees
                                        */
    int32_t j_Yaw_angle;              /**< Spacecraft roll angle.
                                        *
										*   Baseline C only
                                        *
										*   Derived from star trackers.
                                        *
                                        *   Units: 10<SUP>-7</SUP> degrees
                                        */
    int16_t i_N_Valid;                  /**< Number of valid records.
		                                 *
                                         *   The number of records in the block
                                         *   of twenty that actually contain
                                         *   data. The last few records of the
                                         *   last block of a datset may be
                                         *   blank blocks inserted to bring the
                                         *   the file up to a multiple of twenty.
                                         *
										 *   Units: -
                                         */

    L2FinalC t_Corr;                    /**< Correction substructure. */

    L2FinalM at_Meas[ 20 ];             /**< Measurement substructure array. */
} L2Data;

/** Number of bytes IN MEMORY for the Level 2 Struture. */
#define J_L2_STRUCT_SIZE ( sizeof( L2Data ) )

/** Number of bytes ON DISK for the Level 2 structure. */
#define J_L2_RECORD_SIZE 980

/** Bit definitions for the L2 Quality Status Flag, see L2FinalM#uj_Quality_F */
typedef enum _t_Level2QualityStatusFlagBit
{
    L2Q_BLOCK_DEGRADED = 31,          /**< Set if block is degraded. */
    L2Q_ORBIT_ERROR = 30,             /**< Set if an orbit-related error
                                        *  occurred.
                                        */
    L2Q_ORBIT_DISCONTINUITY = 29,     /**< Set if the orbit was
                                        *  discontinuous.
                                        */
    L2Q_HEIGHT_1_ERROR = 28,          /**< Set if any error occurred during
                                        *  the calculation of the height
                                        *  value for retracker 1.
                                        */
    L2Q_HEIGHT_2_ERROR = 27,          /**< Set if any error occurred during
                                        *  the calculation of the height
                                        *  value for retracker 2.
                                        */
    L2Q_HEIGHT_3_ERROR = 26,          /**< Set if any error occurred during
                                        *  the calculation of the height
                                        *  value for retracker 3.
                                        */
    L2Q_BACKSCATTER_1_ERROR = 25,     /**< Set if the backsctter
                                        *  value from retracker 1 is invalid.
                                        */
    L2Q_BACKSCATTER_2_ERROR = 24,     /**< Set if the backsctter
                                        *  value from retracker 2 is invalid.
                                        */
    L2Q_BACKSCATTER_3_ERROR = 23,     /**< Set if the backsctter
                                        *  value from retracker 3 is invalid.
                                        */
    L2Q_SSHA_INTERP_ERROR = 22,       /**< Set if an error occurred during
                                        *  interpolation of the sea-surface
                                        *  height anomaly.
                                        */
    L2Q_PEAKINESS_ERROR = 21,         /**< Set if the peakiness
                                        *  value is invalid.
                                        */
    L2Q_FREEBOARD_ERROR = 20,         /**< Set if the freeboard
                                        *  value is invalid.
                                        */
    L2Q_SAR_DISCRIM_OCEAN = 19,       /**< Set if the SAR
                                        *  discriminator determined
                                        *  the surface to be ocean.
                                        */
    L2Q_SAR_DISCRIM_LEAD = 18,        /**< Set if the SAR
                                        *  discriminator determined
                                        *  the surface to be a lead.
                                        */
    L2Q_SAR_DISCRIM_ICE = 17,         /**< Set if the SAR
                                        *  discriminato determined the
                                        *  surface to be ice.
                                        */
    L2Q_SAR_DISCRIM_UNKNOWN = 16,     /**< Set if the SAR
                                        *  discriminator could not
                                        *  determine the surface type.
                                        */
    L2Q_SARIN_XTRACK_ERROR = 15,      /**< Set if the SARin chain
                                        *  could not determine the
                                        *  cross-track angle.
                                        */
    L2Q_SARIN_RX1_ERROR = 14,         /**< Set if the Rx1 echo was bad. */
    L2Q_SARIN_RX2_ERROR = 13,         /**< Set if the Rx2 echo was bad. */
    L2Q_SARIN_SIDE_B = 12,            /**< Set if SIRAL is on the redundant side. */
    L2Q_SURFACE_MOD = 11,             /**< Set if no DEM/SLOPE model for the current location. */
    L2Q_MISPOINTING_ERROR = 10,       /**< Set if error during mispointing derivation. */
    L2Q_DELTA_TIME_ERROR = 9,         /**< Set if error during delta time calculation. */
    L2Q_LRM_SLOPE_INVALID = 8,        /**< Set if the LRM slope correction was invalid. */
    L2Q_SARIN_BASELINE_ERROR = 7,     /**< Set if the baseline vector was invalid. */
    L2Q_SARIN_RANGE_ERROR = 6,        /**< Set if the computed range was invalid. */
    L2Q_SARIN_VELOCITY_ERROR = 5,     /**< Set if the velocity vector was invalid. */
    L2Q_CALIBRATION_WARNING = 4       /**< Set if any
                                        *  calibration-related error
                                        *  condition was set.
                                        */

} t_Level2QualityStatusFlagBit;

/** Bit definitions for the L2 Corrections Status flag, see L2FinalC#uj_C_status */
typedef enum _t_Level2CorrectionsStatusFlagBit
{
    L2C_DRY_TROPO    = 31,              /**< Set if dry tropospheric
                                         *   correction error flag set
                                         *   at L1b or if correction
                                         *   not available at L1b.
                                         */
    L2C_WET_TROPO    = 30,              /**< Set if wet tropospheric
                                         *   correction error flag set
                                         *   at L1b or if correction
                                         *   not available at L1b.
                                         */
    L2C_INV_BARO     = 29,              /**< Set if inverse barometric
                                         *   correction error flag set
                                         *   at L1b or if correction
                                         *   not available at L1b.
                                         */
    L2C_DAC          = 28,              /**< Set if DAC
                                         *   correction error flag set
                                         *   at L1b or if correction
                                         *   not available at L1b.
                                         */
    L2C_IONO         = 27,              /**< Set if ionospheric
                                         *   correction error flag set
                                         *   at L1b or if correction
                                         *   not available at L1b.
                                         */
    L2C_SSB          = 26,              /**< Set if sea state bias
                                         *   value is invalid.
                                         */
    L2C_OCEAN_TIDE   = 25,              /**< Set if ocean tide
                                         *   correction error flag set
                                         *   at L1b or if correction
                                         *   not available at L1b.
                                         */
    L2C_LPEOT        = 24,              /**< Set if long period
                                         *   equilibrium ocean tide
                                         *   correction error flag set
                                         *   at L1b or if correction
                                         *   not available at L1b.
                                         */
    L2C_OLT          = 23,              /**< Set if ocean loading tide
                                         *   correction error flag set
                                         *   at L1b or if correction
                                         *   not available at L1b.
                                         */
    L2C_SET          = 22,              /**< Set if solid Earth tide
                                         *   correction error flag set
                                         *   at L1b or if correction
                                         *   not available at L1b.
                                         */
    L2C_GPT          = 21,              /**< Set if geocentric polar tide
                                         *   correction error flag set
                                         *   at L1b or if correction
                                         *   not available at L1b.
                                         */
    L2C_SURFACE_TYPE = 20,              /**< Set if surface type
                                         *   flag error flag set
                                         *   at L1b or if correction
                                         *   not available at L1b.
                                         */
    L2C_MSS_GEOID    = 19,              /**< Set if snow depth value is invalid.
                                         */
    L2C_ODLE_MODEL   = 18,              /**< Set if the ODLE model is invalid. */
    L2C_ICE_CONC     = 17,              /**< Set if the ice concentraion moedl is invalid. */

    L2C_SNOW_DEPTH   = 16,              /**< Set if dry tropospheric
                                         *   correction error flag set
                                         *   at L1b or if correction
                                         *   not available at L1b.
                                         */
    L2C_SNOW_DENSITY = 15,              /**< Set if the snow density value is invalid. */

    L2C_SWH          = 14,              /**< Set if significant wave
                                         * height value is invalid.
                                         */
    L2C_WINDSPEED    = 13               /**< Set if windspeed value is invalid.
                                         */
}
t_Level2CorrectionsStatusFlagBit;

/** Bit definitions for the L2 Corrections Application flag, see L2FinalM#uj_C_applied */
typedef enum t_Level2CorrectionsAppliedFlagBit {
    HGT_INTERNAL_PDS      = 31,         /**< Internal callibration. */
    HGT_RAD_DOPPLER_PDS   = 30,         /**< Radial doppler. */
    HGT_DRY_TROPO_PDS     = 29,         /**< Dry tropospheric. */
    HGT_WET_TROPO_PDS     = 28,         /**< Wet tropospheric. */
    HGT_INV_BARO_PDS      = 27,         /**< Inverse barometric. */
    HGT_DAC_PDS           = 26,         /**< Dynamic Atmospheric Correction. */
    HGT_IONO_GIM_PDS      = 25,         /**< GIM ionospheric. */
    HGT_IONO_MODEL_PDS    = 24,         /**< Model ionospheric. */
    HGT_OT_PDS            = 23,         /**< Ocean Tide. */
    HGT_LPEOT_PDS         = 22,         /**< LPEOT. */
    HGT_OLT_PDS           = 21,         /**< Ocean loading time. */
    HGT_SET_PDS           = 20,         /**< Solid Earth tide. */
    HGT_GPT_PDS           = 19,         /**< Geocentric Polar tide. */
    HGT_SLOPE_DOPPLER_PDS = 18,         /**< Slope Doppler. */
    HGT_WINDOW_OFFSET_PDS = 17,         /**< Mode specific window offset. */
    HGT_SAR_RETRK_PDS     = 16,         /**< SAR retracker offset. */
    HGT_SARIN_RETRK_PDS   = 15,         /**< SARIN retracker offset. */
    HGT_LRM_RETRK_PDS     = 14,         /**< LRM retracker offset. */
    HGT_LRM_OCEANBIAS_PDS = 13,         /**< LRM ocean bias applied. */
    HGT_LRM_ICEBIAS_PDS   = 12,         /**< LRM ice bias applied. */
    HGT_SAR_OCEANBIAS_PDS = 11,         /**< SAR ocean bias applied. */
    HGT_SAR_ICEBIAS_PDS   = 10,         /**< SAR ice bias applied. */
    HGT_SARIN_OCEANBIAS_PDS = 9,        /**< SARin ocean bias applied. */
    HGT_SARIN_ICEBIAS_PDS = 8,          /**< SARin ice bias applied. */
    LRM_SLOPE_DATA_VALID_PDS = 7,       /**< LRM slope model data valid. */
    HGT_SARIN_BAD_BASELINE_PDS = 6,     /**< SARin Bad Baseline downward. */
    HGT_SARIN_OUT_OF_RANGE_PDS = 5,     /**< SARin out of range. */
    HGT_SARIN_BAD_VELOCITY_PDS = 4,     /**< SARin Bad Velocity. */
    HGT_SSB_PDS                = 3      /**< SSB Applied */
}
t_Level2CorrectionsAppliedFlagBit;

/** 
 Dump an L2 product in ASCII to a file.

 @param pt Pointer to the product to dump.
 @param pf Pointer to the file to dump to.
*/
void vDump_L2Data( L2Data* pt, FILE* pf );

#endif /* _L2STRUCTURE_H */
