/*-- pre-processor statements for conditional code --*/

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+

    System: UCL Cryosat IO Library

    Sub-system: Cryosat Specific

    Description: Get the offset of a DS from the header of a file.


    ----------------------------------------------
    Documentation cross reference:


    ----------------------------------------------
    This file lives in ---
    ./mssl_cryosat

    Coding by:
    David J Brockley
    http://cryosat.mssl.ucl.ac.uk/public

    ----------------------------------------------
    Filename              :     $RCSfile: jCSGetDSFileOffsetFromHeader.c,v $
    CVS Version Number    :     $Revision: 1.3 $
    Last Update           :     $Date: 2015/06/02 11:07:43 $
    What String           :     @(#)$Id: jCSGetDSFileOffsetFromHeader.c,v 1.3 2015/06/02 11:07:43 djb Exp $
    Log                   :
                                $Log: jCSGetDSFileOffsetFromHeader.c,v $
                                Revision 1.3  2015/06/02 11:07:43  djb
                                Preparing for release 2.2

                                Revision 1.2  2014/12/01 11:30:14  djb
                                Pre-testing for release v2.1

                                Revision 1.1  2010/07/16 14:37:55  djb
                                Port in CryoSat tools from internal branch.



    ----------------------------------------------
    Routines:
    jCSGetDSFileOffsetFromHeader

    -----------------------------------------------------------
    This file contains the Unpublished Intellectual Property of
    University College London and All Rights Are Reserved.
        Copyright (c) University College London, 2005, 2010
    -----------------------------------------------------------
-
*/

/**
 * @file   jCSGetDSFileOffsetFromHeader.c
 *
 * @brief  Implementation of functions to extract the offset of the data set from the ASCII header.
 *
 *
 */

#pragma ident "@(#)$Id: jCSGetDSFileOffsetFromHeader.c,v 1.3 2015/06/02 11:07:43 djb Exp $ MSSL"

/*-- pre-processor statements --*/

/*-- standard C headers --*/
#include <stdlib.h>
#include <string.h>

/*-- programmer created headers --*/
#include "jCSGetDSFileOffsetFromHeader.h"
#include "sGetString.h"
#include "jGetLong.h"
#include "kGetLongLong.h"
#include "cGetChar.h"

/*-- constant and macro headers --*/

/*-- type definitions --*/

/*-- global and external variable declarations --*/

/*-- module private (static) variable declarations --*/
static long int jCheckDSDetails( char*    ac_buffer,
                                 long int j_DSD_Size );

/*-- functions --*/

long int
jCSGetDSFileOffsetFromHeader(fpos_t*    pt_dataset_file_offset,
                              long int*  pj_num_records,
                              char*      s_filename,
                              CS_DS_NAME t_ds_name,
                              long int*  pj_actual_ds )
{
    char*         ac_buffer      = NULL;
    char*         s_ds_name      = NULL;
    char*         p              = NULL;

    /* "SIR1LRM/4.1   " */
    char          ac_ver[15];

    short int     i              = 99;
    short int     j              = 99;
    short int     i_error        = 0;

    long int      j_exit_code    = EXIT_FAILURE;
    long int      j_sph_size     = 99;
    long int      j_matched      = FALSE;
    long int      j_result       = 0;
    long int      j_Num_DSDs     = 99;
    long int      j_DSD_Size     = 99;
    long int      j_SPH_preamble = 99;
    long int      j_num_dsr      = 0;

    long long int k_ds_offset    = 99;

    FILE*         fp;

    /* Attempt to open the file and handle error reporting */
    fp = fopen( s_filename, "rb" );

    if( fp == NULL )
    {
        return EXIT_FAILURE;
    } /* if fp NULL */

    /* Find the data and read in the MPH, but first, reserve memory for the MPH */
    ac_buffer = (char*)malloc( J_MPH_SIZE );

    if( ac_buffer == NULL )
    {
        return EXIT_FAILURE;
    }

    /* Read MPH from file */
    if( fread( ac_buffer, J_MPH_SIZE, 1, fp ) < (size_t)1 )
    {
        fclose( fp );
        free( ac_buffer );
        return EXIT_FAILURE;
    }

    /* Read the sph size from the mph */
    j_sph_size = jGetLong( "SPH_SIZE", ac_buffer, J_MPH_SIZE, &i_error );

    if( i_error )
    {
        fclose( fp );
        free( ac_buffer );
        return EXIT_FAILURE;
    }

    /* Read the number of DSDs  from the MPH */
    j_Num_DSDs = jGetLong( "NUM_DSD", ac_buffer, J_MPH_SIZE, &i_error );

    if( i_error )
    {
        fclose( fp );
        free( ac_buffer );
        return EXIT_FAILURE;
    }

    /* Read the DSD size from the MPH */
    j_DSD_Size = jGetLong( "DSD_SIZE", ac_buffer, J_MPH_SIZE, &i_error );

    if( i_error )
    {
        fclose( fp );
        free( ac_buffer );
        return EXIT_FAILURE;
    }

    /* Work out the size of the SPH preamble by subtracting off the sum of the DSD sizes */
    j_SPH_preamble = j_sph_size - ( j_Num_DSDs * j_DSD_Size );

    /* Read the SPH preamble and discard, but first reserve memory for the SPH preamble */
    ac_buffer = (char*)realloc( ac_buffer, j_SPH_preamble );

    if( ac_buffer == NULL )
    {
        fclose( fp );
        free( ac_buffer );
        return EXIT_FAILURE;
    }

    /* read preamble from file */

    if( fread( ac_buffer, j_SPH_preamble, 1, fp ) != 1 )
    {
        fclose( fp );
        free( ac_buffer );
        return EXIT_FAILURE;
    }

    for( i = 0; i < j_Num_DSDs; i++ )
    {
        ac_buffer = (char*)realloc( ac_buffer, j_DSD_Size );

        if( ac_buffer == NULL )
        {
            fclose( fp );
            free( ac_buffer );
            return EXIT_FAILURE;
        }

        if( fread( ac_buffer, j_DSD_Size, 1, fp ) != 1 )
        {
            fclose( fp );
            free( ac_buffer );
            return EXIT_FAILURE;
        }

        /* Check DS_NAME to see if this is a type that we want */
        s_ds_name = sGetString( "DS_NAME", ac_buffer, j_DSD_Size, &i_error );

        if( i_error )
        {
            fclose( fp );
            free( ac_buffer );
            return EXIT_FAILURE;
        }

        j_matched = FALSE;

        /* A specific DS name */
        if( ( t_ds_name != CS_L2_DS_NAME_ANY ) &&
           ( t_ds_name != CS_L2I_DS_NAME_ANY ) &&
           ( t_ds_name != CS_L1B_DS_NAME_ANY ) &&
           ( t_ds_name != CS_DS_NAME_ANY ) )
        {
            if( !strncmp( s_ds_name, as_CS_DS_NAME[ t_ds_name ],
                         strlen( as_CS_DS_NAME[ t_ds_name ] ) ) )
            {
                switch( jCheckDSDetails( ac_buffer, j_DSD_Size ) )
                {
                    case -1:
                    {
                        fclose( fp );
                        free( ac_buffer );
                        return EXIT_FAILURE;
                    } break;
                    case 1:
                    {
                        *pj_actual_ds = t_ds_name;
                        j_matched = TRUE;
                    } break;
                    case 0:
                    default:
                    {
                        j_matched = FALSE;
                    } break;
                } /* switch */
            }
            else
            {
                j_matched = FALSE;
            }
        }

        /* Anything or specifically L2 */
        if( ( t_ds_name == CS_L2_DS_NAME_ANY ) ||
           ( t_ds_name == CS_DS_NAME_ANY ) )
        {
            for( j = CS_L2_DS_NAME_START; j <= CS_L2_DS_NAME_END; j++ )
            {
                /* Skip this as we don't handle it yet */
                if( j == CS_L2_DS_NAME_FDM ) {continue; }

                if( !strncmp( s_ds_name, as_CS_DS_NAME[ j ],
                             strlen( as_CS_DS_NAME[ j ] ) ) )
                {
                    switch( jCheckDSDetails( ac_buffer, j_DSD_Size ) )
                    {
                        case -1:
                        {
                            fclose( fp );
                            free( ac_buffer );
                            return EXIT_FAILURE;
                        } break;
                        case 1:
                        {
                            j_matched = TRUE;
                        } break;
                        case 0:
                        default:
                        {
                            j_matched = FALSE;
                        } break;
                    } /* switch */

                    if( j_matched == TRUE )
                    {
                        *pj_actual_ds = CS_L2_DS_NAME_ANY;
                        break;
                    }
                }
            }
        }

        /* Anything or specifically L2I */
        if( ( j_matched != TRUE ) &&
           ( ( t_ds_name == CS_L2I_DS_NAME_ANY ) ||
            ( t_ds_name == CS_DS_NAME_ANY ) ) )
        {
            for( j = CS_L2I_DS_NAME_START; j <= CS_L2I_DS_NAME_END; j++ )
            {
                if( !strncmp( s_ds_name, as_CS_DS_NAME[ j ],
                             strlen( as_CS_DS_NAME[ j ] ) ) )
                {
                    switch( jCheckDSDetails( ac_buffer, j_DSD_Size ) )
                    {
                        case -1:
                        {
                            fclose( fp );
                            free( ac_buffer );
                            return EXIT_FAILURE;
                        } break;
                        case 1:
                        {
                            j_matched = TRUE;
                        } break;
                        case 0:
                        default:
                        {
                            j_matched = FALSE;
                        } break;
                    } /* switch */

                    if( j_matched == TRUE )
                    {
                        *pj_actual_ds = CS_L2I_DS_NAME_ANY;
                        break;
                    }
                }
            }
        }

        /* Anything or specifically L1B */
        if( ( j_matched != TRUE ) &&
           ( ( t_ds_name == CS_L1B_DS_NAME_ANY ) ||
            ( t_ds_name == CS_DS_NAME_ANY ) ) )
        {
            for( j = CS_L1B_DS_NAME_START; j <= CS_L1B_DS_NAME_END; j++ )
            {
                /* Skip this as we don't handle it yet */
                if( j == CS_L1B_DS_NAME_FDM ) {continue; }

                if( !strncmp( s_ds_name, as_CS_DS_NAME[ j ],
                             strlen( as_CS_DS_NAME[ j ] ) ) )
                {
                    switch( jCheckDSDetails( ac_buffer, j_DSD_Size ) )
                    {
                        case -1:
                        {
                            fclose( fp );
                            free( ac_buffer );
                            return EXIT_FAILURE;
                        } break;
                        case 1:
                        {
                            *pj_actual_ds = j;
                            j_matched = TRUE;
                        } break;
                        case 0:
                        default:
                        {
                            j_matched = FALSE;
                        } break;
                    } /* switch */

                    if( j_matched == TRUE )
                    {
                        break;
                    }
                }
            }
        }

        if( j_matched == TRUE )
        {
            /* Get offset of data set in file */
            k_ds_offset = kGetLongLong( "DS_OFFSET", ac_buffer, j_DSD_Size, &i_error );

            if( i_error )
            {
                fclose( fp );
                free( ac_buffer );
                return EXIT_FAILURE;
            }

            /* Get the number of records */
            j_num_dsr = jGetLong( "NUM_DSR", ac_buffer, j_DSD_Size, &i_error );

            if( i_error )
            {
                fclose( fp );
                free( ac_buffer );
                return EXIT_FAILURE;
            }

            if( pj_num_records != NULL )
            {
                *pj_num_records = j_num_dsr;
            }

            free( ac_buffer );

            if( pt_dataset_file_offset != NULL )
            {
                j_result = fseek( fp, k_ds_offset, SEEK_SET );

                if( j_result != 0 )
                {
                    fclose( fp );
                    return EXIT_FAILURE;
                }

                j_result = fgetpos( fp, pt_dataset_file_offset );

                if( j_result != 0 )
                {
                    fclose( fp );
                    return EXIT_FAILURE;
                }
            }

            fclose( fp );

            return EXIT_SUCCESS;
        }
    }

    /* Free any memory claimed */
    if( ac_buffer ) {free( ac_buffer ); }

    return j_exit_code;
} /* jCSGetDSFileOffsetFromHeader */

/** \brief Check that this is a measurement dataset

	\param  ac_buffer Pointer to ASCII buffer.
	\param  j_DSD_Size Size of buffer.
	\return -1 in the case of error, 0 if this is not a measurement dataset, 1 if this is a measurement dataset.
**/
static long int
jCheckDSDetails( char*    ac_buffer,
                 long int j_DSD_Size )
{
    char      c_ds_type = '\0';

    short int i_error   = 0;

    /* Check that the DS_TYPE is M */
    c_ds_type = cGetChar( "DS_TYPE", ac_buffer, j_DSD_Size, &i_error );

    if( i_error )
    {
        return -1;
    }

    if( c_ds_type != 'M' )
    {
        return 0;
    }

    return 1;
} /* jCheckDSDetails */
