/*-- pre-processor statements for conditional code --*/

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+

    System: UCL Cryosat IO Library

    Sub-system: Cryosat specific.

    Description:
    Read a L1b product from a file into a structure.

    ----------------------------------------------
    Documentation cross reference:
    [PROD-FMT] Product Format Specification v4.5 20/04/2009 CS-RS-ACS-GS-5106

    ----------------------------------------------
    This file lives in ---
    ./mssl_cryosat

    Coding by:
    David Brockley
    http://cryosat.mssl.ucl.ac.uk/public

    ----------------------------------------------
    Filename              :     $RCSfile: jFillL1bStructFromFile.c,v $
    CVS Version Number    :     $Revision: 1.5 $
    Last Update           :     $Date: 2015/06/02 11:07:43 $
    What String           :     @(#)$Id: jFillL1bStructFromFile.c,v 1.5 2015/06/02 11:07:43 djb Exp $
    Log                   :
                                $Log: jFillL1bStructFromFile.c,v $
                                Revision 1.5  2015/06/02 11:07:43  djb
                                Preparing for release 2.2

                                Revision 1.4  2014/12/01 11:30:14  djb
                                Pre-testing for release v2.1

                                Revision 1.3  2010/11/11 13:57:47  djb
                                Minor changes to types and naming to adhere to standards.

                                Revision 1.2  2010/10/13 08:12:51  djb
                                PRE_RELEASE_1_1

                                Revision 1.1  2010/07/16 14:37:56  djb
                                Port in CryoSat tools from internal branch.


    ----------------------------------------------
    Routines:
    jFillLRML1BStructFromFile()
    jFillSARL1BStructFromFile()
    jFillSARINL1BStructFromFile()

    ----------------------------------------------
    Copyright (c) University College London, 2005, 2010
    ----------------------------------------------
-
*/

/**
 * @file   jFillL1bStructFromFile.c
 *
 * @brief  Implementation of low-level functions to read Cryosat Level 1b data in LRM,
 * SAR, or SARin modes.
 *
 *
 */

#pragma ident "@(#)$Id: jFillL1bStructFromFile.c,v 1.5 2015/06/02 11:07:43 djb Exp $ MSSL"

/*-- pre-processor statements --*/

/*-- standard C headers --*/
#include <assert.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

/*-- programmer created headers --*/
#include "jFillL1bStructFromFile.h"
#include "L1bStructure.h"
#include "jIOFunctions.h"
#include "CS_Defines.h"

/*-- constant and macro headers --*/

/*-- type definitions --*/

/*-- global and external variable declarations --*/

/*-- module private (static) variable declarations --*/

/*-- functions --*/

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 +
    short int jFillLRML1bStructFromFile( L1B_LRM*, FILE* )

    Purpose of routine:  This routine reads LRM L1B data from a file into a
                         structure.

    -----------------------------------------
    Documentation cross reference

    -----------------------------------------

    Coding by:
    D. Brockley       Created: 26/04/2005     Version 1.1

    <author>       Modification: <date>    Version <version>
    <description>

    Reviewed by:
    <reviewer>     <date>  Version <version>

    Test by:
    <tester>               <date>  Version <version>
    -----------------------------------------
    Inputs:
    p_ToFill - pointer to the structure to populate
    pt_input - File pointer to read from

    Outputs: p_ToFill - populated

    Return: Status of processing
    -----------------------------------------
    Pre-condition:
    File open and at start of record. Pointer to valid structure.

    Post-condition:
    Structure filled with a copy of the information represented by
    the L1B file.

    -----------------------------------------
    Calling tree:
    Called by :
    Calls     :
    -----------------------------------------
    Variable documentation

    Global variables used:

    Global constants used:

    ---------------------------------------------------------------------------
    This file is the Unpublished Intellectual
    Property of University College London and
    All Rights are Reserved.

    Copyright (c) University College London, 2005
    ---------------------------------------------------------------------------
 -
 */
long int
jFillL1bLRMStructFromFile(L1B_LRM* p_ToFill,
                          FILE*    pt_input,
                          BASELINE j_Baseline)
{
    unsigned long int i;
    unsigned long int j;

    long int          j_error = 0;
    int32_t           j_Spare;

    static int bolt = 0;

    if( !bolt &&  gj_verbose )
    {
        printf( "Reading L1b LRM with baseline %d and nbins %d\n", (int)j_Baseline, J_LRM_WFM_BINS );
        bolt = 1;
    }
    /*
     * These variables will be used as the destination when a variable must
     * be extracted and discarded. They will also be used as intermediaries
     * when the source and destination varaibles are not of the same type.
     */

    assert( p_ToFill != NULL );

    if( p_ToFill == NULL )
    {
        return EXIT_FAILURE;
    }

    assert( pt_input != NULL );

    if( pt_input == NULL )
    {
        return EXIT_FAILURE;
    }

    /* i_HRB_count repeats of the Time and Orbit Group --------------------- */
    for( i = 0; i < J_NUM_HRB; i++ )
    {
        /* 1 Data Record Time */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Time_Orbit[ i ].j_Day ) );

        if( j_error != EXIT_SUCCESS )
        {
            if( feof( pt_input ) )
            {
                printf( "EOF detected on first read of L1b. OK." );
                return EXIT_SUCCESS;
            }
            else
            {
                printf( "Error while reading L1b j_Day" );
                return EXIT_FAILURE;
            }
        }

        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Time_Orbit[ i ].uj_Sec ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b uj_Sec" );
            return EXIT_FAILURE;
        }

        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Time_Orbit[ i ].uj_Micsec ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b uj_Micsec" );
            return EXIT_FAILURE;
        }

        /* 2 USO Correction Factor */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Time_Orbit[ i ].j_USO_Corr ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_USO_Corr" );
            return EXIT_FAILURE;
        }

        /* 3 Mode ID */
        j_error = jReadShortInt( pt_input,
                                &( p_ToFill->at_Time_Orbit[ i ].ui_Mode_ID ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b ui_Mode_ID" );
            return EXIT_FAILURE;
        }

        /* 4 Source Sequence Counter */
        j_error = jReadShortInt( pt_input,
                                &( p_ToFill->at_Time_Orbit[ i ].ui_SSC ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b ui_SSC" );
            return EXIT_FAILURE;
        }

        /* 5 Instrument Config */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Time_Orbit[ i ].uj_Inst_config ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b uj_Inst_config" );
            return EXIT_FAILURE;
        }

        /* 6 Surface Sample Counter */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Time_Orbit[ i ].uj_Rec_Count ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b uj_Rec_Count" );
            return EXIT_FAILURE;
        }

        /* 7 Lattitude of Measurement */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Time_Orbit[ i ].j_Lat ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_Lat" );
            return EXIT_FAILURE;
        }

        /* 8 Longitude of Measurement */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Time_Orbit[ i ].j_Lon ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_Lon" );
            return EXIT_FAILURE;
        }

        /* 9 Altitude of COG above reference ellipsoid */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Time_Orbit[ i ].j_Alt ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_Alt" );
            return EXIT_FAILURE;
        }

        /* 10 Instantaneous altitude rate derived from orbit */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Time_Orbit[ i ].j_Alt_rate ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_Alt_rate" );
            return EXIT_FAILURE;
        }

        /* 11 Satellite velocity vector in IERF */
        for( j = 0; j < 3; j++ )
        {
            j_error = jReadLongInt( pt_input,
                                   &( p_ToFill->at_Time_Orbit[ i ].
                                          aj_Sat_velocity[ j ] ) );

            if( j_error != EXIT_SUCCESS )
            {
                printf( "Error while reading L1b aj_Sat_velocity" );
                return EXIT_FAILURE;
            }
        }

        /* 12 Real beam direction vector in CRF */
        for( j = 0; j < 3; j++ )
        {
            j_error = jReadLongInt( pt_input,
                                   &( p_ToFill->at_Time_Orbit[ i ].
                                          aj_Real_beam[ j ] ) );

            if( j_error != EXIT_SUCCESS )
            {
                printf( "Error while reading L1b aj_Real_beam" );
                return EXIT_FAILURE;
            }
        }

        /* 13 Interferometer baseline vector in CRF */
        for( j = 0; j < 3; j++ )
        {
            j_error = jReadLongInt( pt_input,
                                   &( p_ToFill->at_Time_Orbit[ i ].
                                          aj_Baseline[ j ] ) );

            if( j_error != EXIT_SUCCESS )
            {
                printf( "Error while reading L1b aj_Baseline" );
                return EXIT_FAILURE;
            }
        }

        if( j_Baseline >= BASELINE_BC )
        {
            j_error = jReadShortInt( pt_input,
                                     &( p_ToFill->at_Time_Orbit[ i ].
                                        i_ST_ID ) );

            if( j_error != EXIT_SUCCESS )
            {
                printf( "Error while reading L1b i_ST_ID" );
                return EXIT_FAILURE;
            }

            j_error = jReadLongInt( pt_input,
                                   &( p_ToFill->at_Time_Orbit[ i ].
                                          j_Roll_angle ) );

            if( j_error != EXIT_SUCCESS )
            {
                printf( "Error while reading L1b j_Roll_angle" );
                return EXIT_FAILURE;
            }

            j_error = jReadLongInt( pt_input,
                                   &( p_ToFill->at_Time_Orbit[ i ].
                                          j_Pitch_angle ) );

            if( j_error != EXIT_SUCCESS )
            {
                printf( "Error while reading L1b j_Pitch_angle" );
                return EXIT_FAILURE;
            }

            j_error = jReadLongInt( pt_input,
                                   &( p_ToFill->at_Time_Orbit[ i ].
                                          j_Yaw_angle ) );

            if( j_error != EXIT_SUCCESS )
            {
                printf( "Error while reading L1b j_Yaw_angle" );
                return EXIT_FAILURE;
            }
        }
        /* 13 Level 1B Measurement Confidence Data */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Time_Orbit[ i ].uj_MCD ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b uj_MCD" );
            return EXIT_FAILURE;
        }

        if( j_Baseline >= BASELINE_BC )
        {
            j_error = jReadLongInt( pt_input,
                                    &j_Spare );

            if( j_error != EXIT_SUCCESS )
            {
                printf( "Error while reading L1b j_Spare" );
                return EXIT_FAILURE;
            }
        }
    } /* end for( i = 0; i < J_NUM_HRB; i++ ) Time and Orbit Group */

    /* J_NUM_HRB repeats of the Measurements Group ----------------------- */
    for( i = 0; i < J_NUM_HRB; i++ )
    {
        /* 15 Window delay */
        j_error = jReadLongLongInt( pt_input,
                                   &( p_ToFill->at_Measurements[ i ].
                                          k_TD ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b k_TD" );
            return EXIT_FAILURE;
        }

        /* 16 Initial Height Word */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Measurements[ i ].
                                      j_H0 ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_H0" );
            return EXIT_FAILURE;
        }

        /* 17 COR2 Height rate per radar cycle */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Measurements[ i ].
                                      j_COR2 ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_COR2" );
            return EXIT_FAILURE;
        }

        /* 18 Coarse Range Word LAI */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_LAI ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_LAI" );
            return EXIT_FAILURE;
        }

        /* 19 Fine Range Word FAI */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_FAI ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_FAI" );
            return EXIT_FAILURE;
        }

        /* 20 AGC ch 1 corrected */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_AGC_ch1 ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_AGC_ch1" );
            return EXIT_FAILURE;
        }

        /* 21 AGC ch 2 corrected */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_AGC_ch2 ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_AGC_ch2" );
            return EXIT_FAILURE;
        }

        /* 22 Overall Tx-Rx gain 1 */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_TR_gain_ch1 ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_TR_gain_ch1" );
            return EXIT_FAILURE;
        }

        /* 23 Overall Tx-Rx gain 2 */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_TR_gain_ch2 ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_TR_gain_ch2" );
            return EXIT_FAILURE;
        }

        /* 24 Transmit power micro-watts */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_Tx_power ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_Tx_power" );
            return EXIT_FAILURE;
        }

        /* Internal Geo_Corrections -------------------------------------------- */

        /* 25 Doppler range correction (radial component) */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_Dopp_rc ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_Dopp_rc" );
            return EXIT_FAILURE;
        }

        /* 26 Instrument range correction t-r antenna */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_TR_inst_rc ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_TR_inst_rc" );
            return EXIT_FAILURE;
        }

        /* 27 Instrument range correction r only antenna */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_R_inst_rc ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_R_inst_rc" );
            return EXIT_FAILURE;
        }

        /* 28 Instrument sigma 0 correction t-r antenna */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_TR_inst_gain_C ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_TR_inst_gain_C" );
            return EXIT_FAILURE;
        }

        /* 29 Instrument sigma 0 correction r antenna */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_R_inst_gain_C ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_R_inst_gain_C" );
            return EXIT_FAILURE;
        }

        /* 30 Internal Phase Correction */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_Int_phase_C ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_Int_phase_C" );
            return EXIT_FAILURE;
        }

        /* 31 External Phase Correction */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_Ext_phase_C ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_Ext_phase_C" );
            return EXIT_FAILURE;
        }

        /* 32 Noise power measurement */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_Noise_pwr ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_Noise_pwr" );
            return EXIT_FAILURE;
        }

        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
            j_Phase_slope_C ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_Phase_slope_C" );
            return EXIT_FAILURE;
        }

        /* 33 Skip 4 bytes of spares */
        for( j = 0; j < 4; j++ )
        {
            j_error = jReadChar( pt_input, &(p_ToFill->at_Measurements[i].spares[j]) );

            if( j_error != EXIT_SUCCESS )
            {
                printf( "Error while reading L1b spares" );
                return EXIT_FAILURE;
            }
        } /* end for( j = 0; j < 4; j++ ) Spares */
    } /* end for( i = 0; i < J_NUM_HRB; i++ ) Measurements Group */

    /* Geo_Corrections Group --------------------------------------------------- */

    /* 34 Dry Tropospheric Correction */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.j_Dry_C ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_Dry_C" );
        return EXIT_FAILURE;
    }

    /* 35 Wet Tropospheric Correction */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.j_Wet_C ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_Wet_C" );
        return EXIT_FAILURE;
    }

    /* 36 Inverse Barometric Correction */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.j_IB_C ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_IB_C" );
        return EXIT_FAILURE;
    }

    /* 36 DAC Correction */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.j_DAC_C ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_DAC_C" );
        return EXIT_FAILURE;
    }

    /* 37 GIM Ionospheric Correction */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.j_Iono_GIM ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_Iono_GIM" );
        return EXIT_FAILURE;
    }

    /* 38 Model Ionospheric Correction */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.j_Iono_mod ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_Iono_C_mod" );
        return EXIT_FAILURE;
    }

    /* 39 Ocean Tide 1 */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.j_H_OT ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_H_OT" );
        return EXIT_FAILURE;
    }

    /* 40 Ocean Tide 2 */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.j_H_LPEOT) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_H_LPEOT" );
        return EXIT_FAILURE;
    }

    /* 41 Ocean Loading Tide */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.j_H_OLT ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_H_OLT" );
        return EXIT_FAILURE;
    }

    /* 42 Solid Earth Tide */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.j_H_SET ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_H_SET" );
        return EXIT_FAILURE;
    }

    /* 43 Geocentric Polar Tide */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.j_H_GPT ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_H_GPT" );
        return EXIT_FAILURE;
    }

    /* 44 Surface Type Flag */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.uj_Surf_type ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b uj_Surf_type" );
        return EXIT_FAILURE;
    }

    /* 47 Skip 4 bytes of spares */
    for( i = 0; i < 4; i++ )
    {
        j_error = jReadChar( pt_input, &(p_ToFill->t_Geo_Corrections.spare1[i]) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b spares" );
            return EXIT_FAILURE;
        }
    } /* end for( i = 0; i < 4; i++ ) */

    /* 45 Correction Status Flags */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.uj_Corr_status ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b uj_Corr_status" );
        return EXIT_FAILURE;
    }

    /* 46 Correction Error Flags */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.
                                  uj_Corr_error ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b uj_Corr_error" );
        return EXIT_FAILURE;
    }

    /* 47 Skip 4 bytes of spares */
    for( i = 0; i < 4; i++ )
    {
        j_error = jReadChar( pt_input, &p_ToFill->t_Geo_Corrections.spare2[i] );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b spares" );
            return EXIT_FAILURE;
        }
    } /* end for( i = 0; i < 4; i++ ) */

    /* Average Waveforms Group --------------------------------------------- */

    /* 48 Echo Time */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Average_PL.
                                  j_Day_1Hz ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_Day_1Hz" );
        return EXIT_FAILURE;
    }

    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Average_PL.
                                  uj_Sec_1Hz ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b uj_Sec_1Hz" );
        return EXIT_FAILURE;
    }

    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Average_PL.
                                  uj_Micsec_1Hz ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b uj_Micsec_1Hz" );
        return EXIT_FAILURE;
    }

    /* 49 Latitude of Measurement */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Average_PL.
                                  j_Lat_1Hz ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_Lat_1Hz" );
        return EXIT_FAILURE;
    }

    /* 50 Longitude of Measurement */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Average_PL.
                                  j_Lon_1Hz ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_Lon_1Hz" );
        return EXIT_FAILURE;
    }

    /* 51 Altitude of COG above reference ellipsoid */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Average_PL.
                                  j_Alt_1Hz ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_Alt_1Hz" );
        return EXIT_FAILURE;
    }

    /* 52 Window Delay */
    j_error = jReadLongLongInt( pt_input,
                               &( p_ToFill->t_Average_PL.
                                      k_TD_1Hz ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b k_TD_1Hz" );
        return EXIT_FAILURE;
    }

    /* 53 Averaged power echo waveform */
    for( i = 0; i < 128; i++ )
    {
        j_error = jReadShortInt( pt_input,
                                &( p_ToFill->t_Average_PL.
                                       pui_avg_wfm[ i ] ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b aui_avg_wfm" );
            return EXIT_FAILURE;
        }
    } /* end for( i = 0; i < J_LRM_WFM_BINS; i++ ) Waveform read */

    /* 54 Echo scale factor */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Average_PL.
                                  j_Linear_wfm_mult ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_Linear_wfm_mult" );
        return EXIT_FAILURE;
    }

    /* 55 Echo scale power */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Average_PL.
                                  j_Power2_wfm_mult ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_Power2_wfm_mult" );
        return EXIT_FAILURE;
    }

    /* 56 Number of echoes averaged */
    j_error = jReadShortInt( pt_input,
                            &( p_ToFill->t_Average_PL.
                                   ui_num_avg_echoes ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b ui_num_avg_echoes" );
        return EXIT_FAILURE;
    }

    /* 57 Flags */
    j_error = jReadShortInt( pt_input,
                            &( p_ToFill->t_Average_PL.
                                   ui_flags ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b ui_flags" );
        return EXIT_FAILURE;
    }

    /* Waveform Group ------------------------------------------------------ */
    for( i = 0; i < J_NUM_HRB; i++ )
    {
        /* 68 Averaged Power Echo Waveform */
        for( j = 0; j < J_LRM_WFM_BINS; j++ )
        {
            j_error = jReadShortInt( pt_input,
                                    &( p_ToFill->
                                           at_Waveforms[ i ].
                                           pui_wfm[ j ] ) );

            if( j_error != EXIT_SUCCESS )
            {
                printf( "Error while reading L1b aui_wfm" );
                return EXIT_FAILURE;
            }
        } /* for( j = 0; j < J_LRM_WFM_BINS; j++ ) Waveform */

        /* 69 Echo scale factor */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Waveforms[ i ].
                                      j_Linear_wfm_mult ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_Linear_wfm_mult" );
            return EXIT_FAILURE;
        }

        /* 70 Echo scale power */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Waveforms[ i ].
                                      j_Power2_wfm_mult ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_Power2_wfm_mult" );
            return EXIT_FAILURE;
        }

        /* 71 Number of echoes averaged */
        j_error = jReadShortInt( pt_input,
                                &( p_ToFill->at_Waveforms[ i ].
                                       ui_num_avg_echoes ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b ui_num_avg_echoes" );
            return EXIT_FAILURE;
        }

        /* 72 Flags */
        j_error = jReadShortInt( pt_input,
                                &( p_ToFill->at_Waveforms[ i ].
                                       ui_flags ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b ui_wfm_flag" );
            return EXIT_FAILURE;
        }
    } /* end for( i = 0; i < J_NUM_HRB; i++ ) for Waveform Group */

    return EXIT_SUCCESS;
} /* jFillL1bLRMStructFromFile */

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 +
 short int jFillSARL1bStructFromFile( L1B_LRM*, FILE* )

 Purpose of routine:  This routine reads LRM L1B data from a file into a
                      structure.

 -----------------------------------------
 Documentation cross reference

 -----------------------------------------

 Coding by:
 D. Brockley       Created: 26/04/2005     Version 1.1

 <author>       Modification: <date>    Version <version>
 <description>

 Reviewed by:
 <reviewer>     <date>  Version <version>

 Test by:
 <tester>               <date>  Version <version>
 -----------------------------------------
 Inputs:
 p_ToFill - pointer to the structure to populate
 pt_input - File pointer to read from

 Outputs: p_ToFill - populated

 Return: Status of processing
 -----------------------------------------
 Pre-condition:
 File open and at start of record. Pointer to valid structure.

 Post-condition:
 Structure filled with a copy of the information represented by
 the L1B file.

 -----------------------------------------
 Calling tree:
 Called by :
 Calls     :
 -----------------------------------------
 Variable documentation

 Global variables used:

 Global constants used:

 -----------------------------------------
 Copyright University College London
 --------------------------------------------------------------------------
 -
 */
long int
jFillL1bSARStructFromFile(L1B_SAR* p_ToFill,
                           FILE*    pt_input,
                          BASELINE j_Baseline)
{
    unsigned long int i;
    unsigned long int j;

    long int          j_error = 0;
    int32_t           j_Spare;

    static int bolt = 0;

    if( !bolt &&  gj_verbose )
    {
        printf( "Reading L1b SAR with baseline %d and nbins %d\n", (int)j_Baseline, J_SAR_WFM_BINS );
        bolt = 1;
    }

    /*
     * These variables will be used as the destination when a variable must
     * be extracted and discarded. They will also be used as intermediaries
     * when the source and destination varaibles are not of the same type.
     */

    assert( p_ToFill != NULL );

    if( p_ToFill == NULL )
    {
        return EXIT_FAILURE;
    }

    assert( pt_input != NULL );

    if( pt_input == NULL )
    {
        return EXIT_FAILURE;
    }

    /* i_HRB_count repeats of the Time and Orbit Group --------------------- */
    for( i = 0; i < J_NUM_HRB; i++ )
    {
        /* 1 Data Record Time */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Time_Orbit[ i ].j_Day ) );

        if( j_error != EXIT_SUCCESS )
        {
            if( feof( pt_input ) )
            {
                printf( "EOF detected on first read of L1b. OK." );
                return EXIT_SUCCESS;
            }
            else
            {
                printf( "Error while reading L1b j_Day" );
                return EXIT_FAILURE;
            }
        }

        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Time_Orbit[ i ].uj_Sec ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b uj_Sec" );
            return EXIT_FAILURE;
        }

        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Time_Orbit[ i ].uj_Micsec ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b uj_Micsec" );
            return EXIT_FAILURE;
        }

        /* 2 USO Correction Factor */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Time_Orbit[ i ].j_USO_Corr ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_USO_Corr" );
            return EXIT_FAILURE;
        }

        /* 3 Mode ID */
        j_error = jReadShortInt( pt_input,
                                &( p_ToFill->at_Time_Orbit[ i ].ui_Mode_ID ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b ui_Mode_ID" );
            return EXIT_FAILURE;
        }

        /* 4 Source Sequence Counter */
        j_error = jReadShortInt( pt_input,
                                &( p_ToFill->at_Time_Orbit[ i ].ui_SSC ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b ui_SSC" );
            return EXIT_FAILURE;
        }

        /* 5 Instrument Config */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Time_Orbit[ i ].uj_Inst_config ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b uj_Inst_config" );
            return EXIT_FAILURE;
        }

        /* 6 Surface Sample Counter */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Time_Orbit[ i ].uj_Rec_Count ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b uj_Rec_Count" );
            return EXIT_FAILURE;
        }

        /* 7 Lattitude of Measurement */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Time_Orbit[ i ].j_Lat ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_Lat" );
            return EXIT_FAILURE;
        }

        /* 8 Longitude of Measurement */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Time_Orbit[ i ].j_Lon ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_Lon" );
            return EXIT_FAILURE;
        }

        /* 9 Altitude of COG above reference ellipsoid */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Time_Orbit[ i ].j_Alt ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_Alt" );
            return EXIT_FAILURE;
        }

        /* 10 Instantaneous altitude rate derived from orbit */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Time_Orbit[ i ].j_Alt_rate ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_Alt_rate" );
            return EXIT_FAILURE;
        }

        /* 11 Satellite velocity vector in IERF */
        for( j = 0; j < 3; j++ )
        {
            j_error = jReadLongInt( pt_input,
                                   &( p_ToFill->at_Time_Orbit[ i ].
                                          aj_Sat_velocity[ j ] ) );

            if( j_error != EXIT_SUCCESS )
            {
                printf( "Error while reading L1b aj_Sat_velocity" );
                return EXIT_FAILURE;
            }
        }

        /* 12 Real beam direction vector in CRF */
        for( j = 0; j < 3; j++ )
        {
            j_error = jReadLongInt( pt_input,
                                   &( p_ToFill->at_Time_Orbit[ i ].
                                          aj_Real_beam[ j ] ) );

            if( j_error != EXIT_SUCCESS )
            {
                printf( "Error while reading L1b aj_Real_beam" );
                return EXIT_FAILURE;
            }
        }

        /* 13 Interferometer baseline vector in CRF */
        for( j = 0; j < 3; j++ )
        {
            j_error = jReadLongInt( pt_input,
                                   &( p_ToFill->at_Time_Orbit[ i ].
                                          aj_Baseline[ j ] ) );

            if( j_error != EXIT_SUCCESS )
            {
                printf( "Error while reading L1b aj_Baseline" );
                return EXIT_FAILURE;
            }
        }

        if( j_Baseline >= BASELINE_BC )
        {
            j_error = jReadShortInt( pt_input,
                                     &( p_ToFill->at_Time_Orbit[ i ].
                                        i_ST_ID ) );

            if( j_error != EXIT_SUCCESS )
            {
                printf( "Error while reading L1b i_ST_ID" );
                return EXIT_FAILURE;
            }

            j_error = jReadLongInt( pt_input,
                                   &( p_ToFill->at_Time_Orbit[ i ].
                                          j_Roll_angle ) );

            if( j_error != EXIT_SUCCESS )
            {
                printf( "Error while reading L1b j_Roll_angle" );
                return EXIT_FAILURE;
            }

            j_error = jReadLongInt( pt_input,
                                   &( p_ToFill->at_Time_Orbit[ i ].
                                          j_Pitch_angle ) );

            if( j_error != EXIT_SUCCESS )
            {
                printf( "Error while reading L1b j_Pitch_angle" );
                return EXIT_FAILURE;
            }

            j_error = jReadLongInt( pt_input,
                                   &( p_ToFill->at_Time_Orbit[ i ].
                                          j_Yaw_angle ) );

            if( j_error != EXIT_SUCCESS )
            {
                printf( "Error while reading L1b j_Yaw_angle" );
                return EXIT_FAILURE;
            }
        }
        /* 13 Level 1B Measurement Confidence Data */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Time_Orbit[ i ].uj_MCD ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b uj_MCD" );
            return EXIT_FAILURE;
        }

        if( j_Baseline >= BASELINE_BC )
        {
            j_error = jReadLongInt( pt_input,
                                    &j_Spare );

            if( j_error != EXIT_SUCCESS )
            {
                printf( "Error while reading L1b j_Spare" );
                return EXIT_FAILURE;
            }
        }
    } /* end for( i = 0; i < J_NUM_HRB; i++ ) Time and Orbit Group */

    /* J_NUM_HRB repeats of the Measurements Group ----------------------- */
    for( i = 0; i < J_NUM_HRB; i++ )
    {
        /* 15 Window delay */
        j_error = jReadLongLongInt( pt_input,
                                   &( p_ToFill->at_Measurements[ i ].
                                          k_TD ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b k_TD" );
            return EXIT_FAILURE;
        }

        /* 16 Initial Height Word */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Measurements[ i ].
                                      j_H0 ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_H0" );
            return EXIT_FAILURE;
        }

        /* 17 COR2 Height rate per radar cycle */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Measurements[ i ].
                                      j_COR2 ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_COR2" );
            return EXIT_FAILURE;
        }

        /* 18 Coarse Range Word LAI */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_LAI ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_LAI" );
            return EXIT_FAILURE;
        }

        /* 19 Fine Range Word FAI */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_FAI ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_FAI" );
            return EXIT_FAILURE;
        }

        /* 20 AGC ch 1 corrected */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_AGC_ch1 ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_AGC_ch1" );
            return EXIT_FAILURE;
        }

        /* 21 AGC ch 2 corrected */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_AGC_ch2 ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_AGC_ch2" );
            return EXIT_FAILURE;
        }

        /* 22 Overall Tx-Rx gain 1 */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_TR_gain_ch1 ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_TR_gain_ch1" );
            return EXIT_FAILURE;
        }

        /* 23 Overall Tx-Rx gain 2 */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_TR_gain_ch2 ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_TR_gain_ch2" );
            return EXIT_FAILURE;
        }

        /* 24 Transmit power micro-watts */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_Tx_power ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_Tx_power" );
            return EXIT_FAILURE;
        }

        /* Internal Geo_Corrections -------------------------------------------- */

        /* 25 Doppler range correction (radial component) */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_Dopp_rc ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_Dopp_rc" );
            return EXIT_FAILURE;
        }

        /* 26 Instrument range correction t-r antenna */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_TR_inst_rc ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_TR_inst_rc" );
            return EXIT_FAILURE;
        }

        /* 27 Instrument range correction r only antenna */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_R_inst_rc ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_R_inst_rc" );
            return EXIT_FAILURE;
        }

        /* 28 Instrument sigma 0 correction t-r antenna */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_TR_inst_gain_C ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_TR_inst_gain_C" );
            return EXIT_FAILURE;
        }

        /* 29 Instrument sigma 0 correction r antenna */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_R_inst_gain_C ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_R_inst_gain_C" );
            return EXIT_FAILURE;
        }

        /* 30 Internal Phase Correction */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_Int_phase_C ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_Int_phase_C" );
            return EXIT_FAILURE;
        }

        /* 31 External Phase Correction */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_Ext_phase_C ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_Ext_phase_C" );
            return EXIT_FAILURE;
        }

        /* 32 Noise power measurement */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_Noise_pwr ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_Noise_pwr" );
            return EXIT_FAILURE;
        }

        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
            j_Phase_slope_C ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_Phase_slope_C" );
            return EXIT_FAILURE;
        }

        /* 33 Skip 4 bytes of spares */
        for( j = 0; j < 4; j++ )
        {
            j_error = jReadChar( pt_input, &(p_ToFill->at_Measurements[i].spares[j]) );

            if( j_error != EXIT_SUCCESS )
            {
                printf( "Error while reading L1b spares" );
                return EXIT_FAILURE;
            }
        } /* end for( j = 0; j < 4; j++ ) Spares */
    } /* end for( i = 0; i < J_NUM_HRB; i++ ) Measurements Group */

    /* Geo_Corrections Group --------------------------------------------------- */

    /* 34 Dry Tropospheric Correction */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.j_Dry_C ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_Dry_C" );
        return EXIT_FAILURE;
    }

    /* 35 Wet Tropospheric Correction */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.j_Wet_C ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_Wet_C" );
        return EXIT_FAILURE;
    }

    /* 36 Inverse Barometric Correction */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.j_IB_C ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_IB_C" );
        return EXIT_FAILURE;
    }

    /* 36 DAC Correction */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.j_DAC_C ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_DAC_C" );
        return EXIT_FAILURE;
    }

    /* 37 GIM Ionospheric Correction */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.j_Iono_GIM ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_Iono_GIM" );
        return EXIT_FAILURE;
    }

    /* 38 Model Ionospheric Correction */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.j_Iono_mod ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_Iono_C_mod" );
        return EXIT_FAILURE;
    }

    /* 39 Ocean Tide 1 */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.j_H_OT ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_H_OT" );
        return EXIT_FAILURE;
    }

    /* 40 Ocean Tide 2 */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.j_H_LPEOT) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_H_LPEOT" );
        return EXIT_FAILURE;
    }

    /* 41 Ocean Loading Tide */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.j_H_OLT ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_H_OLT" );
        return EXIT_FAILURE;
    }

    /* 42 Solid Earth Tide */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.j_H_SET ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_H_SET" );
        return EXIT_FAILURE;
    }

    /* 43 Geocentric Polar Tide */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.j_H_GPT ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_H_GPT" );
        return EXIT_FAILURE;
    }

    /* 44 Surface Type Flag */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.uj_Surf_type ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b uj_Surf_type" );
        return EXIT_FAILURE;
    }

    /* 47 Skip 4 bytes of spares */
    for( i = 0; i < 4; i++ )
    {
        j_error = jReadChar( pt_input, &(p_ToFill->t_Geo_Corrections.spare1[i]) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b spares" );
            return EXIT_FAILURE;
        }
    } /* end for( i = 0; i < 4; i++ ) */

    /* 45 Correction Status Flags */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.uj_Corr_status ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b uj_Corr_status" );
        return EXIT_FAILURE;
    }

    /* 46 Correction Error Flags */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.
                                  uj_Corr_error ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b uj_Corr_error" );
        return EXIT_FAILURE;
    }

    /* 47 Skip 4 bytes of spares */
    for( i = 0; i < 4; i++ )
    {
        j_error = jReadChar( pt_input, &(p_ToFill->t_Geo_Corrections.spare2[i]) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b spares" );
            return EXIT_FAILURE;
        }
    } /* end for( i = 0; i < 4; i++ ) */

    /* Average Waveforms Group --------------------------------------------- */

    /* 48 Echo Time */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Average_PL.
                                  j_Day_1Hz ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_Day_1Hz" );
        return EXIT_FAILURE;
    }

    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Average_PL.
                                  uj_Sec_1Hz ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b uj_Sec_1Hz" );
        return EXIT_FAILURE;
    }

    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Average_PL.
                                  uj_Micsec_1Hz ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b uj_Micsec_1Hz" );
        return EXIT_FAILURE;
    }

    /* 49 Latitude of Measurement */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Average_PL.
                                  j_Lat_1Hz ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_Lat_1Hz" );
        return EXIT_FAILURE;
    }

    /* 50 Longitude of Measurement */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Average_PL.
                                  j_Lon_1Hz ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_Lon_1Hz" );
        return EXIT_FAILURE;
    }

    /* 51 Altitude of COG above reference ellipsoid */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Average_PL.
                                  j_Alt_1Hz ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_Alt_1Hz" );
        return EXIT_FAILURE;
    }

    /* 52 Window Delay */
    j_error = jReadLongLongInt( pt_input,
                               &( p_ToFill->t_Average_PL.
                                      k_TD_1Hz ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b k_TD_1Hz" );
        return EXIT_FAILURE;
    }

    /* 53 Averaged power echo waveform */
    for( i = 0; i < 128; i++ )
    {
        j_error = jReadShortInt( pt_input,
                                &( p_ToFill->t_Average_PL.
                                       pui_avg_wfm[ i ] ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b aui_avg_wfm" );
            return EXIT_FAILURE;
        }
    } /* end for( i = 0; i < J_SAR_WFM_BINS; i++ ) Waveform read */

    /* 54 Echo scale factor */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Average_PL.
                                  j_Linear_wfm_mult ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_Linear_wfm_mult" );
        return EXIT_FAILURE;
    }

    /* 55 Echo scale power */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Average_PL.
                                  j_Power2_wfm_mult ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_Power2_wfm_mult" );
        return EXIT_FAILURE;
    }

    /* 56 Number of echoes averaged */
    j_error = jReadShortInt( pt_input,
                            &( p_ToFill->t_Average_PL.
                                   ui_num_avg_echoes ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b ui_num_avg_echoes" );
        return EXIT_FAILURE;
    }

    /* 57 Flags */
    j_error = jReadShortInt( pt_input,
                            &( p_ToFill->t_Average_PL.
                                   ui_flags ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b ui_flags" );
        return EXIT_FAILURE;
    }

    /* Waveform Group ------------------------------------------------------ */
    for( i = 0; i < J_NUM_HRB; i++ )
    {
        /* 73 Averaged Power Echo Waveform */
        for( j = 0; j < J_SAR_WFM_BINS; j++ )
        {
            j_error = jReadShortInt( pt_input,
                                    &( p_ToFill->
                                           at_Waveforms[ i ].
                                           pui_wfm[ j ] ) );

            if( j_error != EXIT_SUCCESS )
            {
                printf( "Error while reading L1b aui_wfm" );
                return EXIT_FAILURE;
            }
        } /* for( j = 0; j < J_SAR_WFM_BINS; j++ ) Waveform */

        /* 74 Echo scale factor */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Waveforms[ i ].
                                      j_Linear_wfm_mult ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_Linear_wfm_mult" );
            return EXIT_FAILURE;
        }

        /* 75 Echo scale power */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Waveforms[ i ].
                                      j_Power2_wfm_mult ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_Power2_wfm_mult" );
            return EXIT_FAILURE;
        }

        /* 76 Number of echoes averaged */
        j_error = jReadShortInt( pt_input,
                                &( p_ToFill->at_Waveforms[ i ].
                                       ui_num_avg_echoes ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b ui_num_avg_echoes" );
            return EXIT_FAILURE;
        }

        /* 77 Flags */
        j_error = jReadShortInt( pt_input,
                                &( p_ToFill->at_Waveforms[ i ].
                                       ui_flags ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b ui_flags" );
            return EXIT_FAILURE;
        }

        /* 78 Beam Behaviour Parameter */
        for( j = 0; j < J_NUM_BB_PARAMETERS; j++ )
        {
            j_error = jReadShortInt( pt_input,
                                    &( p_ToFill->at_Waveforms[ i ].
                                           ai_bb_param[ j ] ) );

            if( j_error != EXIT_SUCCESS )
            {
                printf( "Error while reading L1b ai_bb_param" );
                return EXIT_FAILURE;
            }
        } /* for( j = 0; j < J_NUM_BB_PARAMETERS; j++ ) */
    } /* end for( i = 0; i < J_NUM_HRB; i++ ) for Waveform Group */

    return EXIT_SUCCESS;
} /* jFillL1bSARStructFromFile */

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 +
 short int jFillSARINL1bStructFromFile( L1B_LRM*, FILE* )

 Purpose of routine:  This routine reads LRM L1B data from a file into a
                      structure.

 -----------------------------------------
 Documentation cross reference

 -----------------------------------------

 Coding by:
 D. Brockley       Created: 26/04/2005     Version 1.1

 <author>       Modification: <date>    Version <version>
 <description>

 Reviewed by:
 <reviewer>     <date>  Version <version>

 Test by:
 <tester>               <date>  Version <version>
 -----------------------------------------
 Inputs:
 p_ToFill - pointer to the structure to populate
 pt_input - File pointer to read from

 Outputs: p_ToFill - populated

 Return: Status of processing
 -----------------------------------------
 Pre-condition:
 File open and at start of record. Pointer to valid structure.

 Post-condition:
 Structure filled with a copy of the information represented by
 the L1B file.

 -----------------------------------------
 Calling tree:
 Called by :
 Calls     :
 -----------------------------------------
 Variable documentation

 Global variables used:

 Global constants used:

 -----------------------------------------
 Copyright University College London
 --------------------------------------------------------------------------
 -
 */
long int
jFillL1bSARINStructFromFile(L1B_SARIN* p_ToFill,
                             FILE*      pt_input , BASELINE j_Baseline)
{
    unsigned long int i;
    unsigned long int j;

    long int          j_error = 0;
    int32_t           j_Spare;

    static int bolt = 0;

    if( !bolt &&  gj_verbose )
    {
        printf( "Reading L1b SARIN with baseline %d and nbins %d\n", (int)j_Baseline, J_SARIN_WFM_BINS );
        bolt = 1;
    }

    /*
     * These variables will be used as the destination when a variable must
     * be extracted and discarded. They will also be used as intermediaries
     * when the source and destination varaibles are not of the same type.
     */

    assert( p_ToFill != NULL );

    if( p_ToFill == NULL )
    {
        return EXIT_FAILURE;
    }

    assert( pt_input != NULL );

    if( pt_input == NULL )
    {
        return EXIT_FAILURE;
    }
    /* i_HRB_count repeats of the Time and Orbit Group --------------------- */
    for( i = 0; i < J_NUM_HRB; i++ )
    {
        /* 1 Data Record Time */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Time_Orbit[ i ].j_Day ) );

        if( j_error != EXIT_SUCCESS )
        {
            if( feof( pt_input ) )
            {
                printf( "EOF detected on first read of L1b. OK." );
                return EXIT_SUCCESS;
            }
            else
            {
                printf( "Error while reading L1b j_Day" );
                return EXIT_FAILURE;
            }
        }

        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Time_Orbit[ i ].uj_Sec ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b uj_Sec" );
            return EXIT_FAILURE;
        }

        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Time_Orbit[ i ].uj_Micsec ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b uj_Micsec" );
            return EXIT_FAILURE;
        }

        /* 2 USO Correction Factor */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Time_Orbit[ i ].j_USO_Corr ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_USO_Corr" );
            return EXIT_FAILURE;
        }

        /* 3 Mode ID */
        j_error = jReadShortInt( pt_input,
                                &( p_ToFill->at_Time_Orbit[ i ].ui_Mode_ID ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b ui_Mode_ID" );
            return EXIT_FAILURE;
        }

        /* 4 Source Sequence Counter */
        j_error = jReadShortInt( pt_input,
                                &( p_ToFill->at_Time_Orbit[ i ].ui_SSC ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b ui_SSC" );
            return EXIT_FAILURE;
        }

        /* 5 Instrument Config */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Time_Orbit[ i ].uj_Inst_config ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b uj_Inst_config" );
            return EXIT_FAILURE;
        }

        /* 6 Surface Sample Counter */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Time_Orbit[ i ].uj_Rec_Count ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b uj_Rec_Count" );
            return EXIT_FAILURE;
        }

        /* 7 Lattitude of Measurement */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Time_Orbit[ i ].j_Lat ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_Lat" );
            return EXIT_FAILURE;
        }

        /* 8 Longitude of Measurement */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Time_Orbit[ i ].j_Lon ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_Lon" );
            return EXIT_FAILURE;
        }

        /* 9 Altitude of COG above reference ellipsoid */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Time_Orbit[ i ].j_Alt ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_Alt" );
            return EXIT_FAILURE;
        }

        /* 10 Instantaneous altitude rate derived from orbit */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Time_Orbit[ i ].j_Alt_rate ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_Alt_rate" );
            return EXIT_FAILURE;
        }

        /* 11 Satellite velocity vector in IERF */
        for( j = 0; j < 3; j++ )
        {
            j_error = jReadLongInt( pt_input,
                                   &( p_ToFill->at_Time_Orbit[ i ].
                                          aj_Sat_velocity[ j ] ) );

            if( j_error != EXIT_SUCCESS )
            {
                printf( "Error while reading L1b aj_Sat_velocity" );
                return EXIT_FAILURE;
            }
        }

        /* 12 Real beam direction vector in CRF */
        for( j = 0; j < 3; j++ )
        {
            j_error = jReadLongInt( pt_input,
                                   &( p_ToFill->at_Time_Orbit[ i ].
                                          aj_Real_beam[ j ] ) );

            if( j_error != EXIT_SUCCESS )
            {
                printf( "Error while reading L1b aj_Real_beam" );
                return EXIT_FAILURE;
            }
        }

        /* 13 Interferometer baseline vector in CRF */
        for( j = 0; j < 3; j++ )
        {
            j_error = jReadLongInt( pt_input,
                                   &( p_ToFill->at_Time_Orbit[ i ].
                                          aj_Baseline[ j ] ) );

            if( j_error != EXIT_SUCCESS )
            {
                printf( "Error while reading L1b aj_Baseline" );
                return EXIT_FAILURE;
            }
        }

        if( j_Baseline >= BASELINE_BC )
        {
            j_error = jReadShortInt( pt_input,
                                     &( p_ToFill->at_Time_Orbit[ i ].
                                        i_ST_ID ) );

            if( j_error != EXIT_SUCCESS )
            {
                printf( "Error while reading L1b i_ST_ID" );
                return EXIT_FAILURE;
            }

            j_error = jReadLongInt( pt_input,
                                   &( p_ToFill->at_Time_Orbit[ i ].
                                          j_Roll_angle ) );

            if( j_error != EXIT_SUCCESS )
            {
                printf( "Error while reading L1b j_Roll_angle" );
                return EXIT_FAILURE;
            }

            j_error = jReadLongInt( pt_input,
                                   &( p_ToFill->at_Time_Orbit[ i ].
                                          j_Pitch_angle ) );

            if( j_error != EXIT_SUCCESS )
            {
                printf( "Error while reading L1b j_Pitch_angle" );
                return EXIT_FAILURE;
            }

            j_error = jReadLongInt( pt_input,
                                   &( p_ToFill->at_Time_Orbit[ i ].
                                          j_Yaw_angle ) );

            if( j_error != EXIT_SUCCESS )
            {
                printf( "Error while reading L1b j_Yaw_angle" );
                return EXIT_FAILURE;
            }
        }
        /* 13 Level 1B Measurement Confidence Data */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Time_Orbit[ i ].uj_MCD ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b uj_MCD" );
            return EXIT_FAILURE;
        }

        if( j_Baseline >= BASELINE_BC )
        {
            j_error = jReadLongInt( pt_input,
                                    &j_Spare );

            if( j_error != EXIT_SUCCESS )
            {
                printf( "Error while reading L1b j_Spare" );
                return EXIT_FAILURE;
            }
        }
    } /* end for( i = 0; i < J_NUM_HRB; i++ ) Time and Orbit Group */

    /* J_NUM_HRB repeats of the Measurements Group ----------------------- */
    for( i = 0; i < J_NUM_HRB; i++ )
    {
        /* 15 Window delay */
        j_error = jReadLongLongInt( pt_input,
                                   &( p_ToFill->at_Measurements[ i ].
                                          k_TD ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b k_TD" );
            return EXIT_FAILURE;
        }

        /* 16 Initial Height Word */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Measurements[ i ].
                                      j_H0 ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_H0" );
            return EXIT_FAILURE;
        }

        /* 17 COR2 Height rate per radar cycle */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Measurements[ i ].
                                      j_COR2 ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_COR2" );
            return EXIT_FAILURE;
        }

        /* 18 Coarse Range Word LAI */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_LAI ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_LAI" );
            return EXIT_FAILURE;
        }

        /* 19 Fine Range Word FAI */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_FAI ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_FAI" );
            return EXIT_FAILURE;
        }

        /* 20 AGC ch 1 corrected */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_AGC_ch1 ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_AGC_ch1" );
            return EXIT_FAILURE;
        }

        /* 21 AGC ch 2 corrected */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_AGC_ch2 ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_AGC_ch2" );
            return EXIT_FAILURE;
        }

        /* 22 Overall Tx-Rx gain 1 */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_TR_gain_ch1 ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_TR_gain_ch1" );
            return EXIT_FAILURE;
        }

        /* 23 Overall Tx-Rx gain 2 */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_TR_gain_ch2 ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_TR_gain_ch2" );
            return EXIT_FAILURE;
        }

        /* 24 Transmit power micro-watts */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_Tx_power ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_Tx_power" );
            return EXIT_FAILURE;
        }

        /* Internal Geo_Corrections -------------------------------------------- */

        /* 25 Doppler range correction (radial component) */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_Dopp_rc ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_Dopp_rc" );
            return EXIT_FAILURE;
        }

        /* 26 Instrument range correction t-r antenna */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_TR_inst_rc ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_TR_inst_rc" );
            return EXIT_FAILURE;
        }

        /* 27 Instrument range correction r only antenna */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_R_inst_rc ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_R_inst_rc" );
            return EXIT_FAILURE;
        }

        /* 28 Instrument sigma 0 correction t-r antenna */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_TR_inst_gain_C ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_TR_inst_gain_C" );
            return EXIT_FAILURE;
        }

        /* 29 Instrument sigma 0 correction r antenna */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_R_inst_gain_C ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_R_inst_gain_C" );
            return EXIT_FAILURE;
        }

        /* 30 Internal Phase Correction */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_Int_phase_C ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_Int_phase_C" );
            return EXIT_FAILURE;
        }

        /* 31 External Phase Correction */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_Ext_phase_C ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_Ext_phase_C" );
            return EXIT_FAILURE;
        }

        /* 32 Noise power measurement */
        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
                                                 j_Noise_pwr ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_Noise_pwr" );
            return EXIT_FAILURE;
        }

        j_error = jReadLongInt( pt_input, &( p_ToFill->at_Measurements[ i ].
            j_Phase_slope_C ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_Phase_slope_C" );
            return EXIT_FAILURE;
        }

        /* 33 Skip 4 bytes of spares */
        for( j = 0; j < 4; j++ )
        {
            j_error = jReadChar( pt_input, &(p_ToFill->at_Measurements[ i ].spares[j]) );

            if( j_error != EXIT_SUCCESS )
            {
                printf( "Error while reading L1b spares" );
                return EXIT_FAILURE;
            }
        } /* end for( j = 0; j < 4; j++ ) Spares */
    } /* end for( i = 0; i < J_NUM_HRB; i++ ) Measurements Group */

    /* Geo_Corrections Group --------------------------------------------------- */

    /* 34 Dry Tropospheric Correction */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.j_Dry_C ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_Dry_C" );
        return EXIT_FAILURE;
    }

    /* 35 Wet Tropospheric Correction */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.j_Wet_C ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_Wet_C" );
        return EXIT_FAILURE;
    }

    /* 36 Inverse Barometric Correction */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.j_IB_C ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_IB_C" );
        return EXIT_FAILURE;
    }

    /* 36 DAC Correction */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.j_DAC_C ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_DAC_C" );
        return EXIT_FAILURE;
    }

    /* 37 GIM Ionospheric Correction */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.j_Iono_GIM ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_Iono_GIM" );
        return EXIT_FAILURE;
    }

    /* 38 Model Ionospheric Correction */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.j_Iono_mod ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_Iono_C_mod" );
        return EXIT_FAILURE;
    }

    /* 39 Ocean Tide 1 */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.j_H_OT ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_H_OT" );
        return EXIT_FAILURE;
    }

    /* 40 Ocean Tide 2 */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.j_H_LPEOT) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_H_LPEOT" );
        return EXIT_FAILURE;
    }

    /* 41 Ocean Loading Tide */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.j_H_OLT ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_H_OLT" );
        return EXIT_FAILURE;
    }

    /* 42 Solid Earth Tide */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.j_H_SET ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_H_SET" );
        return EXIT_FAILURE;
    }

    /* 43 Geocentric Polar Tide */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.j_H_GPT ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_H_GPT" );
        return EXIT_FAILURE;
    }

    /* 44 Surface Type Flag */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.uj_Surf_type ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b uj_Surf_type" );
        return EXIT_FAILURE;
    }

    /* 47 Skip 4 bytes of spares */
    for( i = 0; i < 4; i++ )
    {
        j_error = jReadChar( pt_input, &(p_ToFill->t_Geo_Corrections.spare1[i]) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b spares" );
            return EXIT_FAILURE;
        }
    } /* end for( i = 0; i < 4; i++ ) */

    /* 45 Correction Status Flags */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.uj_Corr_status ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b uj_Corr_status" );
        return EXIT_FAILURE;
    }

    /* 46 Correction Error Flags */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Geo_Corrections.
                                  uj_Corr_error ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b uj_Corr_error" );
        return EXIT_FAILURE;
    }

    /* 47 Skip 4 bytes of spares */
    for( i = 0; i < 4; i++ )
    {
        j_error = jReadChar( pt_input, &(p_ToFill->t_Geo_Corrections.spare1[i]) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b spares" );
            return EXIT_FAILURE;
        }
    } /* end for( i = 0; i < 4; i++ ) */

    /* Average Waveforms Group --------------------------------------------- */
    /* 58 Echo Time */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Average_PL.
                                  j_Day_1Hz ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_Day_1Hz" );
        return EXIT_FAILURE;
    }

    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Average_PL.
                                  uj_Sec_1Hz ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b uj_Sec_1Hz" );
        return EXIT_FAILURE;
    }

    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Average_PL.
                                  uj_Micsec_1Hz ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b uj_Micsec_1Hz" );
        return EXIT_FAILURE;
    }

    /* 59 Latitude of Measurement */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Average_PL.
                                  j_Lat_1Hz ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_Lat_1Hz" );
        return EXIT_FAILURE;
    }

    /* 60 Longitude of Measurement */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Average_PL.
                                  j_Lon_1Hz ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_Lon_1Hz" );
        return EXIT_FAILURE;
    }

    /* 61 Altitude of COG above reference ellipsoid */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Average_PL.
                                  j_Alt_1Hz ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_Alt_1Hz" );
        return EXIT_FAILURE;
    }

    /* 62 Window Delay */
    j_error = jReadLongLongInt( pt_input,
                               &( p_ToFill->t_Average_PL.
                                      k_TD_1Hz ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b k_TD_1Hz" );
        return EXIT_FAILURE;
    }

    /* 63 Averaged power echo waveform */
    for( i = 0; i < 512; i++ )
    {
        j_error = jReadShortInt( pt_input,
                                &( p_ToFill->t_Average_PL.
                                       pui_avg_wfm[ i ] ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b aui_avg_wfm" );
            return EXIT_FAILURE;
        }
    } /* end for( i = 0; i < J_SARIN_WFM_BINS; i++ ) Waveform read */

    /* 64 Echo scale factor */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Average_PL.
                                  j_Linear_wfm_mult ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_Linear_wfm_mult" );
        return EXIT_FAILURE;
    }

    /* 65 Echo scale power */
    j_error = jReadLongInt( pt_input,
                           &( p_ToFill->t_Average_PL.
                                  j_Power2_wfm_mult ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b j_Power2_wfm_mult" );
        return EXIT_FAILURE;
    }

    /* 66 Number of echoes averaged */
    j_error = jReadShortInt( pt_input,
                            &( p_ToFill->t_Average_PL.
                                   ui_num_avg_echoes ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b ui_num_avg_echoes" );
        return EXIT_FAILURE;
    }

    /* 67 Flags */
    j_error = jReadShortInt( pt_input,
                            &( p_ToFill->t_Average_PL.
                                   ui_flags ) );

    if( j_error != EXIT_SUCCESS )
    {
        printf( "Error while reading L1b ui_flags" );
        return EXIT_FAILURE;
    }

    /* Waveform Group ------------------------------------------------------ */
    for( i = 0; i < J_NUM_HRB; i++ )
    {
        /* 79 Averaged Power Echo Waveform */
        for( j = 0; j < J_SARIN_WFM_BINS; j++ )
        {
            j_error = jReadShortInt( pt_input,
                                    &( p_ToFill->
                                           at_Waveforms[ i ].
                                           pui_wfm[ j ] ) );

            if( j_error != EXIT_SUCCESS )
            {
                printf( "Error while reading L1b aui_wfm" );
                return EXIT_FAILURE;
            }
        } /* for( j = 0; j < J_SARIN_WFM_BINS; j++ ) Waveform */

        /* 80 Echo scale factor */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Waveforms[ i ].
                                      j_Linear_wfm_mult ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_Linear_wfm_mult" );
            return EXIT_FAILURE;
        }

        /* 81 Echo scale power */
        j_error = jReadLongInt( pt_input,
                               &( p_ToFill->at_Waveforms[ i ].
                                      j_Power2_wfm_mult ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b j_Power2_wfm_mult" );
            return EXIT_FAILURE;
        }

        /* 82 Number of echoes averaged */
        j_error = jReadShortInt( pt_input,
                                &( p_ToFill->at_Waveforms[ i ].
                                       ui_num_avg_echoes ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b ui_num_avg_echoes" );
            return EXIT_FAILURE;
        }

        /* 83 Flags */
        j_error = jReadShortInt( pt_input,
                                &( p_ToFill->at_Waveforms[ i ].
                                       ui_flags ) );

        if( j_error != EXIT_SUCCESS )
        {
            printf( "Error while reading L1b ui_flags" );
            return EXIT_FAILURE;
        }

        /* 84 Beam Behaviour Parameter */
        for( j = 0; j < J_NUM_BB_PARAMETERS; j++ )
        {
            j_error = jReadShortInt( pt_input,
                                    &( p_ToFill->
                                           at_Waveforms[ i ].
                                           ai_bb_param[ j ] ) );

            if( j_error != EXIT_SUCCESS )
            {
                printf( "Error while reading L1b ai_bb_param" );
                return EXIT_FAILURE;
            }
        } /* for( j = 0; j < J_NUM_BB_PARAMETERS; j++ ) */

        /* 85 Coherence */
        for( j = 0; j < J_SARIN_WFM_BINS; j++ )
        {
            j_error = jReadShortInt( pt_input,
                                    &( p_ToFill->
                                           at_Waveforms[ i ].
                                           pui_coherence[ j ] ) );

            if( j_error != EXIT_SUCCESS )
            {
                printf( "Error while reading L1b pj_coherence" );
                return EXIT_FAILURE;
            }
        } /* for( j = 0; j < J_SARIN_WFM_BINS; j++ ) Coherence */

        /* 86 Phase Difference */
        for( j = 0; j < J_SARIN_WFM_BINS; j++ )
        {
            /* This is in the range (-pi->pi)*1000 */
            j_error = jReadLongInt( pt_input,
                                   &( p_ToFill->
                                          at_Waveforms[ i ].
                                          pj_phase_diff[ j ] ) );

            if( j_error != EXIT_SUCCESS )
            {
                printf( "Error while reading L1b pj_phase_diff" );
                return EXIT_FAILURE;
            }
        } /* for( j = 0; j < J_SARIN_WFM_BINS; j++ ) Phase Difference */
    } /* end for( i = 0; i < J_NUM_HRB; i++ ) for Waveform Group */

    return EXIT_SUCCESS;
} /* jFillL1bSARINStructFromFile */
