/*-- pre-processor statements for conditional code --*/

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+

    System: UCL Cryosat IO Library

    Sub-system: Cryosat Specific

    Description:
    Fill a L2I structure from a file.

    ----------------------------------------------
    Documentation cross reference:
    [C2_FMT] L2 Products Format Specification v2.5 09/06/2009 CS-RS-ACS-GS-5123

    ----------------------------------------------
    This file lives in ---
    ./mssl_cryosat

    Coding by:
    David J Brockley
    http://cryosat.mssl.ucl.ac.uk/public

    ----------------------------------------------
    Filename              :     $RCSfile: jFillL2IStructFromFile.c,v $
    CVS Version Number    :     $Revision: 1.4 $
    Last Update           :     $Date: 2015/06/02 11:07:43 $
    What String           :     @(#)$Id: jFillL2IStructFromFile.c,v 1.4 2015/06/02 11:07:43 djb Exp $
    Log                   :
                                $Log: jFillL2IStructFromFile.c,v $
                                Revision 1.4  2015/06/02 11:07:43  djb
                                Preparing for release 2.2

                                Revision 1.3  2014/12/01 11:30:14  djb
                                Pre-testing for release v2.1

                                Revision 1.2  2010/11/11 13:57:47  djb
                                Minor changes to types and naming to adhere to standards.

                                Revision 1.1  2010/07/16 14:37:56  djb
                                Port in CryoSat tools from internal branch.



    ----------------------------------------------
    Routines:
    jFillL2IStructFromFile

    ---------------------------------------------------------------------------
    This file is the Unpublished Intellectual
    Property of University College London and
    All Rights are Reserved.

    Copyright (c) University College London, 2005, 2010
    ---------------------------------------------------------------------------
-
*/

/**
 * @file   jFillL2iStructFromFile.c
 *
 * @brief  Implementation of low-level functions to read Cryosat Level 2 interim data.
 *
 *
 */

#pragma ident "@(#)$Id: jFillL2IStructFromFile.c,v 1.4 2015/06/02 11:07:43 djb Exp $ MSSL"

/*-- pre-processor statements --*/

/*-- standard C headers --*/
#include <assert.h>
#include <stdlib.h>
#include <stdio.h>

/*-- programmer created headers --*/
#include "jFillL2IStructFromFile.h"
#include "jIOFunctions.h"
#include "CS_Defines.h"
/*-- constant and macro headers --*/

/*-- type definitions --*/

/*-- global and external variable declarations --*/

/*-- module private (static) variable declarations --*/

/*-- functions --*/

long int
jFillL2IStructFromFile(L2IData* pt_Level2,
                        FILE*    pt_File , BASELINE j_Baseline)
{
    unsigned short int i;
    long int           j_error;
    int16_t            i_Spare;

    assert( pt_Level2 != NULL );
    assert( pt_File != NULL );

    static int bolt = 0;

    if( !bolt &&  gj_verbose )
    {
        printf( "Reading L2i with baseline %d\n", (int)j_Baseline );
        bolt = 1;
    }

    /* Read and fill j_Day */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Day ) );

    if( feof( pt_File ) )
    {
        printf( "EOF on first read for L2. OK" );
        return EXIT_SUCCESS;
    }

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Day" );
        return EXIT_FAILURE;
    }

    /* Read and fill uj_Sec */
    j_error = jReadLongInt( pt_File, &( pt_Level2->uj_Sec ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 uj_Time_uj_Sec" );
        return EXIT_FAILURE;
    }

    /* Read and fill uj_Micsec */
    j_error = jReadLongInt( pt_File, &( pt_Level2->uj_Micsec ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 uj_Micsec" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_USO_Corr */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_USO_Corr ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_USO_Corr" );
        return EXIT_FAILURE;
    }

    /* Read and fill ui_Mode_ID */
    j_error = jReadShortInt( pt_File, &( pt_Level2->ui_Mode_ID ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 ui_Mode_ID" );
        return EXIT_FAILURE;
    }

    /* Read and fill ui_SSC */
    j_error = jReadShortInt( pt_File, &( pt_Level2->ui_SSC ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 ui_SSC" );
        return EXIT_FAILURE;
    }

    /* Read and fill uj_Inst_config */
    j_error = jReadLongInt( pt_File, &( pt_Level2->uj_Inst_config ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 uj_Inst_config" );
        return EXIT_FAILURE;
    }

    /* Read and fill uj_Rec_Count */
    j_error = jReadLongInt( pt_File, &( pt_Level2->uj_Rec_Count ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 uj_Rec_Count" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_Lat */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Lat ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Lat" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_Lon */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Lon ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Lon" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_Alt */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Alt ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Alt" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_Alt_rate */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Alt_rate ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Alt_rate" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_Sat_velocity */
    for( i = 0; i < 3; i++ )
    {
        j_error = jReadLongInt( pt_File, &( pt_Level2->aj_Sat_velocity[ i ] ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 j_Sat_velocity" );
            return EXIT_FAILURE;
        }
    }

    /* Read and fill j_Real_beam */
    for( i = 0; i < 3; i++ )
    {
        j_error = jReadLongInt( pt_File, &( pt_Level2->aj_Real_beam[ i ] ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 j_Real_beam" );
            return EXIT_FAILURE;
        }
    }

    /* Read and fill j_Baseline */
    for( i = 0; i < 3; i++ )
    {
        j_error = jReadLongInt( pt_File, &( pt_Level2->aj_Baseline[ i ] ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 j_Baseline" );
            return EXIT_FAILURE;
        }
    }

    if( j_Baseline >= BASELINE_BC )
    {
        j_error = jReadShortInt( pt_File, &( pt_Level2->i_ST_ID ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 i_ST_ID" );
            return EXIT_FAILURE;
        }

        j_error = jReadShortInt( pt_File, &( i_Spare ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 i_Spare" );
            return EXIT_FAILURE;
        }

        j_error = jReadLongInt( pt_File, &( pt_Level2->j_Roll_angle ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 j_Roll_angle" );
            return EXIT_FAILURE;
        }

        j_error = jReadLongInt( pt_File, &( pt_Level2->j_Pitch_angle ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 j_Pitch_angle" );
            return EXIT_FAILURE;
        }

        j_error = jReadLongInt( pt_File, &( pt_Level2->j_Yaw_angle ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 j_Yaw_angle" );
            return EXIT_FAILURE;
        }
    }

    /* Read and fill uj_L2_MCD */
    j_error = jReadLongInt( pt_File, &( pt_Level2->uj_L2_MCD ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 uj_L2_MCD" );
        return EXIT_FAILURE;
    }

    /* ---------- */

    /* Read and fill j_Height */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Height ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Height" );
        return EXIT_FAILURE;
    }

    if( j_Baseline >= BASELINE_BC )
    {
        j_error = jReadLongInt( pt_File, &( pt_Level2->j_Height_2 ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 j_Height_2" );
            return EXIT_FAILURE;
        }

        j_error = jReadLongInt( pt_File, &( pt_Level2->j_Height_3 ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 j_Height_3" );
            return EXIT_FAILURE;
        }

    }
    /* Read and fill j_Sig0 */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Sig0 ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Sig0" );
        return EXIT_FAILURE;
    }

    if( j_Baseline >= BASELINE_BC )
    {
        /* Read and fill j_Sig0 */
        j_error = jReadLongInt( pt_File, &( pt_Level2->j_Sig0_2 ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 j_Sig0_2" );
            return EXIT_FAILURE;
        }

        /* Read and fill j_Sig0 */
        j_error = jReadLongInt( pt_File, &( pt_Level2->j_Sig0_3 ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 j_Sig0_3" );
            return EXIT_FAILURE;
        }
    }

    /* Read and fill j_SWH */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_SWH ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_SWH" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_Peakiness */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Peakiness ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Peakiness" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_Retrk_range_C */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Retrk_range_C ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Retrk_range_C" );
        return EXIT_FAILURE;
    }

    if( j_Baseline >= BASELINE_BC )
    {
        j_error = jReadLongInt( pt_File, &( pt_Level2->j_Retrk_range_C_2 ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 j_Retrk_range_C_2" );
            return EXIT_FAILURE;
        }

        j_error = jReadLongInt( pt_File, &( pt_Level2->j_Retrk_range_C_3 ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 j_Retrk_range_C_3" );
            return EXIT_FAILURE;
        }

    }

    /* Read and fill j_Retrk_sig0_C */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Retrk_sig0_C ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Retrk_sig0_C" );
        return EXIT_FAILURE;
    }

    if( j_Baseline >= BASELINE_BC )
    {
        j_error = jReadLongInt( pt_File, &( pt_Level2->j_Retrk_sig0_C_2 ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 j_Retrk_sig0_C_2" );
            return EXIT_FAILURE;
        }

        j_error = jReadLongInt( pt_File, &( pt_Level2->j_Retrk_sig0_C_3 ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 j_Retrk_sig0_C_3" );
            return EXIT_FAILURE;
        }
    }

    if( j_Baseline >= BASELINE_BC )
    {
        j_error = jReadLongInt( pt_File, &( pt_Level2->j_Retrk_sig0_C_2 ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 j_Retrk_sig0_C_2" );
            return EXIT_FAILURE;
        }

        j_error = jReadLongInt( pt_File, &( pt_Level2->j_Retrk_sig0_C_2 ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 j_Retrk_sig0_C_2" );
            return EXIT_FAILURE;
        }

        j_error = jReadLongInt( pt_File, &( pt_Level2->j_Retrk_sig0_C_3 ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 j_Retrk_sig0_C_3" );
            return EXIT_FAILURE;
        }
    }

    /* Read and fill j_Retrk_3 */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Retrk_3 ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Retrk_3" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_Retrk_4 */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Retrk_4 ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Retrk_4" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_Retrk_5 */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Retrk_5 ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Retrk_5" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_Retrk_6 */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Retrk_6 ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Retrk_6" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_Retrk_7 */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Retrk_7 ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Retrk_7" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_Retrk_8 */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Retrk_8 ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Retrk_8" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_Retrk_9 */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Retrk_9 ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Retrk_9" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_Retrk_10 */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Retrk_10 ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Retrk_10" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_Retrk_11 */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Retrk_11 ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Retrk_11" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_Retrk_12 */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Retrk_12 ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Retrk_12" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_Retrk_13 */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Retrk_13 ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Retrk_13" );
        return EXIT_FAILURE;
    }

    if( j_Baseline >= BASELINE_BC )
    {
        j_error = jReadLongInt( pt_File, &( pt_Level2->j_Retrk_14 ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 j_Retrk_14" );
            return EXIT_FAILURE;
        }

        j_error = jReadLongInt( pt_File, &( pt_Level2->j_Retrk_15 ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 j_Retrk_15" );
            return EXIT_FAILURE;
        }

        j_error = jReadLongInt( pt_File, &( pt_Level2->j_Retrk_16 ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 j_Retrk_16" );
            return EXIT_FAILURE;
        }

        j_error = jReadLongInt( pt_File, &( pt_Level2->j_Retrk_17 ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 j_Retrk_17" );
            return EXIT_FAILURE;
        }

        j_error = jReadLongInt( pt_File, &( pt_Level2->j_Retrk_18 ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 j_Retrk_18" );
            return EXIT_FAILURE;
        }

        j_error = jReadLongInt( pt_File, &( pt_Level2->j_Retrk_19 ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 j_Retrk_19" );
            return EXIT_FAILURE;
        }

        j_error = jReadLongInt( pt_File, &( pt_Level2->j_Retrk_20 ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 j_Retrk_20" );
            return EXIT_FAILURE;
        }

        j_error = jReadLongInt( pt_File, &( pt_Level2->j_Retrk_21 ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 j_Retrk_21" );
            return EXIT_FAILURE;
        }

        j_error = jReadLongInt( pt_File, &( pt_Level2->j_Retrk_22 ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 j_Retrk_22" );
            return EXIT_FAILURE;
        }

        j_error = jReadLongInt( pt_File, &( pt_Level2->j_Retrk_23 ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 j_Retrk_23" );
            return EXIT_FAILURE;
        }
    }

    /* Read and fill j_echo_shape */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_echo_shape ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_echo_shape" );
        return EXIT_FAILURE;
    }

    /* Read and fill ai_bb_param */
    for( i = 0; i < 50; i++ )
    {
        j_error = jReadShortInt( pt_File, &( pt_Level2->ai_bb_param[ i ] ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 ai_bb_param" );
            return EXIT_FAILURE;
        }
    }

    /* Read and fill j_XTrack_angle */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_XTrack_angle ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_XTrack_angle" );
        return EXIT_FAILURE;
    }

    if( j_Baseline >= BASELINE_BC )
    {
        j_error = jReadLongInt( pt_File, &( pt_Level2->j_XTrack_angle_C ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 j_XTrack_angle_C" );
            return EXIT_FAILURE;
        }

    }

    /* Read and fill j_Coherence */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Coherence ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Coherence" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_Ocean_ht */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Ocean_ht ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Ocean_ht" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_Freeboard */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Freeboard ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Freeboard" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_SHA */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_SHA ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_SHA" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_SSHA_interp */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_SSHA_interp ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_SSHA_interp" );
        return EXIT_FAILURE;
    }

    /* Read and fill ui_interp_err */
    j_error = jReadShortInt( pt_File, &( pt_Level2->ui_interp_err ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 ui_interp_err" );
        return EXIT_FAILURE;
    }

    /* Read and fill ui_interp_cnt_fwd */
    j_error = jReadShortInt( pt_File, &( pt_Level2->ui_interp_cnt_fwd ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 ui_interp_cnt_fwd" );
        return EXIT_FAILURE;
    }

    /* Read and fill ui_interp_cnt_bkwd */
    j_error = jReadShortInt( pt_File, &( pt_Level2->ui_interp_cnt_bkwd ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 ui_interp_cnt_bkwd" );
        return EXIT_FAILURE;
    }

    /* Read and fill ui_interp_time_fwd */
    j_error = jReadShortInt( pt_File, &( pt_Level2->ui_interp_time_fwd ) );

    if( j_error != 0 )
    {
        printf( "Error reading  L2 ui_interp_time_fwd" );
        return EXIT_FAILURE;
    }

    /* Read and fill ui_interp_time_bkwd */
    j_error = jReadShortInt( pt_File, &( pt_Level2->ui_interp_time_bkwd ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 ui_interp_time_bkwd" );
        return EXIT_FAILURE;
    }

    /* Read and fill ui_interp_error_F */
    j_error = jReadShortInt( pt_File, &( pt_Level2->ui_interp_error_F ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 ui_interp_error_F" );
        return EXIT_FAILURE;
    }

    /* Read and fill uj_Meas_Mode */
    j_error = jReadLongInt( pt_File, &( pt_Level2->uj_Meas_Mode ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 uj_Meas_Mode" );
        return EXIT_FAILURE;
    }

    /* Read and fill uj_Quality_F */
    j_error = jReadLongInt( pt_File, &( pt_Level2->uj_Quality_F ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 uj_Quality_F" );
        return EXIT_FAILURE;
    }

    /* Read and fill uj_Retracker_F */
    j_error = jReadLongInt( pt_File, &( pt_Level2->uj_Retracker_F ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 uj_Retracker_F" );
        return EXIT_FAILURE;
    }

    /* Read and fill uj_Ht_status */
    j_error = jReadLongInt( pt_File, &( pt_Level2->uj_Ht_status ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 uj_Ht_status" );
        return EXIT_FAILURE;
    }

    /* Read and fill uj_Freeb_status */
    j_error = jReadLongInt( pt_File, &( pt_Level2->uj_Freeb_status ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 uj_Freeb_status" );
        return EXIT_FAILURE;
    }

    /* Read and fill ui_n_avg */
    j_error = jReadShortInt( pt_File, &( pt_Level2->ui_n_avg ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 ui_n_avg" );
        return EXIT_FAILURE;
    }

    /* Read and fill ui_Wind_speed */
    j_error = jReadShortInt( pt_File, &( pt_Level2->ui_Wind_speed ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 ui_Wind_speed" );
        return EXIT_FAILURE;
    }

    /* Read and fill auc_spares1 */
    for( i = 0; i < 12; i++ )
    {
        j_error = jReadChar( pt_File, &( pt_Level2->auc_spares1[ i ] ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 auc_spares1" );
            return EXIT_FAILURE;
        }
    }

    /* Read and fill j_Ice_conc */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Ice_conc ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Ice_conc" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_Snow_depth */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Snow_depth ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Snow_depth0" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_Snow_density */
    j_error = jReadLongInt( pt_File, &(pt_Level2->j_Snow_density) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Snow_density" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_Discriminator */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Discriminator ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Discriminator" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_SARin_disc_1 */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_SARin_disc_1 ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_SARin_disc_1" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_SARin_disc_2 */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_SARin_disc_2 ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_SARin_disc_2" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_SARin_disc_3 */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_SARin_disc_3 ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_SARin_disc_3" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_SARin_disc_4 */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_SARin_disc_4 ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_SARin_disc_4" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_SARin_disc_5 */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_SARin_disc_5 ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_SARin_disc_5" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_SARin_disc_6 */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_SARin_disc_6 ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_SARin_disc_6" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_SARin_disc_7 */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_SARin_disc_7 ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_SARin_disc_7" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_SARin_disc_8 */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_SARin_disc_8 ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_SARin_disc_8" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_SARin_disc_9 */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_SARin_disc_9 ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_SARin_disc_9" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_SARin_disc_10 */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_SARin_disc_10 ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_SARin_disc_10" );
        return EXIT_FAILURE;
    }

    /* Read and fill uj_Discrim_F */
    j_error = jReadLongInt( pt_File, &( pt_Level2->uj_Discrim_F ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 uj_Discrim_F" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_Attitude */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Attitude ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Attitude" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_Azimuth */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Azimuth ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Azimuth" );
        return EXIT_FAILURE;
    }

    if( j_Baseline >= BASELINE_BC )
    {
        j_error = jReadLongInt( pt_File, &( pt_Level2->j_Slope_Doppler_C ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 j_Slope_Doppler_C" );
            return EXIT_FAILURE;
        }
    }

    /* Read and fill j_Lat_sat */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Lat_sat ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Lat_sat" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_Lon_sat */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Lon_sat ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Lon_sat" );
        return EXIT_FAILURE;
    }

    /* Read and fill uj_Ambiguity */
    j_error = jReadLongInt( pt_File, &( pt_Level2->uj_Ambiguity ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 uj_Ambiguity" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_MSS_mod */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_MSS_mod ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_MSS_mod" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_Geoid_mod */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Geoid_mod ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Geoid_mod" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_ODLE_mod */
    j_error = jReadLongInt( pt_File, &(pt_Level2->j_ODLE_mod) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_ODLE_mod" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_DEM_elev */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_DEM_elev ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_DEM_elev" );
        return EXIT_FAILURE;
    }

    /* Read and fill uj_DEM_id */
    j_error = jReadLongInt( pt_File, &( pt_Level2->uj_DEM_id ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 uj_DEM_id" );
        return EXIT_FAILURE;
    }

    /* Read and fill auc_spares2 */
    for( i = 0; i < 16; i++ )
    {
        j_error = jReadChar( pt_File, &( pt_Level2->auc_spares2[ i ] ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 auc_spares2" );
            return EXIT_FAILURE;
        }
    }

    /* Read and fill j_Dry_C */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Dry_C ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Dry_C" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_Wet_C_mod */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Wet_C ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Wet_C" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_IB_C */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_IB_C ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_IB_C" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_DAC_C */
    j_error = jReadLongInt( pt_File, &(pt_Level2->j_DAC_C) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_DAC_C" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_Iono_GIM */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Iono_GIM ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Iono_GIM" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_Iono_C_mod */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Iono_mod ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Iono_mod" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_H_OT1 */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_H_OT ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_H_OT" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_H_LPEOT */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_H_LPEOT ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_H_LPEOT" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_H_OLT */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_H_OLT ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_H_OLT" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_H_SET */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_H_SET ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_H_SET" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_H_GPT */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_H_GPT ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_H_GPT" );
        return EXIT_FAILURE;
    }

    /* Read and fill uj_Surf_type */
    j_error = jReadLongInt( pt_File, &( pt_Level2->uj_Surf_type ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 uj_Surf_type" );
        return EXIT_FAILURE;
    }

    /* Read and fill uj_Corr_status */
    j_error = jReadLongInt( pt_File, &( pt_Level2->uj_Corr_status ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 uj_Corr_status" );
        return EXIT_FAILURE;
    }

    /* Read and fill uj_Corr_error */
    j_error = jReadLongInt( pt_File, &( pt_Level2->uj_Corr_error ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 uj_Corr_error" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_SSB */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_SSB ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_SSB" );
        return EXIT_FAILURE;
    }
    /* Read and fill auc_spares3 */
    for( i = 0; i < 8; i++ )
    {
        j_error = jReadChar( pt_File, &( pt_Level2->auc_spares3[ i ] ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 auc_spares3" );
            return EXIT_FAILURE;
        }
    }

    /* Read and fill j_Dopp_rc */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Dopp_rc ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Dopp_rc" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_TR_inst_rc */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_TR_inst_rc ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_TR_inst_rc" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_R_inst_rc */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_R_inst_rc ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_R_inst_rc" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_TR_inst_gain_C */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_TR_inst_gain_C ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_TR_inst_gain_C" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_R_inst_gain_C */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_R_inst_gain_C ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_R_inst_gain_C" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_Int_phase_C */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Int_phase_C ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Int_phase_C" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_Ext_phase_C */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Ext_phase_C ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Ext_phase_C" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_Noise_pwr */
    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Noise_pwr ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Noise_pwr" );
        return EXIT_FAILURE;
    }

    /* Read and fill j_Phase_slope_C */
    j_error = jReadLongInt( pt_File, &(pt_Level2->j_Phase_slope_C) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Phase_slope_C" );
        return EXIT_FAILURE;
    }

    /* Read and fill auc_spares4 */
    for( i = 0; i < 8; i++ )
    {
        j_error = jReadChar( pt_File, &(pt_Level2->auc_spares4[ i ]) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 auc_spares4" );
            return EXIT_FAILURE;
        }
    }

    return EXIT_SUCCESS;
} /* jFillL2IStructFromFile */
