/*-- pre-processor statements for conditional code --*/

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+

    System: UCL Cryosat IO Library

    Sub-system: Cryosat Specific

    Description:
    Fill a L2 structure from a file.

    ----------------------------------------------
    Documentation cross reference:
    [C2_FMT] L2 Products Format Specification v2.5 09/06/2009 CS-RS-ACS-GS-5123

    ----------------------------------------------
    This file lives in ---
    ./mssl_cryosat

    Coding by:
    David J Brockley
    http://cryosat.mssl.ucl.ac.uk/public

    ----------------------------------------------
    Filename              :     $RCSfile: jFillL2StructFromFile.c,v $
    CVS Version Number    :     $Revision: 1.4 $
    Last Update           :     $Date: 2015/06/02 11:07:44 $
    What String           :     @(#)$Id: jFillL2StructFromFile.c,v 1.4 2015/06/02 11:07:44 djb Exp $
    Log                   :
                                $Log: jFillL2StructFromFile.c,v $
                                Revision 1.4  2015/06/02 11:07:44  djb
                                Preparing for release 2.2

                                Revision 1.3  2014/12/01 11:30:14  djb
                                Pre-testing for release v2.1

                                Revision 1.2  2010/11/11 13:57:47  djb
                                Minor changes to types and naming to adhere to standards.

                                Revision 1.1  2010/07/16 14:37:56  djb
                                Port in CryoSat tools from internal branch.



    ----------------------------------------------
    Routines:
    jFillL2StructFromFile

    ---------------------------------------------------------------------------
    This file is the Unpublished Intellectual
    Property of University College London and
    All Rights are Reserved.

    Copyright (c) University College London, 2005, 2010
    ---------------------------------------------------------------------------
-
*/

/**
 * @file   jFillL2StructFromFile.c
 *
 * @brief  Implementation of low-level functions to read Cryosat Level 2 data.
 *
 *
 */

#pragma ident "@(#)$Id: jFillL2StructFromFile.c,v 1.4 2015/06/02 11:07:44 djb Exp $ MSSL"

/*-- pre-processor statements --*/

/*-- standard C headers --*/
#include <assert.h>
#include <stdlib.h>
#include <stdio.h>

/*-- programmer created headers --*/
#include "jFillL2StructFromFile.h"
#include "jIOFunctions.h"
#include "CS_Defines.h"

/*-- constant and macro headers --*/

/*-- type definitions --*/

/*-- global and external variable declarations --*/

/*-- module private (static) variable declarations --*/

/*-- functions --*/

long int
jFillL2StructFromFile(L2Data* pt_Level2,
                       FILE*   pt_File,
                      BASELINE j_Baseline)
{
    unsigned short int i, j;
    long int           j_error;
    uint16_t i_Spare;

    static int bolt = 0;

    if( !bolt &&  gj_verbose )
    {
        printf( "Reading L2 with baseline %d\n", (int)j_Baseline );
        bolt = 1;
    }

    assert( pt_Level2 != NULL );
    assert( pt_File != NULL );

    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Day ) );

    if( feof( pt_File ) )
    {
        printf( "EOF on first read for L2. OK" );
        return EXIT_SUCCESS;
    }

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Day" );
        return EXIT_FAILURE;
    }

    j_error = jReadLongInt( pt_File, &( pt_Level2->uj_Sec ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 uj_Sec" );
        return EXIT_FAILURE;
    }

    j_error = jReadLongInt( pt_File, &( pt_Level2->uj_Micsec ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 uj_Micsec" );
        return EXIT_FAILURE;
    }

    j_error = jReadLongLongInt( pt_File, &( pt_Level2->uk_Siral_mode ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 uk_Siral_mode" );
        return EXIT_FAILURE;
    }

    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Lat_1Hz ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Lat_1Hz" );
        return EXIT_FAILURE;
    }

    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Lon_1Hz ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Lon_1Hz" );
        return EXIT_FAILURE;
    }

    j_error = jReadLongInt( pt_File, &( pt_Level2->j_Alt_1Hz ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_Alt_1Hz" );
        return EXIT_FAILURE;
    }

    if( j_Baseline < BASELINE_C )
    {
        j_error = jReadShortInt( pt_File, &( pt_Level2->i_Mispointing ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 i_Mispointing" );
            return EXIT_FAILURE;
        }
    }
    else
    {
        j_error = jReadLongInt( pt_File, &( pt_Level2->j_Roll_angle ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 j_Roll_angle" );
            return EXIT_FAILURE;
        }

        j_error = jReadLongInt( pt_File, &( pt_Level2->j_Pitch_angle ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 j_Pitch_angle" );
            return EXIT_FAILURE;
        }

        j_error = jReadLongInt( pt_File, &( pt_Level2->j_Yaw_angle ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 j_Yaw_angle" );
            return EXIT_FAILURE;
        }

        j_error = jReadShortInt( pt_File, &( i_Spare ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 i_Spare" );
            return EXIT_FAILURE;
        }
    }

    j_error = jReadShortInt( pt_File, &( pt_Level2->i_N_Valid ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 i_N_Valid" );
        return EXIT_FAILURE;
    }

    /* ---------- */

    j_error = jReadShortInt( pt_File, &( pt_Level2->t_Corr.i_Dry_C ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 i_Dry_C" );
        return EXIT_FAILURE;
    }

    j_error = jReadShortInt( pt_File, &( pt_Level2->t_Corr.i_Wet_C ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 i_Wet_C_mod" );
        return EXIT_FAILURE;
    }

    j_error = jReadShortInt( pt_File, &( pt_Level2->t_Corr.i_IB_C ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 i_IB_C" );
        return EXIT_FAILURE;
    }

    j_error = jReadShortInt( pt_File, &( pt_Level2->t_Corr.i_DAC_C ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 i_DAC_C" );
        return EXIT_FAILURE;
    }

    j_error = jReadShortInt( pt_File, &( pt_Level2->t_Corr.i_Iono_C ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 i_Iono_C" );
        return EXIT_FAILURE;
    }

    j_error = jReadShortInt( pt_File, &( pt_Level2->t_Corr.i_SSB ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 i_SSB" );
        return EXIT_FAILURE;
    }

    j_error = jReadShortInt( pt_File, &( pt_Level2->t_Corr.i_H_OT ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 i_H_OT" );
        return EXIT_FAILURE;
    }

    j_error = jReadShortInt( pt_File, &( pt_Level2->t_Corr.i_H_LPEOT ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 i_H_LPEOT" );
        return EXIT_FAILURE;
    }

    j_error = jReadShortInt( pt_File, &( pt_Level2->t_Corr.i_H_OLT ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 i_H_OLT" );
        return EXIT_FAILURE;
    }

    j_error = jReadShortInt( pt_File, &( pt_Level2->t_Corr.i_H_SET ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 i_H_SET" );
        return EXIT_FAILURE;
    }

    j_error = jReadShortInt( pt_File, &( pt_Level2->t_Corr.i_H_GPT ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 i_H_GPT" );
        return EXIT_FAILURE;
    }

    j_error = jReadShortInt( pt_File, &( pt_Level2->t_Corr.i_Spare1 ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 i_Spare1" );
        return EXIT_FAILURE;
    }

    j_error = jReadLongLongInt( pt_File, &( pt_Level2->t_Corr.uk_Surf_type ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 uk_Surf_type" );
        return EXIT_FAILURE;
    }

    j_error = jReadLongInt( pt_File, &( pt_Level2->t_Corr.j_MSS_Geoid ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_MSS_Geoid" );
        return EXIT_FAILURE;
    }

    j_error = jReadLongInt( pt_File, &( pt_Level2->t_Corr.j_ODLE ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 j_ODLE" );
        return EXIT_FAILURE;
    }

    j_error = jReadShortInt( pt_File, &( pt_Level2->t_Corr.i_Ice_conc ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 i_Snow_depth0" );
        return EXIT_FAILURE;
    }

    j_error = jReadShortInt( pt_File, &( pt_Level2->t_Corr.i_Snow_depth ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 i_Snow_depth" );
        return EXIT_FAILURE;
    }

    j_error = jReadShortInt( pt_File, &( pt_Level2->t_Corr.i_Snow_density ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 i_Snow_density" );
        return EXIT_FAILURE;
    }

    j_error = jReadShortInt( pt_File, &( i_Spare ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 i_Spare2" );
        return EXIT_FAILURE;
    }

    j_error = jReadLongInt( pt_File, &( pt_Level2->t_Corr.uj_C_status ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 uj_C_status" );
        return EXIT_FAILURE;
    }

    j_error = jReadShortInt( pt_File, &( pt_Level2->t_Corr.i_SWH ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 i_SWH" );
        return EXIT_FAILURE;
    }

    j_error = jReadShortInt( pt_File, &( pt_Level2->t_Corr.ui_Wind_speed ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 ui_Wind_speed" );
        return EXIT_FAILURE;
    }

    j_error = jReadShortInt( pt_File, &( i_Spare ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 i_Spare3" );
        return EXIT_FAILURE;
    }

    j_error = jReadShortInt( pt_File, &( i_Spare ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 i_Spare4" );
        return EXIT_FAILURE;
    }

    j_error = jReadShortInt( pt_File, &( i_Spare ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 i_Spare5" );
        return EXIT_FAILURE;
    }

    j_error = jReadShortInt( pt_File, &( i_Spare ) );

    if( j_error != 0 )
    {
        printf( "Error reading L2 i_Spare6" );
        return EXIT_FAILURE;
    }

    /* ---------- */

    for( i = 0; i < 20; i++ )
    {
        j_error = jReadLongInt( pt_File, &( pt_Level2->at_Meas[ i ].j_D_time_mics ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 j_Delta_Time_micsec" );
            return EXIT_FAILURE;
        }

        j_error = jReadLongInt( pt_File, &( pt_Level2->at_Meas[ i ].j_Lat ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 j_Lat" );
            return EXIT_FAILURE;
        }

        j_error = jReadLongInt( pt_File, &( pt_Level2->at_Meas[ i ].j_Lon ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 j_Lon" );
            return EXIT_FAILURE;
        }

        j_error = jReadLongInt( pt_File, &( pt_Level2->at_Meas[ i ].j_Height ) );

        if( j_error != 0 )
        {
            printf( "Error reading L2 j_Height" );
            return EXIT_FAILURE;
        }

        if( j_Baseline >= BASELINE_C )
        {
            j_error = jReadLongInt( pt_File, &( pt_Level2->at_Meas[ i ].j_Height_2 ) );

            if( j_error != 0 )
            {
                printf( "Error reading L2 j_Height_2" );
                return EXIT_FAILURE;
            }

            j_error = jReadLongInt( pt_File, &( pt_Level2->at_Meas[ i ].j_Height_3 ) );

            if( j_error != 0 )
            {
                printf( "Error reading L2 j_Height_3" );
                return EXIT_FAILURE;
            }
        }

        if( j_Baseline >= BASELINE_C )
        {
            j_error = jReadShortInt( pt_File, &( pt_Level2->at_Meas[ i ].i_Sig0 ) );

            if( j_error != 0 )
            {
                printf( "Error reading L2 i_Sig0" );
                return EXIT_FAILURE;
            }
            j_error = jReadShortInt( pt_File, &( pt_Level2->at_Meas[ i ].i_Sig0_2 ) );

            if( j_error != 0 )
            {
                printf( "Error reading L2 i_Sig0_2" );
                return EXIT_FAILURE;
            }

            j_error = jReadShortInt( pt_File, &( pt_Level2->at_Meas[ i ].i_Sig0_3 ) );

            if( j_error != 0 )
            {
                printf( "Error reading L2 i_Sig0_3" );
                return EXIT_FAILURE;
            }

            j_error = jReadShortInt( pt_File, &( pt_Level2->at_Meas[ i ].i_Freeboard ) );

            if( j_error != 0 )
            {
                printf( "Error reading L2 i_Freeboard" );
                return EXIT_FAILURE;
            }

            j_error = jReadShortInt( pt_File, &( pt_Level2->at_Meas[ i ].i_SSHA_interp ) );

            if( j_error != 0 )
            {
                printf( "Error reading L2 i_SSHA_interp" );
                return EXIT_FAILURE;
            }

            j_error = jReadShortInt( pt_File, &( pt_Level2->at_Meas[ i ].i_SSHA_num ) );

            if( j_error != 0 )
            {
                printf( "Error reading L2 i_SSHA_num" );
                return EXIT_FAILURE;
            }

            j_error = jReadShortInt( pt_File, &( pt_Level2->at_Meas[ i ].i_SSHA_qual ) );

            if( j_error != 0 )
            {
                printf( "Error reading L2 i_SSHA_qual" );
                return EXIT_FAILURE;
            }

            j_error = jReadShortInt( pt_File, &( pt_Level2->at_Meas[ i ].i_Peakiness ) );

            if( j_error != 0 )
            {
                printf( "Error reading L2 i_Peakiness" );
                return EXIT_FAILURE;
            }

            j_error = jReadShortInt( pt_File, &( pt_Level2->at_Meas[ i ].i_N_avg ) );

            if( j_error != 0 )
            {
                printf( "Error reading L2 i_N_avg" );
                return EXIT_FAILURE;
            }

            j_error = jReadShortInt( pt_File, &( i_Spare ) );

            if( j_error != 0 )
            {
                printf( "Error reading L2 i_Spare1" );
                return EXIT_FAILURE;
            }

            j_error = jReadLongInt( pt_File, &( pt_Level2->at_Meas[ i ].uj_Quality_F ) );

            if( j_error != 0 )
            {
                printf( "Error reading L2 uj_Quality_flag" );
                return EXIT_FAILURE;
            }

            j_error = jReadLongInt( pt_File, &( pt_Level2->at_Meas[ i ].uj_C_applied ) );

            if( j_error != 0 )
            {
                printf( "Error reading L2 uj_C_applied" );
                return EXIT_FAILURE;
            }

            j_error = jReadLongInt( pt_File, &( pt_Level2->at_Meas[ i ].j_Quality_1) );

            if( j_error != 0 )
            {
                printf( "Error reading L2 j_Quality_1" );
                return EXIT_FAILURE;
            }

            j_error = jReadLongInt( pt_File, &( pt_Level2->at_Meas[ i ].j_Quality_2) );

            if( j_error != 0 )
            {
                printf( "Error reading L2 j_Quality_2" );
                return EXIT_FAILURE;
            }

            j_error = jReadLongInt( pt_File, &( pt_Level2->at_Meas[ i ].j_Quality_3) );

            if( j_error != 0 )
            {
                printf( "Error reading L2 j_Quality_3" );
                return EXIT_FAILURE;
            }
        }
        else
        {
            j_error = jReadShortInt( pt_File, &( pt_Level2->at_Meas[ i ].i_SSHA_interp ) );

            if( j_error != 0 )
            {
                printf( "Error reading L2 i_SSHA_interp" );
                return EXIT_FAILURE;
            }

            j_error = jReadShortInt( pt_File, &( pt_Level2->at_Meas[ i ].i_SSHA_num ) );

            if( j_error != 0 )
            {
                printf( "Error reading L2 i_SSHA_num" );
                return EXIT_FAILURE;
            }

            j_error = jReadShortInt( pt_File, &( pt_Level2->at_Meas[ i ].i_SSHA_qual ) );

            if( j_error != 0 )
            {
                printf( "Error reading L2 i_SSHA_qual" );
                return EXIT_FAILURE;
            }


            j_error = jReadShortInt( pt_File, &( pt_Level2->at_Meas[ i ].i_Sig0 ) );

            if( j_error != 0 )
            {
                printf( "Error reading L2 i_Sig0" );
                return EXIT_FAILURE;
            }

            j_error = jReadShortInt( pt_File, &( pt_Level2->at_Meas[ i ].i_Peakiness ) );

            if( j_error != 0 )
            {
                printf( "Error reading L2 i_Peakiness" );
                return EXIT_FAILURE;
            }

            j_error = jReadShortInt( pt_File, &( pt_Level2->at_Meas[ i ].i_Freeboard ) );

            if( j_error != 0 )
            {
                printf( "Error reading L2 i_Freeboard" );
                return EXIT_FAILURE;
            }

            j_error = jReadShortInt( pt_File, &( pt_Level2->at_Meas[ i ].i_N_avg ) );

            if( j_error != 0 )
            {
                printf( "Error reading L2 i_N_avg" );
                return EXIT_FAILURE;
            }

            j_error = jReadShortInt( pt_File, &( i_Spare ) );

            if( j_error != 0 )
            {
                printf( "Error reading L2 i_Spare1" );
                return EXIT_FAILURE;
            }

            j_error = jReadLongInt( pt_File, &( pt_Level2->at_Meas[ i ].uj_Quality_F ) );

            if( j_error != 0 )
            {
                printf( "Error reading L2 uj_Quality_flag" );
                return EXIT_FAILURE;
            }

            j_error = jReadShortInt( pt_File, &( i_Spare ) );

            if( j_error != 0 )
            {
                printf( "Error reading L2 i_Spare2" );
                return EXIT_FAILURE;
            }

            j_error = jReadShortInt( pt_File, &( i_Spare ) );

            if( j_error != 0 )
            {
                printf( "Error reading L2 i_Spare3" );
                return EXIT_FAILURE;
            }

            j_error = jReadShortInt( pt_File, &( i_Spare ) );

            if( j_error != 0 )
            {
                printf( "Error reading L2 i_Spare4" );
                return EXIT_FAILURE;
            }

            j_error = jReadShortInt( pt_File, &( i_Spare ) );

            if( j_error != 0 )
            {
                printf( "Error reading L2 i_Spare5" );
                return EXIT_FAILURE;
            }
        }
    }

    return EXIT_SUCCESS;
} /* jFillL2StructFromFile */
