/*-- pre-processor statements for conditional code --*/

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+

    System: UCL Cryosat IO Library

    Sub-system: Cryosat Specific

    Description: Get a file handle


    ----------------------------------------------
    Documentation cross reference:


    ----------------------------------------------
    This file lives in ---
    ./mssl_cryosat

    Coding by:
    David J Brockley
    http://cryosat.mssl.ucl.ac.uk/public

    ----------------------------------------------
    Filename              :     $RCSfile: ptCSGetFileHandle.c,v $
    CVS Version Number    :     $Revision: 1.4 $
    Last Update           :     $Date: 2015/06/02 11:07:44 $
    What String           :     @(#)$Id: ptCSGetFileHandle.c,v 1.4 2015/06/02 11:07:44 djb Exp $
    Log                   :
                                $Log: ptCSGetFileHandle.c,v $
                                Revision 1.4  2015/06/02 11:07:44  djb
                                Preparing for release 2.2

                                Revision 1.3  2014/12/01 11:30:14  djb
                                Pre-testing for release v2.1

                                Revision 1.2  2010/10/13 08:12:51  djb
                                PRE_RELEASE_1_1

                                Revision 1.1  2010/07/16 14:37:56  djb
                                Port in CryoSat tools from internal branch.


    ----------------------------------------------
    Routines:
    ptCSGetFileHandle

    -----------------------------------------------------------
    This file contains the Unpublished Intellectual Property of
    University College London and All Rights Are Reserved.
        Copyright (c) University College London, 2005, 2010
    -----------------------------------------------------------
-
*/

/**
 * @file   ptCSGetFileHandle.c
 *
 * @brief  Implementation of a function to get a handle object representing a file.
 *
 *
 */

#pragma ident "@(#)$Id: ptCSGetFileHandle.c,v 1.4 2015/06/02 11:07:44 djb Exp $ MSSL"

/*-- pre-processor statements --*/

/*-- standard C headers --*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <libgen.h>

/*-- programmer created headers --*/
#include "ptCSGetFileHandle.h"
#include "CS_Defines.h"
#include "kGetFileSize.h"

#include "L2Structure.h"
#include "L2IStructure.h"
#include "L1bStructure.h"

#include "jCheckFileForMPH.h"
#include "jCSGetDSFileOffsetFromHeader.h"

/*-- constant and macro headers --*/

/*-- type definitions --*/

/*-- global and external variable declarations --*/

/*-- module private (static) variable declarations --*/

/*-- functions --*/

t_cs_filehandle
ptCSGetFileHandle(char* s_filename, BASELINE j_baseline)
{
    long int        j_result      = 0;
    long int        j_num_records = 0;
    long long int   k_file_size   = 0;

    t_cs_filehandle t_handle      = NULL;

    fpos_t          t_dataset_file_offset;

    t_handle = (t_cs_filehandle)calloc( sizeof( t_cs_fileinfo ), 1 );
    t_handle->j_baseline = j_baseline;

    if( t_handle == NULL )
    {
        return NULL;
    }

    t_handle->pt_filepointer = fopen( s_filename, "rb" );

    if( t_handle->pt_filepointer == NULL )
    {
        free( t_handle );
        return NULL;
    }

    /*
     * To allow this code to handle multiple datasets in one file, you
     * would need to extend the parsing of the MPH done below to fill
     * this in with the number of datasets of the requested type in the
     * file and make the call return an array of offsets to those
     * datasets.
     */
    t_handle->j_num_datasets         = 1;

    t_handle->pat_data_start_offsets = calloc( sizeof( fpos_t ), t_handle->j_num_datasets );

    if( t_handle->pat_data_start_offsets == NULL )
    {
        return NULL;
    }

    t_handle->paj_num_records = calloc( sizeof( long int ), t_handle->j_num_datasets );

    if( t_handle->paj_num_records == NULL )
    {
        return NULL;
    }

    /*
     Determine if this is a SIR file with an MPH and SPH, or an INT
     file that just has raw data in it.
     */
    j_result = jCheckFileForMPH( s_filename );

    if( j_result == TRUE )
    {
        /* Parse the MPH and SPH */
        j_result = jCSGetDSFileOffsetFromHeader(&t_dataset_file_offset,
                                                 &j_num_records,
                                                 s_filename,
                                                 CS_DS_NAME_ANY,
                                                 &t_handle->j_type );

        if( j_result == EXIT_FAILURE )
        {
            return NULL;
        }

        t_handle->pat_data_start_offsets[ 0 ] = t_dataset_file_offset;
        t_handle->paj_num_records[ 0 ]        = j_num_records;
    }
    else
    {
        char* s_filename_cpy = strdup( s_filename );
        char* s_filename_nopath = basename( s_filename_cpy );

        if( j_baseline == BASELINE_UNKNOWN )
        {
            /* We can't figure out the baseline without a header */
            return NULL;
        }

        switch( j_baseline )
        {
            case BASELINE_A:
            case BASELINE_B:
                break;
            case BASELINE_BC:
                J_CS_L1B_LRM_RECORD_SIZE = 9444;
                J_CS_L1B_SAR_RECORD_SIZE = 16564;
                J_CS_L1B_SARIN_RECORD_SIZE = 170932;
                J_CS_L2I_RECORD_SIZE = 664;
                break;
            case BASELINE_C:
                J_CS_L1B_LRM_RECORD_SIZE = 9444;
                J_CS_L1B_SAR_RECORD_SIZE = 16564;
                J_CS_L1B_SARIN_RECORD_SIZE = 170932;
                J_CS_L2_RECORD_SIZE = 1392;
                J_CS_L2I_RECORD_SIZE = 664;
                break;
            default:
                return NULL;
        }

        j_result = fgetpos( t_handle->pt_filepointer, &t_handle->pat_data_start_offsets[ 0 ] );

        if( j_result != EXIT_SUCCESS )
        {
            return NULL;
        }

        /*
         * Need to work out the number of records based on the
         * filesize.
         */
        k_file_size = kGetFileSize( t_handle->pt_filepointer );

        if( k_file_size == 0 )
        {
            return NULL;
        }

        /* We need to know the filetype to get the record size */
        if( strstr( s_filename_nopath, "LRM_2" ) ||
           strstr( s_filename_nopath, "SAR_2" ) ||
           strstr( s_filename_nopath, "SIN_2" ) ||
           strstr( s_filename_nopath, "SID_2" ) )
        {
            printf("Detected L2 filetype\n");
            j_num_records = k_file_size / J_CS_L2_RECORD_SIZE;
            t_handle->j_type = CS_L2_DS_NAME_ANY;
        }
        else if( strstr( s_filename_nopath, "LRMI2" ) ||
                strstr( s_filename_nopath, "SARI" ) ||
                strstr( s_filename_nopath, "SINI2" ) ||
                strstr( s_filename_nopath, "SIDI2" ) )
        {
            printf("Detected L2I filetype\n");
            j_num_records = k_file_size / J_CS_L2I_RECORD_SIZE;
            t_handle->j_type = CS_L2I_DS_NAME_ANY;
        }
        else if( strstr( s_filename_nopath, "LRM_1B" ) )
        {
            printf("Detected L1b LRM filetype\n");
            j_num_records = k_file_size / J_CS_L1B_LRM_RECORD_SIZE;
            t_handle->j_type = CS_L1B_DS_NAME_LRM;
        }
        else if( strstr( s_filename_nopath, "SAR_1B" ) || strstr( s_filename, "SAR__A" ) )
        {
            printf("Detected L1b SAR filetype\n");

            j_num_records = k_file_size / J_CS_L1B_SAR_RECORD_SIZE;
            t_handle->j_type = CS_L1B_DS_NAME_SAR;
        }
        else if( strstr( s_filename_nopath, "SIN_1B" ) ||
                strstr( s_filename_nopath, "SID_1B" ) )
        {
            printf("Detected L1b SARin filetype\n");

            j_num_records = k_file_size / J_CS_L1B_SARIN_RECORD_SIZE;
            t_handle->j_type = CS_L1B_DS_NAME_SARIN;
        }
        else
        {
            return NULL;
        }

        t_handle->paj_num_records[ 0 ] = j_num_records;

        free( s_filename_cpy );
    }

    return t_handle;
} /* ptCSGetFileHandle */
