/*-- pre-processor statements for conditional code --*/

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+

    System: UCL Cryosat IO Library

    Sub-system: Cryosat Specific

    Description: Get L2 records from file.


    ----------------------------------------------
    Documentation cross reference:


    ----------------------------------------------
    This file lives in ---
    ./mssl_cryosat

    Coding by:
    David J Brockley
    http://cryosat.mssl.ucl.ac.uk/public

    ----------------------------------------------
    Filename              :     $RCSfile: ptCSGetL1b.c,v $
    CVS Version Number    :     $Revision: 1.3 $
    Last Update           :     $Date: 2015/06/02 11:07:44 $
    What String           :     @(#)$Id: ptCSGetL1b.c,v 1.3 2015/06/02 11:07:44 djb Exp $
    Log                   :
                                $Log: ptCSGetL1b.c,v $
                                Revision 1.3  2015/06/02 11:07:44  djb
                                Preparing for release 2.2

                                Revision 1.2  2014/12/01 11:30:14  djb
                                Pre-testing for release v2.1

                                Revision 1.1  2010/07/16 14:37:56  djb
                                Port in CryoSat tools from internal branch.



    ----------------------------------------------
    Routines:
    ptCSGetL1bLRM
    ptCSGetL1bSAR
    ptCSGetL1bSARin

    -----------------------------------------------------------
    This file contains the Unpublished Intellectual Property of
    University College London and All Rights Are Reserved.
        Copyright (c) University College London, 2005, 2010
    -----------------------------------------------------------
-
*/

/**
 * @file   ptCSGetL1b.c
 *
 * @brief  Implementation of functions to read Cryosat Level 1b data files.
 *
 *
 */

#pragma ident "@(#)$Id: ptCSGetL1b.c,v 1.3 2015/06/02 11:07:44 djb Exp $ MSSL"

/*-- pre-processor statements --*/

/*-- standard C headers --*/
#include <stdlib.h>

/*-- programmer created headers --*/
#include "ptCSGetL1b.h"
#include "CS_Defines.h"
#include "jFillL1bStructFromFile.h"

/*-- constant and macro headers --*/

/*-- type definitions --*/

/*-- global and external variable declarations --*/

/*-- module private (static) variable declarations --*/

/*-- functions --*/

L1B_LRM*
ptCSGetL1bLRM( t_cs_filehandle        t_handle,
               long int               j_dataset_index,
               unsigned long long int uk_num_records,
               unsigned long long int uk_skip_records,
               L1B_LRM*               pt_data )
{
    /** Total number of records to read from the file */
    unsigned long long int uk_records_to_read = 0;

    /** Loop counter for file reading */
    unsigned long long int uk_count = 0;

    /** The index of the current record being read */
    unsigned long long int uk_read_record = 0;

    /** Return value of function calls */
    long int j_result = 0;

    /** Loop counter */
    int i;
    int j;

    if( t_handle == NULL )
    {
        return NULL;
    }

    if( t_handle->pt_filepointer == NULL )
    {
        return NULL;
    }

    if( t_handle->j_num_datasets <= j_dataset_index )
    {
        return NULL;
    }

    if( !( t_handle->j_type == CS_L1B_DS_NAME_LRM ||
           t_handle->j_type == CS_L1B_DS_NAME_SAR ||
           t_handle->j_type == CS_L1B_DS_NAME_SARIN ) )
     {
         printf( "File is not an L1b filetype.\n" );
         return NULL;
     }

    if( t_handle->j_baseline == BASELINE_UNKNOWN )
    {
         printf( "Unable to read a file when the baseline is not known.\n" );
         return NULL;
    }

    switch( t_handle->j_baseline )
    {
        case BASELINE_A:
        case BASELINE_B:
        case BASELINE_BC:
        case BASELINE_C:
            J_LRM_WFM_BINS = 128;
            break;
        default:
            printf( "The number of waveform bins is not known for this baseline.\n" );
            return NULL;
    }

    if( pt_data == NULL )
    {
        if( uk_num_records == 0 )
        {
            /* Determine number of records in entire file */
            uk_num_records = t_handle->paj_num_records[ j_dataset_index ];
        }

        if( uk_skip_records != 0 )
        {
            uk_records_to_read = uk_num_records - uk_skip_records;
        }
        else
        {
            uk_records_to_read = uk_num_records;
        }

        pt_data = malloc( sizeof( L1B_LRM ) * uk_records_to_read );

        if( pt_data == NULL )
        {
            return NULL;
        }

        for( i = 0; i < uk_records_to_read; i++ )
        {
            pt_data[i].t_Average_PL.pui_avg_wfm = calloc( J_LRM_WFM_BINS, sizeof(uint16_t ) );
            if( pt_data[i].t_Average_PL.pui_avg_wfm == NULL ) return NULL;

            for( j = 0; j < J_NUM_HRB; j++ )
            {
                pt_data[i].at_Waveforms[j].pui_wfm = calloc( J_LRM_WFM_BINS, sizeof(uint16_t ) );
                if( pt_data[i].at_Waveforms[j].pui_wfm == NULL ) return NULL;
            }
        }
    }

    /* Put the filepointer at the start of the data */
    j_result = fsetpos( t_handle->pt_filepointer,
                        &t_handle->pat_data_start_offsets[ j_dataset_index ] );

    if( j_result != EXIT_SUCCESS )
    {
        return NULL;
    }

    /* Read and fill */
    for( uk_count = uk_skip_records; uk_count < uk_num_records; uk_count++ )
    {
        uk_read_record = uk_count - uk_skip_records;

        j_result       = jFillL1bLRMStructFromFile(&pt_data[ uk_read_record ],
                                                    t_handle->pt_filepointer, t_handle->j_baseline );

        if( j_result != EXIT_SUCCESS )
        {
            return NULL;
        }
    }

    return pt_data;
} /* ptCSGetL1bLRM */

L1B_SAR*
ptCSGetL1bSAR( t_cs_filehandle        t_handle,
               long int               j_dataset_index,
               unsigned long long int uk_num_records,
               unsigned long long int uk_skip_records,
               L1B_SAR*               pt_data )
{
    /** Total number of records to read from the file */
    unsigned long long int uk_records_to_read = 0;

    /** Loop counter for file reading */
    unsigned long long int uk_count = 0;

    /** The index of the current record being read */
    unsigned long long int uk_read_record = 0;

    /** Return value of function calls */
    long int j_result = 0;

    /** Loop counter */
    int i;
    int j;

    if( t_handle == NULL )
    {
        return NULL;
    }

    if( t_handle->pt_filepointer == NULL )
    {
        return NULL;
    }

    if( t_handle->j_num_datasets <= j_dataset_index )
    {
        return NULL;
    }

    if( t_handle->j_baseline == BASELINE_UNKNOWN )
    {
         printf( "Unable to read a file when the baseline is not known.\n" );
         return NULL;
    }

    switch( t_handle->j_baseline )
    {
        case BASELINE_A:
        case BASELINE_B:
            J_SAR_WFM_BINS = 128;
            break;
        case BASELINE_BC:
        case BASELINE_C:
            J_SAR_WFM_BINS = 256;
            break;
        default:
            printf( "The number of waveform bins is not known for this baseline.\n" );
            return NULL;
    }

    if( pt_data == NULL )
    {
        if( uk_num_records == 0 )
        {
            /* Determine number of records in entire file */
            uk_num_records = t_handle->paj_num_records[ j_dataset_index ];
        }

        if( uk_skip_records != 0 )
        {
            uk_records_to_read = uk_num_records - uk_skip_records;
        }
        else
        {
            uk_records_to_read = uk_num_records;
        }

        pt_data = malloc( sizeof( L1B_SAR ) * uk_records_to_read );

        if( pt_data == NULL )
        {
            return NULL;
        }

        for( i = 0; i < uk_records_to_read; i++ )
        {
            pt_data[i].t_Average_PL.pui_avg_wfm = calloc( 128, sizeof(uint16_t ) );
            if( pt_data[i].t_Average_PL.pui_avg_wfm == NULL ) return NULL;

            for( j = 0; j < J_NUM_HRB; j++ )
            {
                pt_data[i].at_Waveforms[j].pui_wfm = calloc( J_SAR_WFM_BINS, sizeof(uint16_t ) );
                if( pt_data[i].at_Waveforms[j].pui_wfm == NULL ) return NULL;
            }
        }
    }

    /* Put the filepointer at the start of the data */
    j_result = fsetpos( t_handle->pt_filepointer,
                        &t_handle->pat_data_start_offsets[ j_dataset_index ] );

    if( j_result != EXIT_SUCCESS )
    {
        return NULL;
    }

    /* Read and fill */
    for( uk_count = uk_skip_records; uk_count < uk_num_records; uk_count++ )
    {
        uk_read_record = uk_count - uk_skip_records;

        j_result       = jFillL1bSARStructFromFile(&pt_data[ uk_read_record ],
                                                    t_handle->pt_filepointer, t_handle->j_baseline );

        if( j_result != EXIT_SUCCESS )
        {
            return NULL;
        }
    }

    return pt_data;
} /* ptCSGetL1bSAR */

L1B_SARIN*
ptCSGetL1bSARIN( t_cs_filehandle        t_handle,
                 long int               j_dataset_index,
                 unsigned long long int uk_num_records,
                 unsigned long long int uk_skip_records,
                 L1B_SARIN*             pt_data )
{
    /** Total number of records to read from the file */
    unsigned long long int uk_records_to_read = 0;

    /** Loop counter for file reading */
    unsigned long long int uk_count = 0;

    /** The index of the current record being read */
    unsigned long long int uk_read_record = 0;

    /** Return value of function calls */
    long int j_result = 0;

    /** Loop counter */
    int i;
    int j;

    if( t_handle == NULL )
    {
        return NULL;
    }

    if( t_handle->pt_filepointer == NULL )
    {
        return NULL;
    }

    if( t_handle->j_num_datasets <= j_dataset_index )
    {
        return NULL;
    }

    if( t_handle->j_baseline == BASELINE_UNKNOWN )
    {
         printf( "Unable to read a file when the baseline is not known.\n" );
         return NULL;
    }

    switch( t_handle->j_baseline )
    {
        case BASELINE_A:
        case BASELINE_B:
            J_SARIN_WFM_BINS = 512;
            break;
        case BASELINE_BC:
        case BASELINE_C:
            J_SARIN_WFM_BINS = 1024;
            break;
        default:
            printf( "The number of waveform bins is not known for this baseline.\n" );
            return NULL;
    }

    if( pt_data == NULL )
    {
        if( uk_num_records == 0 )
        {
            /* Determine number of records in entire file */
            uk_num_records = t_handle->paj_num_records[ j_dataset_index ];
        }

        if( uk_skip_records != 0 )
        {
            uk_records_to_read = uk_num_records - uk_skip_records;
        }
        else
        {
            uk_records_to_read = uk_num_records;
        }

        pt_data = malloc( sizeof( L1B_SARIN ) * uk_records_to_read );

        if( pt_data == NULL )
        {
            return NULL;
        }

        for( i = 0; i < uk_records_to_read; i++ )
        {
            pt_data[i].t_Average_PL.pui_avg_wfm = calloc( 512, sizeof(uint16_t ) );
            if( pt_data[i].t_Average_PL.pui_avg_wfm == NULL ) return NULL;

            for( j = 0; j < J_NUM_HRB; j++ )
            {
                pt_data[i].at_Waveforms[j].pui_wfm = calloc( J_SARIN_WFM_BINS, sizeof(uint16_t ) );
                if( pt_data[i].at_Waveforms[j].pui_wfm == NULL ) return NULL;
                pt_data[i].at_Waveforms[j].pui_coherence = calloc( J_SARIN_WFM_BINS, sizeof(uint16_t ) );
                if( pt_data[i].at_Waveforms[j].pui_coherence == NULL ) return NULL;
                pt_data[i].at_Waveforms[j].pj_phase_diff = calloc( J_SARIN_WFM_BINS, sizeof(int32_t ) );
                if( pt_data[i].at_Waveforms[j].pj_phase_diff == NULL ) return NULL;
            }
        }
    }

    /* Put the filepointer at the start of the data */
    j_result = fsetpos( t_handle->pt_filepointer,
                        &t_handle->pat_data_start_offsets[ j_dataset_index ] );

    if( j_result != EXIT_SUCCESS )
    {
        return NULL;
    }

    /* Read and fill */
    for( uk_count = uk_skip_records; uk_count < uk_num_records; uk_count++ )
    {
        uk_read_record = uk_count - uk_skip_records;

        j_result       = jFillL1bSARINStructFromFile( &pt_data[ uk_read_record ],
                                                      t_handle->pt_filepointer, t_handle->j_baseline );

        if( j_result != EXIT_SUCCESS )
        {
            return NULL;
        }
    }

    return pt_data;
} /* ptCSGetL1bSARIN */
