#ifndef BITMASKUTILITIES_H_
#define BITMASKUTILITIES_H_

/*-- pre-processor statements for conditional code --*/

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+

    System: UCL Cryosat IO Library

    Sub-system: MSSL Shared

    Description:
    Manipulate bits by their index within the word. 0 is the LSB.

    ----------------------------------------------
    Documentation cross reference:

    ----------------------------------------------
    This file lives in ---
    ./mssl_shared

    Coding by:
    David J Brockley
    http://cryosat.mssl.ucl.ac.uk/public

    ----------------------------------------------
    Filename              :     $RCSfile: BitmaskUtilities.h,v $
    CVS Version Number    :     $Revision: 1.2 $
    Last Update           :     $Date: 2010/08/27 09:22:59 $
    What String           :     @(#)$Id: BitmaskUtilities.h,v 1.2 2010/08/27 09:22:59 djb Exp $
    Log                   :
                                $Log: BitmaskUtilities.h,v $
                                Revision 1.2  2010/08/27 09:22:59  djb
                                Prototype changes to fix on 64 bit machines and to correct the size of the quality flag in L2.

                                Revision 1.1  2010/07/16 14:37:56  djb
                                Port in CryoSat tools from internal branch.

                                Revision 1.2  2005/10/13 14:04:46  djb
                                More updates for 0.3 release

                                Revision 1.1  2005/10/12 13:40:26  djb
                                Updates for 0.3 release.



    ----------------------------------------------
    Routines:
    iCheckBitLong
    iCheckBitLongLong
    iCheckBitShort
    vClearBitLong
    vClearBitLongLong
    vClearBitShort
    vSetBitLong
    vSetBitLongLong
    vSetBitShort

    -----------------------------------------------------------
    This file contains the Unpublished Intellectual Property of
    University College London and All Rights Are Reserved.
        Copyright (c) University College London, 2005
    -----------------------------------------------------------
-
*/

#pragma ident "@(#)$Id: BitmaskUtilities.h,v 1.2 2010/08/27 09:22:59 djb Exp $ MSSL"

/**
 * @file   BitmaskUtilities.h
 *
 * @brief  Bitmask manipulation utilities.
 *
 *
 */

/*-- pre-processor statements --*/

/*-- standard C headers --*/
#include <stdint.h>
#include <inttypes.h>
/*-- programmer created headers --*/


/*-- constant and macro headers --*/

/*-- type definitions --*/

/*-- global and external variable declarations --*/

/*-- module private (static) variable declarations --*/

/*-- functions --*/

/*
 * DEFINITION OF TERMS
 *
 * The LSB is identified as bit 0. Setting bit 0
 * in this format for an u int16_t gives a value of 1. Setting
 * bit 15 should give a value of 32768.
 *
 */

/**
* \example examples/example_bit_manipulation.c
*
* \latexonly
* \label{ex3}
* \endlatexonly
*
* This is an example of the use of the bitmask handling functions to check
* the status of a bit contained within a bitfield stored within a int16_t.
* The process is identical for bitfields stored within a int32_t, except
* for the replacement of the word Short in the function names with Long e.g.
* vSetBitShort becomes vSetBitLong.
*
*/

/**
 * Set a bit in a int16_teger bitmask.
 *
 * @param pBitmask A pointer to the bitmask.
 * @param bit The bit index to set. Bit 0 is the LSB.
 *
 * @ingroup UTILITIES
 */
void vSetBitShort( uint16_t* pBitmask,
                   uint16_t  bit );

/**
 * Test the value of a bit in a int16_teger bitmask.
 *
 * @param pBitmask A pointer to the bitmask.
 * @param bit The bit index to test. Bit 0 is the LSB.
 *
 * @return true if the bit is set. false otherwise.
 *
 * @ingroup UTILITIES
 */
int16_t iCheckBitShort( uint16_t* pBitmask,
                          uint16_t  bit );

/**
 * Clear a bit in a int16_teger bitmask.
 *
 * @param pBitmask A pointer to the bitmask.
 * @param bit The bit index to clear. Bit 0 is the LSB.
 *
 * @ingroup UTILITIES
 */
void vClearBitShort( uint16_t* pBitmask,
                     uint16_t  bit );

/**
 * Set a bit in a int32_teger bitmask.
 *
 * @param pBitmask A pointer to the bitmask.
 * @param bit The bit index to set. Bit 0 is the LSB.
 * @ingroup UTILITIES
 */
void vSetBitLong( uint32_t* pBitmask,
                  uint16_t bit );

/**
 * Test the value of a bit in a int32_teger bitmask.
 *
 * @param pBitmask A pointer to the bitmask.
 * @param bit The bit index to test. Bit 0 is the LSB.
 *
 * @return true if the bit is set. false otherwise.
 * @ingroup UTILITIES
 */
int16_t iCheckBitLong( uint32_t* pBitmask,
                         uint16_t bit );

/**
 * Clear a bit in a int32_teger bitmask.
 *
 * @param pBitmask A pointer to the bitmask.
 * @param bit The bit index to clear. Bit 0 is the LSB.
 */
void vClearBitLong( uint32_t* pBitmask,
                    uint16_t bit );

/**
 * Set a bit in a long int32_teger bitmask.
 *
 * @param pBitmask A pointer to the bitmask.
 * @param bit The bit index to set. Bit 0 is the LSB.
 * @ingroup UTILITIES
 */
void vSetBitLongLong( uint64_t* pBitmask,
                      uint16_t      bit );

/**
 * Test the value of a bit in a long int32_teger bitmask.
 *
 * @param pBitmask A pointer to the bitmask.
 * @param bit The bit index to test. Bit 0 is the LSB.
 *
 * @return true if the bit is set. false otherwise.
 * @ingroup UTILITIES
 */
int16_t iCheckBitLongLong( uint64_t* pBitmask,
                             uint16_t      bit );

/**
 * Clear a bit in a long int32_teger bitmask.
 *
 * @param pBitmask A pointer to the bitmask.
 * @param bit The bit index to clear. Bit 0 is the LSB.
 * @ingroup UTILITIES
 */
void vClearBitLongLong( uint64_t* pBitmask,
                        uint16_t      bit );

#endif /* ifndef BITMASKUTILITIES_H_ */
