/*-- pre-processor statements for conditional code --*/

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+

    System: UCL Cryosat IO Library

    Sub-system: MSSL Shared

    Description:
    Parse and return a character

    ----------------------------------------------
    Documentation cross reference:


    ----------------------------------------------
    This file lives in ---
    ./mssl_shared

    Coding by:
    David J Brockley
    http://cryosat.mssl.ucl.ac.uk/public

    ----------------------------------------------
    Filename              :     $RCSfile: cGetChar.c,v $
    CVS Version Number    :     $Revision: 1.1 $
    Last Update           :     $Date: 2010/07/16 14:37:56 $
    What String           :     @(#)$Id: cGetChar.c,v 1.1 2010/07/16 14:37:56 djb Exp $
    Log                   :
                                $Log: cGetChar.c,v $
                                Revision 1.1  2010/07/16 14:37:56  djb
                                Port in CryoSat tools from internal branch.

                                Revision 1.2  2005/09/26 09:50:20  djb
                                Changes to documentation.

                                Revision 1.1.1.1  2005/09/23 13:51:56  djb
                                Initial import of sources to CVS.



    ----------------------------------------------
    Routines:
    cGetChar

    -----------------------------------------------------------
    This file contains the Unpublished Intellectual Property of
    University College London and All Rights Are Reserved.
        Copyright (c) University College London, 2005
    -----------------------------------------------------------
-
*/

#pragma ident "@(#)$Id: cGetChar.c,v 1.1 2010/07/16 14:37:56 djb Exp $ MSSL"

/*-- pre-processor statements --*/

/*-- standard C headers --*/
#include <assert.h>
#include <stdlib.h>

/*-- programmer created headers --*/
#include "cGetChar.h"
#include "sFindParam.h"

/*-- constant and macro headers --*/

/*-- type definitions --*/

/*-- global and external variable declarations --*/

/*-- module private (static) variable declarations --*/

/*-- functions --*/

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
    char cGetChar(parameter,buffer,buffer length,return error)

    Purpose of routine:  This routine parses a char.

     -----------------------------------------
    Documentation cross reference

     -----------------------------------------
    Coding by:
    D J Waring  Created: 03-Feb-97	Version 1.1
    <author>	Modification: <date>	Version <version>
        <description>

    Reviewed by:
      <reviewer>	<date>	Version <version>

    Test by:
      <tester>		<date>	Version <version>
     -----------------------------------------
    Inputs:  parameter name, product record buffer and length.

    Outputs: error code

    Return:  char value for parameter or error.
     -----------------------------------------
    Pre-condition:
    buffer contains ASCII text in ESA product standard.

    Post-condition:
    Error set or parameter value returned.
     -----------------------------------------
    Calling tree: Called by :  ()
                  Calls     :
     -----------------------------------------
    Variable documentation

    Global variables used:

    Global constants used:

     -----------------------------------------
     --------------------------------------------------------------------------
-
*/

char
cGetChar( const char* s_parameter, /* parameter to find */
          const char* s_buffer, /* buffer containing params */
          long int    j_buffer_len, /* length of buffer */
          short int*  pi_error /* error return -1=no param,0=ok, */
          /*  1=syntax error,2=range */
          )
{
    char*         s_param_contents; /* pointer to param string */
    char          c_return_value = 0; /* return value */
    unsigned char uc_work_value  = 0;   /* working return value */
    short int     i;        /* loop counter */
    short int     b_sign         = 0; /* 0='+', 1='-' */

    /* get parameter contents */
#ifdef DEBUG
    puts( "\tcGetChar()" );
#endif
    s_param_contents = sFindParam( s_parameter, s_buffer, j_buffer_len, pi_error );
    if( *pi_error != 0 )
    {
        /* parameter doesn't exist */
        return c_return_value;
    }

#ifdef DEBUG
    puts( "FindParam() returned:" );
    puts( s_param_contents );
#endif

    /* check format and get number */
    if( s_param_contents[ 0 ] != '+' )
    {
        if( s_param_contents[ 0 ] != '-' )
        {
            /* if the first char is not a '+' or a '-' the the param should be a single char */
            if( ( s_param_contents[ 0 ] == '\n' ) || ( s_param_contents[ 1 ] != '\n' ) )
            {
                /* syntax error, first char isn't a '+' for an unsigned number */
                /* and param isn't 1 char long */
                *pi_error = 1;
                return c_return_value;
            }
            else
            {
                return s_param_contents[ 0 ];
            }
        }
        b_sign = 1;
    }

    /* extract char value from param */
    s_param_contents++;
    *pi_error = 0;
    if( s_param_contents[ 0 ] != '\n' )
    {
        for( i = 0; i < 3; i++ )
        {
            if( ( s_param_contents[ 0 ] < '0' ) || ( s_param_contents[ 0 ] > '9' ) )
            {
                /* syntax error, there should be 10 digits after the + sign */
                *pi_error = 1;
                return c_return_value;
            }
            if( i == 2 )
            { /* check value within the range -128 -> 127 */
                if( ( uc_work_value > 12 ) ||
                   ( ( uc_work_value == 12 ) && ( s_param_contents[ 0 ] > ( b_sign ? '8' : '7' ) ) ) )
                {
                    /* range check when reading last digit failed, number out of range */
                    *pi_error = 2;
                }
            }
            uc_work_value *= 10;
            uc_work_value += s_param_contents[ 0 ] - '0';
            s_param_contents++;
        }
        c_return_value = ( b_sign ? -1 : 1 ) * uc_work_value;
        if( s_param_contents[ 0 ] != '\n' )
        {
            /* check for units */
            if( s_param_contents[ 0 ] != '<' )
            {
                /* syntax error, number should finish after 10th digit */
                *pi_error = 1;
                return c_return_value;
            }
            while( s_param_contents[ 0 ] != '>' && s_param_contents[ 0 ] != '\n' )
            {
                s_param_contents++;
            }
            if( s_param_contents[ 0 ] != '>' )
            {
                /* syntax error, units string not terminated */
                *pi_error = 1;
                return c_return_value;
            }
            s_param_contents++;
            if( s_param_contents[ 0 ] != '\n' )
            {
                /* syntax error, eol should follow units */
                *pi_error = 1;
                return c_return_value;
            }
        }
    }
    else
    {
        c_return_value = ( b_sign ? '-' : '+' );
    }

    /* return result */
    return c_return_value;
} /* end of cGetChar() */
