/*-- pre-processor statements for conditional code --*/

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+

    System: UCL Cryosat IO Library

    Sub-system: MSSL Shared

    Description:
    Parse and return a double

    ----------------------------------------------
    Documentation cross reference:


    ----------------------------------------------
    This file lives in ---
    ./mssl_shared

    Coding by:
    David J Brockley
    http://cryosat.mssl.ucl.ac.uk/public

    ----------------------------------------------
    Filename              :     $RCSfile: dGetDouble.c,v $
    CVS Version Number    :     $Revision: 1.1 $
    Last Update           :     $Date: 2010/07/16 14:37:56 $
    What String           :     @(#)$Id: dGetDouble.c,v 1.1 2010/07/16 14:37:56 djb Exp $
    Log                   :
                                $Log: dGetDouble.c,v $
                                Revision 1.1  2010/07/16 14:37:56  djb
                                Port in CryoSat tools from internal branch.

                                Revision 1.2  2005/09/26 09:50:20  djb
                                Changes to documentation.

                                Revision 1.1.1.1  2005/09/23 13:51:56  djb
                                Initial import of sources to CVS.



    ----------------------------------------------
    Routines:
    dGetDouble

    -----------------------------------------------------------
    This file contains the Unpublished Intellectual Property of
    University College London and All Rights Are Reserved.
        Copyright (c) University College London, 2005
    -----------------------------------------------------------
-
*/

#pragma ident "@(#)$Id: dGetDouble.c,v 1.1 2010/07/16 14:37:56 djb Exp $ MSSL"

/*-- pre-processor statements --*/

/*-- standard C headers --*/
#include <assert.h>
#include <stdlib.h>
#include <math.h>

/*-- programmer created headers --*/
#include "dGetDouble.h"
#include "sFindParam.h"

/*-- constant and macro headers --*/

/*-- type definitions --*/

/*-- global and external variable declarations --*/

/*-- module private (static) variable declarations --*/

/*-- functions --*/

static double dParseDouble06( char*      param,
                              double     sign,
                              short int* error );

static double dParseDouble46or73( char*      param,
                                  double     value,
                                  short int* error );

static double dParseDouble73( char*      param,
                              double     value,
                              short int* error );

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
    double dGetDouble(parameter,buffer,buffer length,return error)

    Purpose of routine:  This routine parses a double.

     -----------------------------------------
    Documentation cross reference
    PO-RS-MDA-GS-2009

     -----------------------------------------
    Coding by:
    D J Waring  Created: 03-Feb-97	Version 1.1
    D J Waring	Modification: 20-Mar-97	Version 1.2
        Modified because the latest revision of GS-2009 has 3 new
         formats for doubles. (revision B)
    <author>	Modification: <date>	Version <version>
        <description>

    Reviewed by:
      <reviewer>	<date>	Version <version>

    Test by:
      <tester>		<date>	Version <version>
     -----------------------------------------
    Inputs:  parameter name, product record buffer and length.

    Outputs: error code

    Return:  short value for parameter or error.
     -----------------------------------------
    Pre-condition:
    buffer contains ASCII text in ESA product standard.

    Post-condition:
    Error set or parameter value returned.
     -----------------------------------------
    Calling tree: Called by :  ()
                  Calls     :
     -----------------------------------------
    Variable documentation

    Global variables used:

    Global constants used:

     -----------------------------------------
     --------------------------------------------------------------------------
-
*/

double
dGetDouble( const char* s_parameter, /* parameter to find */
            const char* s_buffer, /* buffer containing params */
            long int    j_buffer_len, /* length of buffer */
            short int*  pi_error /* error return -1=no param,0=ok, */
            /*  1=syntax error,2=range */
            )
{
    char*     s_param_contents;     /* pointer to param string */
    double    d_return_value = 0.0; /* return value */
    short int i;            /* loop counter */
    double    d_sign         = 1.0; /* 1='+', -1='-' */
    double    d_exp          = 0.0; /* exponent part */

    /* get parameter contents */
    s_param_contents = sFindParam( s_parameter, s_buffer, j_buffer_len, pi_error );
    if( *pi_error != 0 )
    {
        /* parameter doesn't exist */
        return d_return_value;
    }

    /* check format and get number */
    /* parse sign */
    if( s_param_contents[ 0 ] != '+' )
    {
        if( s_param_contents[ 0 ] != '-' )
        {
            /* syntax error, first char must be a '+' or a '-' for a double */
            *pi_error = 1;
            return d_return_value;
        }
        d_sign = -1.0;
    }
    s_param_contents++;

    /* parse digit */
    if( ( s_param_contents[ 0 ] < '0' ) || ( s_param_contents[ 0 ] > '9' ) )
    {
        /* no digit, maybe an Ado06 */
        if( s_param_contents[ 0 ] == '.' )
        {
            return dParseDouble06( s_param_contents + 1, d_sign, pi_error );
        }

        /* syntax error, there should be 1 digit after the + sign */
        *pi_error = 1;
        return d_return_value;
    }
    d_return_value += ( s_param_contents[ 0 ] - '0' ) * d_sign;
    d_sign         /= 10.0;
    s_param_contents++;
    if( s_param_contents[ 0 ] != '.' )
    {
        /* try and parse as a Ado46 or Ado73 */
        return dParseDouble46or73( s_param_contents, d_return_value, pi_error );
    }
    s_param_contents++;
    for( i = 0; i < 17; i++ ) /* 17 digits */
    {
        if( ( s_param_contents[ 0 ] < '0' ) || ( s_param_contents[ 0 ] > '9' ) )
        {
            /* syntax error, there should be 17 digits after the '.' */
            *pi_error = 1;
            return d_return_value;
        }
        d_return_value += ( s_param_contents[ 0 ] - '0' ) * d_sign;
        d_sign         /= 10.0;
        s_param_contents++;
    }

    /* 'E' */
    if( s_param_contents[ 0 ] != 'E' )
    {
        /* syntax error, 'E' should follow the 17th decimal place */
        *pi_error = 1;
        return d_return_value;
    }
    s_param_contents++;

    /* parse sign */
    switch( s_param_contents[ 0 ] )
    {
        case '+':
        { d_sign = 1.0; } break;
        case '-':
        { d_sign = -1.0; } break;
        default:

            /* syntax error, sign should follow the 'E' */
            *pi_error = 1;
            return d_return_value;
    } /* switch */
    s_param_contents++;

    /* parse exponent */
    for( i = 0; i < 3; i++ ) /* 3 digits */
    {
        if( ( s_param_contents[ 0 ] < '0' ) || ( s_param_contents[ 0 ] > '9' ) )
        {
            /* syntax error, there should be 3 digits after the exponent sign */
            *pi_error = 1;
            return d_return_value;
        }
        d_exp *= 10;
        d_exp += s_param_contents[ 0 ] - '0';
        s_param_contents++;
    }
    d_exp          *= d_sign;
    d_return_value *= pow( 10.0, d_exp );

    /* parse terminator or units */
    if( s_param_contents[ 0 ] != '\n' )
    {
        /* check for units */
        if( s_param_contents[ 0 ] != '<' )
        {
            /* syntax error, number should finish after 10th digit */
            *pi_error = 1;
            return d_return_value;
        }
        while( s_param_contents[ 0 ] != '>' && s_param_contents[ 0 ] != '\n' )
        {
            s_param_contents++;
        }
        if( s_param_contents[ 0 ] != '>' )
        {
            /* syntax error, units string not terminated */
            *pi_error = 1;
            return d_return_value;
        }
        s_param_contents++;
        if( s_param_contents[ 0 ] != '\n' )
        {
            /* syntax error, eol should follow units */
            *pi_error = 1;
            return d_return_value;
        }
    }

    /* return result with no error */
    *pi_error = 0;
    return d_return_value;
} /* end of dGetDouble() */

static double
dParseDouble06( char*      s_param,
                double     d_sign,
                short int* pi_error )
{
    double    d_mul    = d_sign / 10.0; /* multiplier for parsed numbers */
    short int i;        /* loop counter */
    double    d_result = 0;

    for( i = 0; i < 6; i++ )
    {
        if( ( s_param[ 0 ] < '0' ) || ( s_param[ 0 ] > '9' ) )
        {
            /* syntax error, there should be 6 digits */
            *pi_error = 1;
            return d_result;
        }
        d_result += ( s_param[ 0 ] - '0' ) * d_mul;
        d_mul    /= 10.0;
        s_param++;
    }

    /* parse terminator or units */
    if( s_param[ 0 ] != '\n' )
    {
        /* check for units */
        if( s_param[ 0 ] != '<' )
        {
            /* syntax error, number should finish after 10th digit */
            *pi_error = 1;
            return d_result;
        }
        while( s_param[ 0 ] != '>' && s_param[ 0 ] != '\n' )
        {
            s_param++;
        }
        if( s_param[ 0 ] != '>' )
        {
            /* syntax error, units string not terminated */
            *pi_error = 1;
            return d_result;
        }
        s_param++;
        if( s_param[ 0 ] != '\n' )
        {
            /* syntax error, eol should follow units */
            *pi_error = 1;
            return d_result;
        }
    }
    *pi_error = 1;
    return d_result;
} /* dParseDouble06 */

static double
dParseDouble46or73( char*      s_param,
                    double     d_val,
                    short int* pi_error )
{
    short int i;    /* loop counter */
    double    d_result = d_val;

    for( i = 0; i < 3; i++ ) /* another 3 numbers at least */
    {
        if( ( s_param[ 0 ] < '0' ) || ( s_param[ 0 ] > '9' ) )
        {
            /* syntax error, there should be 3 digits */
            *pi_error = 1;
            return d_result;
        }
        d_result *= 10.0;
        d_result += s_param[ 0 ] - '0';
        s_param++;
    }

    /* if next char is a '.' then it's a 46 otherwise it's a 73 */
    if( s_param[ 0 ] != '.' )
    {
        return dParseDouble73( s_param, d_result, pi_error );
    }
    s_param++;

    /* rest of number is just like Ado06 so use that code now */
    return d_result + dParseDouble06( s_param, ( d_result < 0 ) ? -1.0 : 1.0, pi_error );
} /* dParseDouble46or73 */

static double
dParseDouble73( char*      s_param,
                double     d_val,
                short int* pi_error )
{
    double    d_mul;    /* multiplier for fraction */
    short int i;    /* loop counter */
    double    d_result = d_val;

    d_mul = ( d_result < 0 ) ? -0.1 : 0.1;
    for( i = 0; i < 3; i++ ) /* another 3 numbers left */
    {
        if( ( s_param[ 0 ] < '0' ) || ( s_param[ 0 ] > '9' ) )
        {
            /* syntax error, there should be 3 digits */
            *pi_error = 1;
            return d_result;
        }
        d_result *= 10.0;
        d_result += s_param[ 0 ] - '0';
        s_param++;
    }

    /* if next char is a '.' then syntax is ok */
    if( s_param[ 0 ] != '.' )
    {
        *pi_error = 1;
        return d_result;
    }
    s_param++;
    for( i = 0; i < 3; i++ )
    {
        if( ( s_param[ 0 ] < '0' ) || ( s_param[ 0 ] > '9' ) )
        {
            /* syntax error, there should be 3 digits */
            *pi_error = 1;
            return d_result;
        }
        d_result += ( s_param[ 0 ] - '0' ) * d_mul;
        d_mul    /= 10.0;
        s_param++;
    }

    /* parse terminator or units */
    if( s_param[ 0 ] != '\n' )
    {
        /* check for units */
        if( s_param[ 0 ] != '<' )
        {
            /* syntax error, number should finish after 10th digit */
            *pi_error = 1;
            return d_result;
        }
        while( s_param[ 0 ] != '>' && s_param[ 0 ] != '\n' )
        {
            s_param++;
        }
        if( s_param[ 0 ] != '>' )
        {
            /* syntax error, units string not terminated */
            *pi_error = 1;
            return d_result;
        }
        s_param++;
        if( s_param[ 0 ] != '\n' )
        {
            /* syntax error, eol should follow units */
            *pi_error = 1;
            return d_result;
        }
    }
    *pi_error = 1;
    return d_result;
} /* dParseDouble73 */
