/*-- pre-processor statements for conditional code --*/

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+

    System: UCL Cryosat IO Library

    Sub-system: MSSL Shared

    Description:
    Parse and extract a float

    ----------------------------------------------
    Documentation cross reference:


    ----------------------------------------------
    This file lives in ---
    ./mssl_shared

    Coding by:
    David J Brockley
    http://cryosat.mssl.ucl.ac.uk/public

    ----------------------------------------------
    Filename              :     $RCSfile: fGetFloat.c,v $
    CVS Version Number    :     $Revision: 1.1 $
    Last Update           :     $Date: 2010/07/16 14:37:56 $
    What String           :     @(#)$Id: fGetFloat.c,v 1.1 2010/07/16 14:37:56 djb Exp $
    Log                   :
                                $Log: fGetFloat.c,v $
                                Revision 1.1  2010/07/16 14:37:56  djb
                                Port in CryoSat tools from internal branch.

                                Revision 1.2  2005/09/26 09:50:20  djb
                                Changes to documentation.

                                Revision 1.1.1.1  2005/09/23 13:51:56  djb
                                Initial import of sources to CVS.



    ----------------------------------------------
    Routines:
    fGetFloat

    -----------------------------------------------------------
    This file contains the Unpublished Intellectual Property of
    University College London and All Rights Are Reserved.
        Copyright (c) University College London, 2005
    -----------------------------------------------------------
-
*/

#pragma ident "@(#)$Id: fGetFloat.c,v 1.1 2010/07/16 14:37:56 djb Exp $ MSSL"

/*-- pre-processor statements --*/

/*-- standard C headers --*/
#include <assert.h>
#include <stdlib.h>
#include <math.h>

/*-- programmer created headers --*/
#include "fGetFloat.h"
#include "sFindParam.h"

/*-- constant and macro headers --*/

/*-- type definitions --*/

/*-- global and external variable declarations --*/

/*-- module private (static) variable declarations --*/

/*-- functions --*/

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
    float fGetFloat(parameter,buffer,buffer length,return error)

    Purpose of routine:  This routine parses a float.

     -----------------------------------------
    Documentation cross reference
    PO-RS-MDA-GS-2009

     -----------------------------------------
    Coding by:
    D J Waring  Created: 03-Feb-97	Version 1.1
    <author>	Modification: <date>	Version <version>
        <description>

    Reviewed by:
      <reviewer>	<date>	Version <version>

    Test by:
      <tester>		<date>	Version <version>
     -----------------------------------------
    Inputs:  parameter name, product record buffer and length.

    Outputs: error code

    Return:  short value for parameter or error.
     -----------------------------------------
    Pre-condition:
    buffer contains ASCII text in ESA product standard.

    Post-condition:
    Error set or parameter value returned.
     -----------------------------------------
    Calling tree: Called by :  ()
                  Calls     :
     -----------------------------------------
    Variable documentation

    Global variables used:

    Global constants used:

     -----------------------------------------
     --------------------------------------------------------------------------
-
*/

float
fGetFloat( const char* s_parameter, /* parameter to find */
           const char* s_buffer, /* buffer containing params */
           long int    j_buffer_len, /* length of buffer */
           short int*  pi_error /* error return -1=no param,0=ok, */
           /*  1=syntax error,2=range */
           )
{
    char*     s_param_contents;     /* pointer to param string */
    float     f_return_value = 0.0; /* return value */
    short int i;            /* loop counter */
    float     f_sign         = 1.0; /* 1='+', -1='-' */
    float     f_exp          = 0.0; /* exponent part */

    /* get parameter contents */
    s_param_contents = sFindParam( s_parameter, s_buffer, j_buffer_len, pi_error );
    if( *pi_error != 0 )
    {
        /* parameter doesn't exist */
        return f_return_value;
    }

    /* check format and get number */
    /* parse sign */
    if( s_param_contents[ 0 ] != '+' )
    {
        if( s_param_contents[ 0 ] != '-' )
        {
            /* syntax error, first char must be a '+' or a '-' for a double */
            *pi_error = 1;
            return f_return_value;
        }
        f_sign = -1.0;
    }
    s_param_contents++;

    /* parse digit */
    if( ( s_param_contents[ 0 ] < '0' ) || ( s_param_contents[ 0 ] > '9' ) )
    {
        /* syntax error, there should be 1 digit after the + sign */
        *pi_error = 1;
        return f_return_value;
    }
    f_return_value += ( s_param_contents[ 0 ] - '0' ) * f_sign;
    f_sign         /= 10.0;
    s_param_contents++;
    if( s_param_contents[ 0 ] != '.' )
    {
        /* syntax error, there should be a '.' after the digit */
        *pi_error = 1;
        return f_return_value;
    }
    s_param_contents++;
    for( i = 0; i < 8; i++ ) /* 8 digits */
    {
        if( ( s_param_contents[ 0 ] < '0' ) || ( s_param_contents[ 0 ] > '9' ) )
        {
            /* syntax error, there should be 8 digits after the '.' */
            *pi_error = 1;
            return f_return_value;
        }
        f_return_value += ( s_param_contents[ 0 ] - '0' ) * f_sign;
        f_sign         /= 10.0;
        s_param_contents++;
    }

    /* 'E' */
    if( s_param_contents[ 0 ] != 'E' )
    {
        /* syntax error, 'E' should follow the 8th decimal place */
        *pi_error = 1;
        return f_return_value;
    }
    s_param_contents++;

    /* parse sign */
    switch( s_param_contents[ 0 ] )
    {
        case '+':
        { f_sign = 1.0; } break;
        case '-':
        { f_sign = -1.0; } break;
        default:

            /* syntax error, sign should follow the 'E' */
            *pi_error = 1;
            return f_return_value;
    } /* switch */
    s_param_contents++;

    /* parse exponent */
    for( i = 0; i < 2; i++ ) /* 2 digits */
    {
        if( ( s_param_contents[ 0 ] < '0' ) || ( s_param_contents[ 0 ] > '9' ) )
        {
            /* syntax error, there should be 3 digits after the exponent sign */
            *pi_error = 1;
            return f_return_value;
        }
        f_exp *= 10;
        f_exp += s_param_contents[ 0 ] - '0';
        s_param_contents++;
    }
    f_exp          *= f_sign;
    f_return_value *= pow( 10.0, f_exp );

    /* parse terminator or units */
    if( s_param_contents[ 0 ] != '\n' )
    {
        /* check for units */
        if( s_param_contents[ 0 ] != '<' )
        {
            /* syntax error, number should finish after 10th digit */
            *pi_error = 1;
            return f_return_value;
        }
        while( s_param_contents[ 0 ] != '>' && s_param_contents[ 0 ] != '\n' )
        {
            s_param_contents++;
        }
        if( s_param_contents[ 0 ] != '>' )
        {
            /* syntax error, units string not terminated */
            *pi_error = 1;
            return f_return_value;
        }
        s_param_contents++;
        if( s_param_contents[ 0 ] != '\n' )
        {
            /* syntax error, eol should follow units */
            *pi_error = 1;
            return f_return_value;
        }
    }

    /* return result with no error */
    *pi_error = 0;
    return f_return_value;
} /* end of fGetFloat() */
