/*-- pre-processor statements for conditional code -- */

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+

System: UCL Cryosat IO Library

Sub-system: MSSL Shared

Description: Routines for I/O to file in big endian format.
Uses WORDS_BIGENDIAN define to byte swap for i386 processors.

----------------------------------------------
Documentation cross reference

----------------------------------------------
This file lives in ---
./mssl_shared

Coding by:
R. Cullen

--------------------------------------------
Filename              :     $RCSfile: jIOFunctions.c,v $
CVS Version Number    :     $Revision: 1.3 $
Last Update           :     $Date: 2014/12/01 11:30:15 $
What String           :     @(#)$Id: jIOFunctions.c,v 1.3 2014/12/01 11:30:15 djb Exp $
Log                   :
$Log: jIOFunctions.c,v $
Revision 1.3  2014/12/01 11:30:15  djb
Pre-testing for release v2.1

Revision 1.2  2010/08/27 09:22:59  djb
Prototype changes to fix on 64 bit machines and to correct the size of the quality flag in L2.

Revision 1.1  2010/07/16 14:37:56  djb
Port in CryoSat tools from internal branch.

Revision 1.3  2005/10/12 13:40:26  djb
Updates for 0.3 release.

Revision 1.2  2005/09/26 09:50:20  djb
Changes to documentation.

Revision 1.1.1.1  2005/09/23 13:51:56  djb
Initial import of sources to CVS.




--------------------------------------------
Routines:
jReadShortInt
jReadLongInt
jReadFloat
jReadDouble
jReadLongLongInt
jWriteShortInt
jWriteLongInt
jWriteFloat
jWriteDouble
jWriteLongLongInt

---------------------------------------------------------------------------
This file is the Unpublished Intellectual
Property of University College London and
All Rights are Reserved.

Copyright (c) University College London, 2005
---------------------------------------------------------------------------

-
*/

#pragma ident "@(#)$Id: jIOFunctions.c,v 1.3 2014/12/01 11:30:15 djb Exp $ MSSL"

/* -- pre-processor statements -- */

/* -- standard C headers -- */

/*-- programmer created headers -- */

#include "jIOFunctions.h"

/* -- constant and macro headers -- */

/* -- type definitions -- */

/* -- global and external variable declarations -- */
short int gi_do_byteswap = 0;

/* -- functions -- */
static void v_swap( void* p_start, short int i_bytes )
{
	unsigned char* puc_start = (unsigned char*)p_start;
	unsigned char* puc_end   = puc_start + ( i_bytes - 1 );

	unsigned char uc_temp;

	while( puc_start < puc_end )
	{
		uc_temp = *puc_end;
		*puc_end = *puc_start;
		*puc_start = uc_temp;

		--puc_end;
		++puc_start;
	}
}

/*
Read a character
*/
long int
jReadChar( FILE* pt_file_pointer,
           void* pc_value )
{
    unsigned char* puc_param    = NULL; /* pointer to value used in byte swapping */
    unsigned char  uc_param_tmp = '\0'; /* temporary unsigned character */
    long int       j_error      = 0; /* error flag returned from function calls */

    /* Read value */
    if( ( j_error = fread( pc_value, 1, 1, pt_file_pointer ) ) != 1 )
    {
        if( ( j_error = feof( pt_file_pointer ) ) )
        {
            return j_error; /* return EOF */
        }
    }

    return EXIT_SUCCESS;
} /* end jReadChar */

/*
Read a short integer, swapping the bytes
*/
long int
jReadShortInt( FILE* pt_file_pointer,
               void* pi_value )
{
    unsigned char* puc_param    = NULL; /* pointer to value used in byte swapping */
    unsigned char  uc_param_tmp = '\0'; /* temporary unsigned character */
    long int       j_error      = 0; /* error flag returned from function calls */

    /* Read value */

    if( ( j_error = fread( pi_value, 2, 1, pt_file_pointer ) ) != 1 )
    {
        if( ( j_error = feof( pt_file_pointer ) ) )
        {
            return j_error; /* return EOF */
        }
    }

    if( gi_do_byteswap )
    {
		v_swap( pi_value, 2 );
	}

    return EXIT_SUCCESS;
} /* end jReadShortInt */

/*
Read a long integer, swapping the bytes
*/
long int
jReadLongInt( FILE* pt_file_pointer,
              void* pj_value )
{
    unsigned char* puc_param    = NULL; /* pointer to value used in byte swapping */
    unsigned char  uc_param_tmp = '\0'; /* temporary unsigned character */
    long int       j_error      = 0; /* error flag returned from function calls */

    /* Read value */

    if( ( j_error = fread( pj_value, 4, 1, pt_file_pointer ) ) != 1 )
    {
        if( ( j_error = feof( pt_file_pointer ) ) )
        {
            return j_error; /* return EOF */
        }
    }

    if( gi_do_byteswap )
    {
		v_swap( pj_value, 4 );
	}

    return EXIT_SUCCESS;
} /* end jReadLongInt */

/*
Read a floating point number
Assume IEEE format
*/
long int
jReadFloat( FILE* pt_file_pointer,
            void* pf_value )
{
    unsigned char* puc_param    = NULL; /* pointer to value used in byte swapping */
    unsigned char  uc_param_tmp = '\0'; /* temporary unsigned character */
    long int       j_error      = 0; /* error flag returned from function calls */

    /* Read value */

    if( ( j_error = fread( pf_value, 4, 1, pt_file_pointer ) ) != 1 )
    {
        if( ( j_error = feof( pt_file_pointer ) ) )
        {
            return j_error; /* return EOF */
        }
    }

    if( gi_do_byteswap )
    {
		v_swap( pf_value, 4 );
	}

    return EXIT_SUCCESS;
} /* end jReadFloat */

/*
Read a double precision number
Assume IEEE
*/
long int
jReadDouble( FILE* pt_file_pointer,
             void* pd_value )
{
    unsigned char* puc_param    = NULL; /* pointer to value used in byte swapping */
    unsigned char  uc_param_tmp = '\0'; /* temporary unsigned character */
    long int       j_error      = 0; /* error flag returned from function calls */

    /* Read value */

    if( ( j_error = fread( pd_value, 8, 1, pt_file_pointer ) ) != 1 )
    {
        if( ( j_error = feof( pt_file_pointer ) ) )
        {
            return j_error; /* return EOF */
        }
    }

    if( gi_do_byteswap )
    {
		v_swap( pd_value, 8 );
	}

    return EXIT_SUCCESS;
}  /* end jReadDouble*/

/*
Read a long long integer
*/
long int
jReadLongLongInt( FILE* pt_file_pointer,
                  void* pk_value )
{
    unsigned char* puc_param    = NULL; /* pointer to value used in byte swapping */
    unsigned char  uc_param_tmp = '\0'; /* temporary unsigned character */
    long int       j_error      = 0; /* error flag returned from function calls */

    /* Read value */

    if( ( j_error = fread( pk_value, 8, 1, pt_file_pointer ) ) != 1 )
    {
        if( ( j_error = feof( pt_file_pointer ) ) )
        {
            return j_error; /* return EOF */
        }
    }

    if( gi_do_byteswap )
    {
		v_swap( pk_value, 8 );
	}

    return EXIT_SUCCESS;
}   /* end jReadLongLongInt*/

/*
Write a character
*/
long int
jWriteChar( FILE* pt_file_pointer,
            void* pc_value )
{
    unsigned char* puc_param    = NULL; /* pointer to value used in byte swapping */
    unsigned char  uc_param_tmp = '\0'; /* temporary unsigned character */
    long int       j_error      = 0; /* error flag returned from function calls */

    /* Write  value */
    if( ( j_error = fwrite( pc_value, 1, 1, pt_file_pointer ) ) != 1 )
    {
        if( ( j_error = feof( pt_file_pointer ) ) )
        {
            return j_error; /* return EOF */
        }
    }

    return EXIT_SUCCESS;
}  /* end jWriteChar*/

/*
Write a short integer, swapping the bytes
*/
long int
jWriteShortInt( FILE* pt_file_pointer,
                void* pi_value )
{
    unsigned char* puc_param    = NULL; /* pointer to value used in byte swapping */
    unsigned char  uc_param_tmp = '\0'; /* temporary unsigned character */
    long int       j_error      = 0; /* error flag returned from function calls */

    if( gi_do_byteswap )
    {
		v_swap( pi_value, 2 );
	}

    /* Write  value */
    if( ( j_error = fwrite( pi_value, 2, 1, pt_file_pointer ) ) != 1 )
    {
        if( ( j_error = feof( pt_file_pointer ) ) )
        {
            return j_error; /* return EOF */
        }
    }

    return EXIT_SUCCESS;
}  /* end jWriteShortInt*/

/*
Read a long integer, swapping the bytes
*/
long int
jWriteLongInt( FILE* pt_file_pointer,
               void* pj_value )
{
    unsigned char* puc_param    = NULL; /* pointer to value used in byte swapping */
    unsigned char  uc_param_tmp = '\0'; /* temporary unsigned character */
    long int       j_error      = 0; /* error flag returned from function calls */

    /* Read value */

    if( gi_do_byteswap )
    {
		v_swap( pj_value, 4 );
	}

    if( ( j_error = fwrite( pj_value, 4, 1, pt_file_pointer ) ) != 1 )
    {
        if( ( j_error = feof( pt_file_pointer ) ) )
        {
            return j_error; /* return EOF */
        }
    }

    return EXIT_SUCCESS;
}   /* end jWriteLongInt*/

/*
Write a floating point number
Assume IEEE format
*/
long int
jWriteFloat( FILE* pt_file_pointer,
             void* pf_value )
{
    unsigned char* puc_param    = NULL; /* pointer to value used in byte swapping */
    unsigned char  uc_param_tmp = '\0'; /* temporary unsigned character */
    long int       j_error      = 0; /* error flag returned from function calls */

    if( gi_do_byteswap )
    {
		v_swap( pf_value, 4 );
	}

    /* Write value */
    if( fwrite( pf_value, 4, 1, pt_file_pointer ) != 1 )
    {
        return EXIT_FAILURE;
    }

    return EXIT_SUCCESS;
}   /* end jWriteFloat*/

/*
Write a double precision number
Assume IEEE
*/
long int
jWriteDouble( FILE* pt_file_pointer,
              void* pd_value )
{
    unsigned char* puc_param    = NULL; /* pointer to value used in byte swapping */
    unsigned char  uc_param_tmp = '\0'; /* temporary unsigned character */
    long int       j_error      = 0; /* error flag returned from function calls */

    if( gi_do_byteswap )
    {
		v_swap( pd_value, 8 );
	}

    /* write value */
    if( ( j_error = fwrite( pd_value, 8, 1, pt_file_pointer ) ) != 1 )
    {
        if( ( j_error = feof( pt_file_pointer ) ) )
        {
            return j_error; /* return EOF */
        }
    }

    return EXIT_SUCCESS;
}  /* end jWriteDouble*/

/*
Write a long long integer
*/
long int
jWriteLongLongInt( FILE* pt_file_pointer,
                   void* pk_value )
{
    unsigned char* puc_param    = NULL; /* pointer to value used in byte swapping */
    unsigned char  uc_param_tmp = '\0'; /* temporary unsigned character */
    long int       j_error      = 0; /* error flag returned from function calls */

    if( gi_do_byteswap )
    {
		v_swap( pk_value, 8 );
	}

    /* write value */
    if( ( j_error = fwrite( pk_value, 8, 1, pt_file_pointer ) ) != 1 )
    {
        if( ( j_error = feof( pt_file_pointer ) ) )
        {
            return j_error; /* return EOF */
        }
    }

    return EXIT_SUCCESS;
}  /* end jWriteLongLongInt */

short int
iTestEndian( void )
{
    long one = 1;

    if( *( (char*)( &one ) ) )
    {
        gi_do_byteswap = 1;
    }
    else
    {
        gi_do_byteswap = 0;
    }

    return gi_do_byteswap;
} /* iTestEndian */
