; %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
;+
;   System:     Cryosat IPF
;   Sub-system: IDL Support
;
; ------------------------------------------------------------------------------
;
;    Filename              :     $RCSfile: cs_l2i_read.pro,v $
;    CVS Version Number    :     $Revision: 1.3 $
;    Last Update           :     $Date: 2014/12/01 14:36:41 $
;    What String           :     "@(#)$Id: cs_l2i_read.pro,v 1.3 2014/12/01 14:36:41 djb Exp $"
;    Log                   :     
;                                 $Log: cs_l2i_read.pro,v $
;                                 Revision 1.3  2014/12/01 14:36:41  djb
;                                 Tested OK
;
;                                 Revision 1.2  2014/12/01 11:30:15  djb
;                                 Pre-testing for release v2.1
;
;
;
; ------------------------------------------------------------------------------
;
;   Purpose : Read Cryosat L2I product, with or without headers.
;
; ------------------------------------------------------------------------------
;
;   Documentation cross reference
;   CS-IS-MSL-GS-2001 L2 IODD v1.2
;
; ------------------------------------------------------------------------------
;
;   Coding by:
;   S.G. Baker      Created: 29/10/02         
;   D. Brockley              13/01/04         Update for v1.1 IODD
;   D. Brockley              08/06/05         Update for v1.2 IODD
;
;   Reviewed by:
;
;   Tested by:
;
; ------------------------------------------------------------------------------
;   Inputs
;    s_filename     : Level 2 intermediate filename            (mandatory)
;
;   Keywords
;    READ=j_max_num_recs : maximum records to read             (optional)
;    SKIP=j_start_rec    : Number of Source Packets to skip    (optional)
;
;   Outputs
;    structure containing L2I product
;    MPH=s_MPH : Keyword vaiable to return the MPH text fields (optional)
;    SPH=s_SPH : Keyword vaiable to return the SPH text fields (optional)
; ------------------------------------------------------------------------------
;   Calling tree : 
;        Included by : none
;        Includes    : cs_l2i.inc
;          Called by : none
;		   Calls  : read_MPH.pro
;                  	    read_SPH.pro
;                           decode_error.pro
; ------------------------------------------------------------------------------
;   Global Variable Documentation
;     n_DSD_lines      from PDSglobals.inc
;     j_max_PDSrecords from PDSglobals.inc
; ------------------------------------------------------------------------------
;
;   Copyright (c) University College London, 2002, 2004, 2005, 2014
;
; ------------------------------------------------------------------------------
;
;-
;
; syntax    NAME=CS_l2i_read('SIR_L2...',READ =j_max_num_recs,
;                             SKIP =j_start_rec, MPH=l2mph, SPH=l2sph)

FUNCTION cs_l2i_read,s_filename, READ=j_max_num_recs, SKIP=j_start_rec, $
                                  MPH=s_MPH, SPH=s_SPH

@PDSglobals.inc     ; include constants for PDS file formats

; -- local variable declarations ----------------------------------------------
    j_file_unit =	long(0)		; file unit
    j_error_status = 	long(0)		; error status (set by catch)
    j_sph_size =        long(0)         ; set by read_MPH()
    s_MPH_fields =      ''              ; set by read_MPH()
    s_SPH_fields =      ''              ; set by read_SPH()
    s_DSD_fields =      ''              ; set by read_SPH()
    s_format =          ''              ; format string
    j_sph_size =        long(0)         ; SPH size from SPH
    j_DS_start =        long(0)         ; start offset of DSRs from SPH
    j_num_DSR =         long(0)         ; num DSRs from SPH
    j_DSR_size =        long(0)         ; DSR size from SPH
    j_num_records =     long(0)         ; records to read after skipping
    j_check_size =      long(0)         ; to check file size
    j_start_offset =    long(0)         ; offset into file to start reading DSR
;   file_info         is a file stat structure returned by FSTAT()
;   out_struct    structure array created from include file for output data
    s_1=''      ; three temp strings for setting formats
    s_2=''      ; must initialise these as strings or they will become floats
    s_3=''

; -----------------------------------------------------------------------------
; Establish IDL error handling
; Communicate error string to the operator
; Close any open file units
; Return control to the main module
;

    CATCH, j_error_status

    if j_error_status ne 0 then $
    begin
        REPORT_ERROR, 'IDL Error: '+!ERROR_STATE.MSG
        if j_file_unit ne 0 then $
        begin
            CLOSE,j_file_unit
            FREE_LUN,j_file_unit
        endif
        RETALL
    endif

; -----------------------------------------------------------------------------
; define the l2 DSR structure
@cs_l2i.inc
  i_record_size = CS_L2I_MDS_REC_SIZE                   ; from CS_l2.inc

; -----------------------------------------------------------------------------
; If this is an interim file it might not have a header
; so we need to deal with header/headerless cases
; Open file (+ get a free LUN)

  OPENR,j_file_unit, s_filename, error=err,/get_lun,/SWAP_IF_LITTLE_ENDIAN 

; -- Test for open error
  if (err NE 0) then begin                             ; file open error
    CLOSE,j_file_unit
    FREE_LUN,j_file_unit
    REPORT_ERROR,'Error opening file!'
    RETALL
  endif
  
  ; OK to continue
    file_info = FSTAT(j_file_unit)
    CLOSE,j_file_unit               ; close file because it may be opened
    FREE_LUN,j_file_unit            ; again by sub routines

; -- Find the number of records in the file
; -- the L2 MDS record size is 544 bytes 
; If MPH & SPH are present header is 8174 bytes 
; and data starts at 8175

  j_num_DSR = long(file_info.size/i_record_size)

  ;print, j_num_DSR, file_info.size, i_record_size
  
; -- Check if file size is a multiple of rec size
  IF (j_num_DSR*i_record_size eq file_info.size) then begin
    ;print, 'No Header on this file'
    ;PRINT,"The number of DSRs is....",j_num_DSR

  ENDIF else begin   ; Assume there is a header to be read...
    ;print, 'Header on this file'

    ; -------------------------------------------------------------------------
    ; Open file and read MPH header block to find the SPH size
    ; extract information from DSD fields -------------------------------------

    s_MPH_fields= read_MPH(s_filename, j_sph_size)
    ; see if caller wants the MPH header back
    if ARG_PRESENT(s_MPH) then s_MPH=s_MPH_fields

    ; Open file and read SPH header block and extract required DSD
    s_SPH_fields = read_SPH(s_filename,j_sph_size,s_DSD_fields,DS_TYPE='CS_L2')
    ; see if caller wants the SPH header back
    if ARG_PRESENT(s_SPH) then s_SPH=s_SPH_fields
    
    ; s_DSD_fields should be an array of 8 strings
    if (N_ELEMENTS(s_DSD_fields) ne n_DSD_lines) then $
    begin
        decode_error, error_number=28            ;Can not find DSD field
        RETALL
    endif
    ; extract information from DSD fields -------------------------------------
    ; extract DS_OFFSET   s_DSD_fields[3] ---------------
    ; an example string is DS_OFFSET=+00000000000000008222<bytes>
    s_format='(A11,A20,A7)'
    READS,s_DSD_fields[3],s_1,s_2,s_3,FORMAT=s_format
    if(s_1 ne 'DS_OFFSET=+') then $
    begin		    
        decode_error, error_number=28            ;Can not find DS_OFFSET field
        RETALL
    endif
    j_DS_start=long(s_2)
    ; PRINT,s_DSD_fields[3]
    ;PRINT,"The offset of the DSD is....",j_DS_start
    
    ; extract number of DSR in the file   s_DSD_fields[5] 
    ; an example string is NUM_DSR=+0000001760
    s_format='(A9,A20)'
    READS,s_DSD_fields[5],s_1,s_2,FORMAT=s_format
    if(s_1 ne 'NUM_DSR=+') then $
    begin		    
        decode_error, error_number=28            ;Can not find NUM_DSR field
        RETALL
    endif
    j_num_DSR=long(s_2)
    ; PRINT,s_DSD_fields[5]
    ;PRINT,"The number of DSRs is....",j_num_DSR
    
; check the record size   s_DSD_fields[6] ---------------
    ; an example string is DSR_SIZE=+0000000544<bytes>
    s_format='(A10,A10,A7)'
    READS,s_DSD_fields[6],s_1,s_2,s_3,FORMAT=s_format
    if(s_1 ne 'DSR_SIZE=+') then $
    begin		    
        decode_error, error_number=28            ;Can not find DSR_SIZE field
        RETALL
    endif
    
    j_DSR_size=long(s_2)
    if(j_DSR_size ne i_record_size) then $
    begin		    
        decode_error, error_number=19            ;DSR is the wrong size
        RETALL
    endif
    
    ; PRINT,s_DSD_fields[6]
    ;PRINT,"The size of the DSR is....",j_DSR_size

  ENDELSE


; -----------------------------------------------------------------------------
; Set read controlling variables (number of records to read and number of 
; records to skip) based on input arguments and records found.
;
  if (N_ELEMENTS(j_max_num_recs) eq 0) then j_max_num_recs=j_num_DSR
  if (N_ELEMENTS(j_start_rec) eq 0) then j_start_rec=0L
  if (j_start_rec gt j_num_DSR) then j_start_rec=0L
  ;PRINT,' records in file ',j_num_DSR
  ;PRINT,' records to skip set to ',j_start_rec

  j_num_records=LONG(j_num_DSR-j_start_rec)
  if (j_num_records gt j_max_num_recs) then j_num_records= LONG(j_max_num_recs)

  ;-- j_max_PDSrecords is a global variable --
  if (j_num_records gt j_max_PDSrecords) then j_num_records=j_max_PDSrecords
  ;PRINT,' records to be read: ',j_num_records

  ; final check of input keywords
  if (j_start_rec lt 0) then $
  begin
      decode_error, error_number=11	;invalid value of keyword skip
      RETALL
  endif

  if (j_num_records le 0) then $
  begin
      decode_error, error_number=10	;invalid input value, j_num_records
      RETALL
  endif

  ; start offset into the file is the start of the DSR plus any records to skip
  j_start_offset= j_DS_start + (j_DSR_size*j_start_rec)

; -- Open file ----------------------------------------------------------------
    OPENR, j_file_unit, s_filename, /get_lun,/SWAP_IF_LITTLE_ENDIAN 
    ; catch handles open errors

;-- check if file sizes are consistent ----------------------------------------
  file_info = FSTAT(j_file_unit)     ; get file size info
  ; minimum size is start of the read plus number of records to read
  j_check_size = j_start_offset +(j_DSR_size*j_num_records)

  if (j_check_size gt file_info.size) then $
  begin
      decode_error, error_number=29	;invalid file size
      print,'File size= ',file_info.size,' Requested size=',j_check_size
      CLOSE,j_file_unit
      FREE_LUN,j_file_unit
      RETALL
  endif

; -- position at start of DSRs ------------------------------------------------
  POINT_LUN, j_file_unit, j_start_offset  ; position at start of read


; -----------------------------------------------------------------------------
; Create storage for L2 product.
;
  out_struct=REPLICATE({CS_l2i_mds},j_num_records)  ; Create structure to hold data

; -----------------------------------------------------------------------------
; Read the L2 product.

    ;PRINT,' Reading file...'
    readu,j_file_unit,out_struct    ; Read data
    ;PRINT,' ...Done'

; -----------------------------------------------------------------------------
; Close files and return control to calling function.
;

; -- Close file ----------------------------------------------------------------
    if(j_file_unit ne 0) then CLOSE, j_file_unit
    if(j_file_unit ne 0) then FREE_LUN, j_file_unit

; -- Return structure to calling function -------------------------------
    RETURN, out_struct

end			; end of CS_l2i_read.pro

