; %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
;+
;   System:     UCL Cryosat IO Library
;   Sub-system: IDL Support
;
; ------------------------------------------------------------------------------
;
;    Filename              :     $RCSfile: read_mph.pro,v $
;    CVS Version Number    :     $Revision: 1.3 $
;    Last Update           :     $Date: 2014/12/01 11:30:15 $
;    What String           :     "@(#)$Id: read_mph.pro,v 1.3 2014/12/01 11:30:15 djb Exp $"
;    Log                   :     
;                                 $Log: read_mph.pro,v $
;                                 Revision 1.3  2014/12/01 11:30:15  djb
;                                 Pre-testing for release v2.1
;
;                                 Revision 1.1  2005/10/04 08:53:05  djb
;                                 Added more files and documentation.
;
;
;
; ------------------------------------------------------------------------------
;
;   Purpose : Read an ASCII MPH header block from an ESA PDS file
;
; ------------------------------------------------------------------------------
;
;   Documentation cross reference
;   none
;
; ------------------------------------------------------------------------------
;
;   Coding by:
;   S.G. Baker, 2005
;
; ------------------------------------------------------------------------------
;   Inputs
;    s_filename  : PDS filename	(mandatory)
;    j_sph_size  : SPH size	(optional : set within function)
;
;   Keywords
;    none
;
;   Outputs
;    s_MPH_fields : string array of all MPH fields
; ------------------------------------------------------------------------------
;   Calling tree : 
;        Included by : none
;        Includes    : PDSglobals.inc
;       Called by : n/a (library routine)
;	Calls	 : report_error
;                          decode_error
; ------------------------------------------------------------------------------
;   Global Variable Documentation
;     n_MPH_lines from PDSglobals.inc
; ------------------------------------------------------------------------------
;
;   Copyright (c) University College London, 2005
;
; ------------------------------------------------------------------------------
;
;-
;   a=read_MPH('l1b.head.ICET',j_sph_size)


FUNCTION read_MPH,s_filename, j_sph_size

@PDSglobals.inc     ; include constants for PDS file formats

; -- local variable declarations ----------------------------------------------
    j_sph_size =  long(0)                      ; SPH size from MPH
    s_prodname =	string('')		; product name
    s_field =		string('')		; current field string
    j_number_lines =	long(n_MPH_lines)  ; number of lines of text in MPH
    j_error_status = 	long(0)			; error status (set by catch)
    j_file_unit =	long(0)			; file unit
    j_lines_loop =	long(0)			; loop counter
    s_format =          ''                      ; format string
;   s_MPH_fields =	String array of MPH fields, dimensions determined at
;                       execution time

    s_1=''      ; three temp strings for setting formats
    s_2=''      ; must initialise these as strings or they will become floats
    s_3=''
; -----------------------------------------------------------------------------
; Establish IDL error handling
; Communicate error string to the operator
; Close any open file units
; Return control to the main module
;

    CATCH, j_error_status

    if j_error_status ne 0 then $
    begin
        REPORT_ERROR, 'IDL Error: '+!ERROR_STATE.MSG
        if j_file_unit ne 0 then $
        begin
            CLOSE,j_file_unit
            FREE_LUN,j_file_unit
        endif
        RETALL
    endif

; -----------------------------------------------------------------------------
; Open file and read ASCII header block
;

; -- Open file ----------------------------------------------------------------
    OPENR, j_file_unit, s_filename, /get_lun,/SWAP_IF_LITTLE_ENDIAN 

; -- read MPH header text -----------------------------------------------------
    s_field=STRING(REPLICATE(32B, 7))          ; prepare 7 char string of blanks
    READU, j_file_unit, s_field
    if(s_field ne 'PRODUCT') then $
    begin		    
	DECODE_ERROR, error_number=21		;file non-conformant to PSD
	CLOSE,j_file_unit
	FREE_LUN,j_file_unit
	RETALL
    endif

    READF, j_file_unit, s_prodname		;   ="product name"
    POINT_LUN, j_file_unit, 0                   ; rewind the file
;    PRINT,' Reading product ',s_prodname
;help

    s_MPH_fields = SINDGEN(j_number_lines)	;create MPH field array
    for j_lines_loop=0L, j_number_lines-1 do begin
	READF, j_file_unit, s_field
	s_MPH_fields[j_lines_loop] = s_field
;print,s_field
    endfor


    ; extract SPH_SIZE ----------------------------
    ; an example string is SPH_SIZE=+0000006975<bytes>
    s_format='(A10,A10,A7)'
    READS,s_MPH_fields[36],s_1,s_2,s_3,FORMAT=s_format
    if(s_1 ne 'SPH_SIZE=+') then $
    begin		    
	DECODE_ERROR, error_number=24		;Can not find SPH_SIZE field
	CLOSE,j_file_unit
	FREE_LUN,j_file_unit
	RETALL
    endif
    j_sph_size=long(s_2)

   ; print,s_MPH_fields[36]
;    PRINT,'The sph size is....',j_sph_size

    ; full SPH is (NUM_DSD * DSD_SIZE)+SPH_SIZE ---



; -- Close file ----------------------------------------------------------------
    CLOSE, j_file_unit
    FREE_LUN, j_file_unit

; -- Return block name array to calling function -------------------------------
    RETURN, s_MPH_fields

end			; end of read_MPH
