; %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
;+
;   System:     UCL Cryosat IO Library
;   Sub-system: IDL Support
;
; ------------------------------------------------------------------------------
;
;    Filename              :     $RCSfile: read_sph.pro,v $
;    CVS Version Number    :     $Revision: 1.5 $
;    Last Update           :     $Date: 2016/05/19 16:17:42 $
;    What String           :     "@(#)$Id: read_sph.pro,v 1.5 2016/05/19 16:17:42 djb Exp $"
;    Log                   :     
;                                 $Log: read_sph.pro,v $
;                                 Revision 1.5  2016/05/19 16:17:42  djb
;                                 Found and fixed a similar problem with SAR L2i files.
;
;                                 Revision 1.4  2016/05/19 16:06:44  djb
;                                 Fixed SIR_GDR_2_ DS name and reordered so reading of Baseline C GDR works.
;
;                                 Revision 1.3  2014/12/01 11:30:15  djb
;                                 Pre-testing for release v2.1
;
;                                 Revision 1.1  2005/10/04 08:53:05  djb
;                                 Added more files and documentation.
;
;
;
; ------------------------------------------------------------------------------
;
;   Purpose : Read an ASCII SPH header block from a PDS file
;
; ------------------------------------------------------------------------------
;
;   Documentation cross reference
;   none
;
; ------------------------------------------------------------------------------
;
;   Coding by:
;   S.G. Baker, 2005
;
; ------------------------------------------------------------------------------
;   Inputs
;    s_filename  : PDS filename	(mandatory)
;    j_sph_size  : SPH size      (mandatory)
;    j_num_DSD   : number of DSDs	(set within function)
;    j_DSD_size  : size of 1 DSD		(set within function)
;    j_num_ds    : num DS in file	(set within function)
;
;   Keywords
;    none
;
;   Outputs
;    s_SPH_fields : string array of all SPH fields
; ------------------------------------------------------------------------------
;   Calling tree : 
;        Included by : none
;        Includes    : PDSglobals.inc
;          Called by : n/a (library routine)
;	   Calls	 : report_error
;                          decode_error
; ------------------------------------------------------------------------------
;   Global Variable Documentation
;     MPH_size    from PDSglobals.inc
;     n_DSD_lines from PDSglobals.inc
; ------------------------------------------------------------------------------
;
;   Copyright (c) University College London, 2005
;
; ------------------------------------------------------------------------------
;
;-
;   a=read_SPH('RA2_filename',j_sph_size)

FUNCTION read_SPH,s_filename,j_sph_size,s_DSD_fields,DS_TYPE=s_DS_kind

@PDSglobals.inc     ; include constants for PDS file formats

; -- local variable declarations ----------------------------------------------
    SPH_START = long(MPH_size)                  ; constant = size of MPH
;    j_sph_size =  long(0)                      ; SPH size from MPH
    s_descriptor =	string('')		; descriptor name
    s_field =		string('')		; current field string
    j_number_lines =    long(n_DSD_lines)       ; 8 fields in a DSD
    j_lines_loop =	long(0)			; loop counter
    j_error_status = 	long(0)			; error status (set by catch)
    j_file_unit =	long(0)			; file unit
;   s_SPH_fields =	String array of SPH fields, dimensions determined at
;                       execution time
    j_DS_start =       long(0)                 ; start offset of DS
    j_num_DSR =        long(0)                 ; number of DSRs
    j_DSR_size =       long(0)                 ; size of 1 of DSR

    s_1=''      ; three temp strings for setting formats
    s_2=''      ; must initialise these as strings or they will become floats
    s_3=''

; -----------------------------------------------------------------------------
; Establish IDL error handling
; Communicate error string to the operator
; Close any open file units
; Return control to the main module
;

    CATCH, j_error_status

    if j_error_status ne 0 then $
    begin
        REPORT_ERROR, 'IDL Error: '+!ERROR_STATE.MSG
        if j_file_unit ne 0 then $
        begin
            CLOSE,j_file_unit
            FREE_LUN,j_file_unit
        endif
        RETALL
    endif

; -----------------------------------------------------------------------------
; Open file and read SPH header block
;
;print,'hello from read sph'

; -- Open file ----------------------------------------------------------------
    OPENR, j_file_unit, s_filename, /get_lun,/SWAP_IF_LITTLE_ENDIAN 

; -- read SPH header text -----------------------------------------------------
    s_field=STRING(REPLICATE(32B, 14))         ; prepare 14char string of blanks
    POINT_LUN, j_file_unit, SPH_START          ; position at start of SPH
    READU, j_file_unit, s_field
    if(s_field ne 'SPH_DESCRIPTOR') then $
    begin		    
	DECODE_ERROR, error_number=22		;file non-conformant to PSD
	CLOSE,j_file_unit
	FREE_LUN,j_file_unit
	RETALL
    endif

    READF, j_file_unit, s_descriptor		;   ="sph descriptor name"
    POINT_LUN, j_file_unit, SPH_START           ;   reposition the file
    pos=0                                       ; to initialise not to set pos
    POINT_LUN, -j_file_unit, pos                ; get location & put in pos
;print,'pos= ',pos

;    PRINT,'SPH_DESCRIPTOR ',s_descriptor

    s_SPH_fields = STRING(REPLICATE(32B, j_sph_size))  ;create array
    READU, j_file_unit, s_SPH_fields

; -- check for input keywords -------------------------------------------------
    if not(KEYWORD_SET(s_DS_kind)) then s_DS_kind='NIL'
    s_DSD_fields = SINDGEN(j_number_lines)	;create DSD field array

;    PRINT,' looking for DSD=',s_DS_kind

    case s_DS_kind of
        "NIL" : begin
             s_DSD_fields='not set'   ; save space
             ; NO DSD required do nothing
        end

        "L1B" : begin
           ; get the DSD for L1B
           ; find the start within the SPH
           j_pos=STRPOS(s_SPH_fields,'DS_NAME="RA2_SCIENCE_LEVEL_1B        "')
           ; position at start of DSD 
           POINT_LUN, j_file_unit, (SPH_START + j_pos)
           for j_lines_loop=0L, j_number_lines-1 do begin
	     READF, j_file_unit, s_field
	     s_DSD_fields[j_lines_loop] = s_field
           endfor

        end

        "L2" : begin
           ; get the DSD for L2
           ; find the start within the SPH
           j_pos=STRPOS(s_SPH_fields,'DS_NAME="RA2_DATA_SET_FOR_LEVEL_2    "')
           ; position at start of DSD 
           POINT_LUN, j_file_unit, (SPH_START + j_pos)
           for j_lines_loop=0L, j_number_lines-1 do begin
	     READF, j_file_unit, s_field
	     s_DSD_fields[j_lines_loop] = s_field
           endfor
        end

        "L2WWV" : begin
           ; get the DSD for L2WWV
           ; find the start within the SPH
           j_pos=STRPOS(s_SPH_fields,'DS_NAME="RA_2_OCEAN_DATA_FOR_LEVEL_2 "')
           ; position at start of DSD 
           POINT_LUN, j_file_unit, (SPH_START + j_pos)
           for j_lines_loop=0L, j_number_lines-1 do begin
	     READF, j_file_unit, s_field
	     s_DSD_fields[j_lines_loop] = s_field
           endfor
        end

        "L2MWR" : begin
           ; get the DSD for L2MWR
           ; find the start within the SPH 
           j_pos=STRPOS(s_SPH_fields,'DS_NAME="MWR_DATA_SET_FOR_LEVEL_2    "')
           ; position at start of DSD 
           POINT_LUN, j_file_unit, (SPH_START + j_pos)
           for j_lines_loop=0L, j_number_lines-1 do begin
	     READF, j_file_unit, s_field
	     s_DSD_fields[j_lines_loop] = s_field
           endfor
        end

        "L2WFM" : begin
           ; get the DSD for L2WFM
           ; find the start within the SPH 
           j_pos=STRPOS(s_SPH_fields,'DS_NAME="RA2_AVERAGE_WAVEFORMS       "')
           ; position at start of DSD 
           POINT_LUN, j_file_unit, (SPH_START + j_pos)
           for j_lines_loop=0L, j_number_lines-1 do begin
	     READF, j_file_unit, s_field
	     s_DSD_fields[j_lines_loop] = s_field
           endfor
        end

        "BURST" : begin
           ; get the DSD for BURST
           ; find the start within the SPH 
           j_pos=STRPOS(s_SPH_fields,'DS_NAME="RA2_BURST_WAVEFORMS         "')
           ; position at start of DSD 
           POINT_LUN, j_file_unit, (SPH_START + j_pos)
           for j_lines_loop=0L, j_number_lines-1 do begin
	     READF, j_file_unit, s_field
	     s_DSD_fields[j_lines_loop] = s_field
           endfor
        end

        "L0" : begin
           ; get the DSD for L0
           ; find the start within the SPH
           j_pos=STRPOS(s_SPH_fields,'DS_NAME="RA2_SOURCE_PACKETS          "')
           ; position at start of DSD 
           POINT_LUN, j_file_unit, (SPH_START + j_pos)
           for j_lines_loop=0L, j_number_lines-1 do begin
	     READF, j_file_unit, s_field
	     s_DSD_fields[j_lines_loop] = s_field
           endfor
        end
  ;-------------------------CryoSat DSDs---------------------------------------
        "CS_L1" : begin
           ; get the DSD for L1B
           ; find the start within the SPH
           j_pos=STRPOS(s_SPH_fields,'DS_NAME="SIR_FBR_SAR                 "')
           if( j_pos eq -1 ) then begin
             j_pos=STRPOS(s_SPH_fields,'DS_NAME="SIR_FBR_SARIN               "')
           endif
           if( j_pos eq -1 ) then begin
             DECODE_ERROR, error_number=28 ; Unable to find correct DSD name
             close, j_file_unit
             free_lun, j_file_unit
             retall
           endif
           ; position at start of DSD 
           POINT_LUN, j_file_unit, (SPH_START + j_pos)
           for j_lines_loop=0L, j_number_lines-1 do begin
	     READF, j_file_unit, s_field
	     s_DSD_fields[j_lines_loop] = s_field
           endfor
        end

        "CS_L1B" : begin
           ; get the DSD for L1B
           ; find the start within the SPH
           j_pos=STRPOS(s_SPH_fields,'DS_NAME="SIR_L1B_LRM                 "')
           if( j_pos eq -1 ) then begin
             j_pos=STRPOS(s_SPH_fields,'DS_NAME="SIR_L1B_SAR                 "')
           endif
           if( j_pos eq -1 ) then begin
             j_pos=STRPOS(s_SPH_fields,'DS_NAME="SIR_L1B_SARIN               "')
           endif
           if( j_pos eq -1 ) then begin
             j_pos=STRPOS(s_SPH_fields,'DS_NAME="SIR_L1B_FDM                 "')
           endif
           if( j_pos eq -1 ) then begin
             DECODE_ERROR, error_number=28 ; Unable to find correct DSD name
             close, j_file_unit
             free_lun, j_file_unit
             retall
           endif
           ; position at start of DSD 
           POINT_LUN, j_file_unit, (SPH_START + j_pos)
           for j_lines_loop=0L, j_number_lines-1 do begin
	     READF, j_file_unit, s_field
	     s_DSD_fields[j_lines_loop] = s_field
           endfor
        end
        "CS_CCAL1" : begin
	    j_pos=STRPOS(s_SPH_fields,'DS_NAME="SIR_COMPLEX_CAL1_SARIN      "')
	    if( j_pos eq -1 ) then begin
             DECODE_ERROR, error_number=28 ; Unable to find correct DSD name
             close, j_file_unit
             free_lun, j_file_unit
             retall
           endif
           ; position at start of DSD 
           POINT_LUN, j_file_unit, (SPH_START + j_pos)
           for j_lines_loop=0L, j_number_lines-1 do begin
	     READF, j_file_unit, s_field
	     s_DSD_fields[j_lines_loop] = s_field
           endfor

        end
        "CS_L2" : begin
           ; get the DSD for L2
           ; find the start within the SPH

           j_pos=STRPOS(s_SPH_fields,'DS_NAME="SIR_LRM_L2                  "')
           if( j_pos eq -1 ) then begin
              j_pos=STRPOS(s_SPH_fields,'DS_NAME="SIR_GDR_2_                  "')
           endif
           if( j_pos eq -1 ) then begin
             j_pos=STRPOS(s_SPH_fields,'DS_NAME="SIR_SAR_L2B                 "')
           endif
           if( j_pos eq -1 ) then begin
             j_pos=STRPOS(s_SPH_fields,'DS_NAME="SIR_SAR_L2                  "')
           endif
           if( j_pos eq -1 ) then begin
             j_pos=STRPOS(s_SPH_fields,'DS_NAME="SIR_FDM_L2                  "')
           endif
           if( j_pos eq -1 ) then begin
             j_pos=STRPOS(s_SPH_fields,'DS_NAME="SIR_SARIL2B                 "')
           endif
           if( j_pos eq -1 ) then begin
             j_pos=STRPOS(s_SPH_fields,'DS_NAME="SIR_SARIL2                  "')
           endif
           if( j_pos eq -1 ) then begin
             j_pos=STRPOS(s_SPH_fields,'DS_NAME="SIR_SAR_L2_I                "')
           endif
           if( j_pos eq -1 ) then begin
             j_pos=STRPOS(s_SPH_fields,'DS_NAME="SIR_SAR_L2B_I               "')
           endif
           if( j_pos eq -1 ) then begin
             j_pos=STRPOS(s_SPH_fields,'DS_NAME="SIR_SAR_L2A                 "')
           endif
           if( j_pos eq -1 ) then begin
             j_pos=STRPOS(s_SPH_fields,'DS_NAME="SIR_SIN_L2                  "')
           endif
           if( j_pos eq -1 ) then begin
             j_pos=STRPOS(s_SPH_fields,'DS_NAME="SIR_SID_L2                  "')
           endif
           if( j_pos eq -1 ) then begin
             j_pos=STRPOS(s_SPH_fields,'DS_NAME="SIR_LRMIL2                  "')
           endif
           if( j_pos eq -1 ) then begin
             j_pos=STRPOS(s_SPH_fields,'DS_NAME="SIR_LRM_L2_I                "')
           endif
           if( j_pos eq -1 ) then begin
             j_pos=STRPOS(s_SPH_fields,'DS_NAME="SIR_SARIL2A                 "')
           endif
           if( j_pos eq -1 ) then begin
             j_pos=STRPOS(s_SPH_fields,'DS_NAME="SIR_SAR_L2A_I               "')
           endif
           if( j_pos eq -1 ) then begin
             j_pos=STRPOS(s_SPH_fields,'DS_NAME="SIR_SAR_L2_I                "')
           endif
           if( j_pos eq -1 ) then begin
             j_pos=STRPOS(s_SPH_fields,'DS_NAME="SIR_SINIL2                  "')
           endif
           if( j_pos eq -1 ) then begin
             j_pos=STRPOS(s_SPH_fields,'DS_NAME="SIR_SIN_L2_I                "')
           endif
           if( j_pos eq -1 ) then begin
             j_pos=STRPOS(s_SPH_fields,'DS_NAME="SIR_SIDIL2                  "')
           endif
           if( j_pos eq -1 ) then begin
             j_pos=STRPOS(s_SPH_fields,'DS_NAME="SIR_SID_L2_I                "')
          endif
           if( j_pos eq -1 ) then begin
             j_pos=STRPOS(s_SPH_fields,'DS_NAME="SIR_GDR_2A                  "')
          endif
           if( j_pos eq -1 ) then begin
             j_pos=STRPOS(s_SPH_fields,'DS_NAME="SIR_GDR_2B                  "')
          endif


           if( j_pos eq -1 ) then begin
             DECODE_ERROR, error_number=28 ; Unable to find correct DSD name
             close, j_file_unit
             free_lun, j_file_unit
             retall
           endif
           ; position at start of DSD 
           POINT_LUN, j_file_unit, (SPH_START + j_pos)
           for j_lines_loop=0L, j_number_lines-1 do begin
	     READF, j_file_unit, s_field
	     s_DSD_fields[j_lines_loop] = s_field
           endfor
        end
        
    else : $   error condition
        begin
            DECODE_ERROR, error_number=14	; Invalid block name
	    CLOSE,j_file_unit
	    FREE_LUN,j_file_unit
	    RETALL
        endelse
    endcase
;print,s_DSD_fields

; -----------------------------------------------------------------------------
; extract info from s_DSD_fields

    if (s_DS_kind ne 'NIL') then $
    begin

      ; extract DS_OFFSET   s_DSD_fields[3] ---------------
      ; an example string is DS_OFFSET=+00000000000000008222<bytes>
      s_format='(A11,A20,A7)'
      READS,s_DSD_fields[3],s_1,s_2,s_3,FORMAT=s_format
      if(s_1 ne 'DS_OFFSET=+') then $
      begin		    
	DECODE_ERROR, error_number=28            ;Can not find DS_OFFSET field
	CLOSE,j_file_unit
	FREE_LUN,j_file_unit
	RETALL
      endif
      j_DS_start=long(s_2)
;print,s_DSD_fields[3]
;print,"The offset of DSD is....",j_DS_start

      ; extract NUM_DSR   s_DSD_fields[5] ---------------
      ; an example string is NUM_DSR=+0000001760
      s_format='(A9,A20)'
      READS,s_DSD_fields[5],s_1,s_2,FORMAT=s_format
      if(s_1 ne 'NUM_DSR=+') then $
      begin		    
	DECODE_ERROR, error_number=28            ;Can not find DS_OFFSET field
	CLOSE,j_file_unit
	FREE_LUN,j_file_unit
	RETALL
      endif
      j_num_DSR=long(s_2)
;print,s_DSD_fields[5]
;print,"The number of DSRs is....",j_num_DSR

      ; extract DSR_SIZE   s_DSD_fields[6] ---------------
      ; an example string is DSR_SIZE=+0000000606<bytes>
      s_format='(A10,A10,A7)'
      READS,s_DSD_fields[6],s_1,s_2,s_3,FORMAT=s_format
      if(s_1 ne 'DSR_SIZE=+') then $
      begin
	DECODE_ERROR, error_number=28            ;Can not find DS_OFFSET field
	CLOSE,j_file_unit
	FREE_LUN,j_file_unit
	RETALL
      endif
      j_DSR_size=long(s_2)
;print,s_DSD_fields[6]
;print,"The size of the DSR is....",j_DSR_size

    endif  ; end of if(s_DS_kind ne 'NIL')


; -- Close file ----------------------------------------------------------------
    CLOSE, j_file_unit
    FREE_LUN, j_file_unit

; -- Return SPH as a string to calling function -------------------------------
    RETURN, s_SPH_fields

end			; end of read_header



; b=read_SPH('l1b.head.ICET',6975L,s_DSD_fields,DS_TYPE='L1B')
